@@ -4,7 +4,6 @@
 // Author:      Chris Breeze
 // Modified by:
 // Created:     21/07/97
-// RCS-ID:      $Id$
 // Copyright:   (c) 1993-1998 Chris Breeze
 // Licence:     wxWindows licence
 //---------------------------------------------------------------------------
@@ -216,7 +215,7 @@ void Game::DoMove(wxDC& dc, Pile* src, Pile* dest)
 void Game::DisplayScore(wxDC& dc)
 {
     wxColour bgColour = FortyApp::BackgroundColour();
-    wxPen* pen = wxThePenList->FindOrCreatePen(bgColour, 1, wxSOLID);
+    wxPen* pen = wxThePenList->FindOrCreatePen(bgColour);
     dc.SetTextBackground(bgColour);
     dc.SetTextForeground(FortyApp::TextColour());
     dc.SetBrush(FortyApp::BackgroundBrush());
