@@ -4,7 +4,6 @@
 // Author:      Guillermo Rodriguez Garcia, <guille@iies.es>
 // Modified by:
 // Created:     Jan/2000
-// RCS-ID:      $Id$
 // Copyright:   (c) 2000, Guillermo Rodriguez Garcia
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -28,6 +27,7 @@
 #include "wx/wfstream.h"
 #include "wx/filedlg.h"
 #include "wx/stockitem.h"
+#include "wx/dcbuffer.h"
 
 #include "life.h"
 #include "game.h"
@@ -38,7 +38,7 @@
 // resources
 // --------------------------------------------------------------------------
 
-#if defined(__WXGTK__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__) || defined(__WXX11__)
+#if defined(__WXGTK__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__) || defined(__WXX11__) || defined(__WXQT__)
     // application icon
     #include "mondrian.xpm"
 
@@ -97,7 +97,7 @@ enum
 // --------------------------------------------------------------------------
 
 // Event tables
-BEGIN_EVENT_TABLE(LifeFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(LifeFrame, wxFrame)
     EVT_MENU            (wxID_NEW,     LifeFrame::OnMenu)
 #if wxUSE_FILEDLG
     EVT_MENU            (wxID_OPEN,    LifeFrame::OnOpen)
@@ -122,13 +122,13 @@ BEGIN_EVENT_TABLE(LifeFrame, wxFrame)
     EVT_COMMAND_SCROLL  (ID_SLIDER,    LifeFrame::OnSlider)
     EVT_TIMER           (ID_TIMER,     LifeFrame::OnTimer)
     EVT_CLOSE           (              LifeFrame::OnClose)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
-BEGIN_EVENT_TABLE(LifeNavigator, wxMiniFrame)
+wxBEGIN_EVENT_TABLE(LifeNavigator, wxMiniFrame)
     EVT_CLOSE           (             LifeNavigator::OnClose)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
-BEGIN_EVENT_TABLE(LifeCanvas, wxWindow)
+wxBEGIN_EVENT_TABLE(LifeCanvas, wxWindow)
     EVT_PAINT           (             LifeCanvas::OnPaint)
     EVT_SCROLLWIN       (             LifeCanvas::OnScroll)
     EVT_SIZE            (             LifeCanvas::OnSize)
@@ -137,11 +137,11 @@ BEGIN_EVENT_TABLE(LifeCanvas, wxWindow)
     EVT_LEFT_UP         (             LifeCanvas::OnMouse)
     EVT_LEFT_DCLICK     (             LifeCanvas::OnMouse)
     EVT_ERASE_BACKGROUND(             LifeCanvas::OnEraseBackground)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 
 // Create a new application object
-IMPLEMENT_APP(LifeApp)
+wxIMPLEMENT_APP(LifeApp);
 
 
 // ==========================================================================
@@ -163,9 +163,8 @@ bool LifeApp::OnInit()
     // create the main application window
     LifeFrame *frame = new LifeFrame();
 
-    // show it and tell the application that it's our main window
+    // show it
     frame->Show(true);
-    SetTopWindow(frame);
 
     // just for Motif
 #ifdef __WXMOTIF__
@@ -199,14 +198,12 @@ LifeFrame::LifeFrame() :
     menuFile->Append(wxID_OPEN, wxEmptyString, _("Open an existing Life pattern"));
 #endif
     menuFile->Append(ID_SAMPLES, _("&Sample game..."), _("Select a sample configuration"));
-#if ! (defined(__SMARTPHONE__) || defined(__POCKETPC__))
     menuFile->AppendSeparator();
     menuFile->Append(wxID_EXIT);
 
     menuView->Append(ID_SHOWNAV, _("Navigation &toolbox"), _("Show or hide toolbox"), wxITEM_CHECK);
     menuView->Check(ID_SHOWNAV, true);
     menuView->AppendSeparator();
-#endif
 
     menuView->Append(ID_ORIGIN, _("&Absolute origin"), _("Go to (0, 0)"));
     menuView->Append(ID_CENTER, _("&Center of mass"), _("Find center of mass"));
@@ -251,7 +248,6 @@ LifeFrame::LifeFrame() :
     toolBar->SetToolBitmapSize(wxSize(16, 16));
 
     ADD_TOOL(wxID_NEW, tbBitmaps[0], wxGetStockLabel(wxID_NEW, wxSTOCK_NOFLAGS), _("Start a new game"));
-#ifndef __POCKETPC__
 #if wxUSE_FILEDLG
     ADD_TOOL(wxID_OPEN, tbBitmaps[1], wxGetStockLabel(wxID_OPEN, wxSTOCK_NOFLAGS), _("Open an existing Life pattern"));
 #endif // wxUSE_FILEDLG
@@ -261,7 +257,6 @@ LifeFrame::LifeFrame() :
     ADD_TOOL(wxID_ZOOM_OUT, tbBitmaps[3], wxGetStockLabel(wxID_ZOOM_OUT, wxSTOCK_NOFLAGS), _("Zoom out"));
     ADD_TOOL(ID_INFO, tbBitmaps[4], _("Description"), _("Show description"));
     toolBar->AddSeparator();
-#endif // __POCKETPC__
     ADD_TOOL(ID_START, tbBitmaps[5], _("Start"), _("Start"));
     ADD_TOOL(wxID_STOP, tbBitmaps[6], _("Stop"), _("Stop"));
 
@@ -333,7 +328,6 @@ LifeFrame::LifeFrame() :
     sizer3->Add( panel2, 0, wxGROW );
     SetSizer( sizer3 );
 
-#ifndef __WXWINCE__
     sizer3->Fit( this );
 
     // set minimum frame size
@@ -341,7 +335,6 @@ LifeFrame::LifeFrame() :
 
     // navigator frame - not appropriate for small devices
     m_navigator = new LifeNavigator(this);
-#endif
 
 }
 
@@ -354,10 +347,10 @@ void LifeFrame::UpdateInfoText()
 {
     wxString msg;
 
-    msg.Printf(_(" Generation: %u (T: %u ms),  Population: %u "),
+    msg.Printf(_(" Generation: %lu (T: %lu ms),  Population: %lu "),
                m_tics,
                m_topspeed? 0 : m_interval,
-               m_life->GetNumCells());
+               static_cast<unsigned long>(m_life->GetNumCells()));
     m_text->SetLabel(msg);
 }
 
@@ -438,11 +431,20 @@ void LifeFrame::OnMenu(wxCommandEvent& event)
             m_running = true;
             m_topspeed = true;
             UpdateUI();
+            
+            const long YIELD_INTERVAL = 1000 / 30;
+            wxMilliClock_t lastyield = 0, now;
+            
             while (m_running && m_topspeed)
             {
                 OnStep();
-                wxYield();
+                if ( (now=wxGetLocalTimeMillis()) - lastyield > YIELD_INTERVAL)
+                {
+                    wxYield();
+                    lastyield = now;
+                }
             }
+ 
             break;
         }
     }
@@ -591,12 +593,13 @@ void LifeFrame::OnStop()
 void LifeFrame::OnStep()
 {
     if (m_life->NextTic())
+    {
         m_tics++;
+        m_canvas->Refresh();
+        UpdateInfoText();
+    }
     else
         OnStop();
-
-    m_canvas->DrawChanged();
-    UpdateInfoText();
 }
 
 
@@ -694,11 +697,7 @@ void LifeNavigator::OnClose(wxCloseEvent& event)
 LifeCanvas::LifeCanvas(wxWindow *parent, Life *life, bool interactive)
           : wxWindow(parent, wxID_ANY, wxDefaultPosition, wxSize(100, 100),
             wxFULL_REPAINT_ON_RESIZE | wxHSCROLL | wxVSCROLL
-#if !defined(__SMARTPHONE__) && !defined(__POCKETPC__)
             |wxSUNKEN_BORDER
-#else
-            |wxSIMPLE_BORDER
-#endif
             )
 {
     m_life        = life;
@@ -715,6 +714,7 @@ LifeCanvas::LifeCanvas(wxWindow *parent, Life *life, bool interactive)
 
     // reduce flicker if wxEVT_ERASE_BACKGROUND is not available
     SetBackgroundColour(*wxWHITE);
+    SetBackgroundStyle(wxBG_STYLE_PAINT);
 }
 
 LifeCanvas::~LifeCanvas()
@@ -793,36 +793,36 @@ void LifeCanvas::DrawCell(wxInt32 i, wxInt32 j, wxDC &dc)
     }
 }
 
-// draw all changed cells
+// draw all changed cells, currently not in use
 void LifeCanvas::DrawChanged()
 {
     wxClientDC dc(this);
-
+    
     size_t ncells;
     LifeCell *cells;
     bool done = false;
-
+    
     m_life->BeginFind(m_viewportX,
                       m_viewportY,
                       m_viewportX + m_viewportW,
                       m_viewportY + m_viewportH,
                       true);
-
+    
     if (m_cellsize == 1)
     {
-        dc.SetPen(*wxBLACK_PEN);
+        dc.SetPen(*wxWHITE_PEN);
     }
     else
     {
         dc.SetPen(*wxTRANSPARENT_PEN);
-        dc.SetBrush(*wxBLACK_BRUSH);
+        dc.SetBrush(*wxWHITE_BRUSH);
     }
-    dc.SetLogicalFunction(wxINVERT);
-
+    dc.SetLogicalFunction(wxXOR);
+    
     while (!done)
     {
         done = m_life->FindMore(&cells, &ncells);
-
+        
         for (size_t m = 0; m < ncells; m++)
             DrawCell(cells[m].i, cells[m].j, dc);
     }
@@ -831,7 +831,7 @@ void LifeCanvas::DrawChanged()
 // event handlers
 void LifeCanvas::OnPaint(wxPaintEvent& WXUNUSED(event))
 {
-    wxPaintDC dc(this);
+    wxAutoBufferedPaintDC dc(this);
     wxRect  rect = GetUpdateRegion().GetBox();
     wxCoord x, y, w, h;
     wxInt32 i0, j0, i1, j1;
