@@ -65,10 +65,8 @@
         #define wxCOMPILER_PREFIX vc120
     #elif _MSC_VER == 1900
         #define wxCOMPILER_PREFIX vc140
-    #elif _MSC_VER >= 1910 && _MSC_VER < 1920
+    #elif _MSC_VER >= 1910 && _MSC_VER < 2000
         #define wxCOMPILER_PREFIX vc141
-    #elif _MSC_VER >= 1920 && _MSC_VER < 1930
-        #define wxCOMPILER_PREFIX vc142
     #else
         #error "Unknown MSVC compiler version, please report to wx-dev."
     #endif
@@ -161,15 +159,18 @@
 #endif
 #if wxUSE_XML && !defined(wxNO_XML_LIB)
     #pragma comment(lib, wxBASE_LIB_NAME("xml"))
+    #if !defined(wxNO_EXPAT_LIB) && !defined(WXUSINGDLL)
+        #pragma comment(lib, wx3RD_PARTY_LIB_NAME("expat"))
+    #endif
 #endif
 #if wxUSE_REGEX && !defined(wxNO_REGEX_LIB) && !defined(WXUSINGDLL)
     #pragma comment(lib, wx3RD_PARTY_LIB_NAME_U("regex"))
 #endif
+#if wxUSE_ZLIB && !defined(wxNO_ZLIB_LIB) && !defined(WXUSINGDLL)
+    #pragma comment(lib, wx3RD_PARTY_LIB_NAME("zlib"))
+#endif
 
 #if wxUSE_GUI
-    #if wxUSE_XML && !defined(wxNO_EXPAT_LIB) && !defined(WXUSINGDLL)
-        #pragma comment(lib, wx3RD_PARTY_LIB_NAME("expat"))
-    #endif
     #if wxUSE_LIBJPEG && !defined(wxNO_JPEG_LIB) && !defined(WXUSINGDLL)
         #pragma comment(lib, wx3RD_PARTY_LIB_NAME("jpeg"))
     #endif
@@ -179,9 +180,6 @@
     #if wxUSE_LIBTIFF && !defined(wxNO_TIFF_LIB) && !defined(WXUSINGDLL)
         #pragma comment(lib, wx3RD_PARTY_LIB_NAME("tiff"))
     #endif
-    #if wxUSE_ZLIB && !defined(wxNO_ZLIB_LIB) && !defined(WXUSINGDLL)
-        #pragma comment(lib, wx3RD_PARTY_LIB_NAME("zlib"))
-    #endif
 
     #pragma comment(lib, wxTOOLKIT_LIB_NAME("core"))
 
@@ -249,8 +247,13 @@
     #endif
 
     #ifdef __WXGTK__
-        #pragma comment(lib, "gtk-win32-2.0.lib")
-        #pragma comment(lib, "gdk-win32-2.0.lib")
+        #ifdef __WXGTK3__
+            #pragma comment(lib, "libgtk-3.dll.a")
+            #pragma comment(lib, "libgdk-3.dll.a")
+        #else
+            #pragma comment(lib, "gtk-win32-2.0.lib")
+            #pragma comment(lib, "gdk-win32-2.0.lib")
+        #endif
         #pragma comment(lib, "pangocairo-1.0.lib")
         #pragma comment(lib, "gdk_pixbuf-2.0.lib")
         #pragma comment(lib, "cairo.lib")
