@@ -4,7 +4,6 @@
  * Author:      Vadim Zeitlin
  * Modified by:
  * Created:     16.10.2003 (extracted from wx/defs.h)
- * RCS-ID:      $Id$
  * Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwidgets.org>
  * Licence:     wxWindows licence
  */
@@ -21,35 +20,28 @@
 #    define WXIMPORT __attribute__ ((visibility("default")))
 #elif defined(__WINDOWS__)
     /*
-       __declspec works in BC++ 5 and later, Watcom C++ 11.0 and later as well
-       as VC++ and gcc
+       __declspec works in BC++ 5 and later as well as VC++.
      */
-#    if defined(__VISUALC__) || defined(__BORLANDC__) || defined(__GNUC__) || defined(__WATCOMC__)
+#    if defined(__VISUALC__) || defined(__BORLANDC__)
 #        define WXEXPORT __declspec(dllexport)
 #        define WXIMPORT __declspec(dllimport)
-#    else /* compiler doesn't support __declspec() */
-#        define WXEXPORT
-#        define WXIMPORT
-#    endif
-#elif defined(__WXPM__)
-#    if defined (__WATCOMC__)
-#        define WXEXPORT __declspec(dllexport)
+    /*
+        While gcc also supports __declspec(dllexport), it created unusably huge
+        DLL files in gcc 4.[56] (while taking horribly long amounts of time),
+        see http://gcc.gnu.org/bugzilla/show_bug.cgi?id=43601. Because of this
+        we rely on binutils auto export/import support which seems to work
+        quite well for 4.5+. However the problem was fixed in 4.7 and later and
+        not exporting everything creates smaller DLLs (~8% size difference), so
+        do use the explicit attributes again for the newer versions.
+     */
+#    elif defined(__GNUC__) && \
+        (!wxCHECK_GCC_VERSION(4, 5) || wxCHECK_GCC_VERSION(4, 7))
         /*
-           __declspec(dllimport) prepends __imp to imported symbols. We do NOT
-           want that!
-         */
-#        define WXIMPORT
-#    elif defined(__EMX__)
-#        define WXEXPORT
-#        define WXIMPORT
-#    elif (!(defined(__VISAGECPP__) && (__IBMCPP__ < 400 || __IBMC__ < 400 )))
-#        define WXEXPORT _Export
-#        define WXIMPORT _Export
-#    endif
-#elif defined(__WXMAC__) || defined(__WXCOCOA__)
-#    ifdef __MWERKS__
-#        define WXEXPORT __declspec(export)
-#        define WXIMPORT __declspec(import)
+            __declspec could be used here too but let's use the native
+            __attribute__ instead for clarity.
+        */
+#       define WXEXPORT __attribute__((dllexport))
+#       define WXIMPORT __attribute__((dllimport))
 #    endif
 #elif defined(__CYGWIN__)
 #    define WXEXPORT __declspec(dllexport)
@@ -86,11 +78,12 @@
 #    define WXMAKINGDLL_XML
 #    define WXMAKINGDLL_XRC
 #    define WXMAKINGDLL_AUI
-#    define WXMAKINGDLL_RIBBON
 #    define WXMAKINGDLL_PROPGRID
+#    define WXMAKINGDLL_RIBBON
 #    define WXMAKINGDLL_RICHTEXT
 #    define WXMAKINGDLL_MEDIA
 #    define WXMAKINGDLL_STC
+#    define WXMAKINGDLL_WEBVIEW
 #endif /* WXMAKINGDLL */
 
 /*
@@ -217,14 +210,6 @@
 #    define WXDLLIMPEXP_AUI
 #endif
 
-#ifdef WXMAKINGDLL_RIBBON
-#    define WXDLLIMPEXP_RIBBON WXEXPORT
-#elif defined(WXUSINGDLL)
-#    define WXDLLIMPEXP_RIBBON WXIMPORT
-#else /* not making nor using DLL */
-#    define WXDLLIMPEXP_RIBBON
-#endif
-
 #ifdef WXMAKINGDLL_PROPGRID
 #    define WXDLLIMPEXP_PROPGRID WXEXPORT
 #    define WXDLLIMPEXP_DATA_PROPGRID(type) WXEXPORT type
@@ -236,6 +221,14 @@
 #    define WXDLLIMPEXP_DATA_PROPGRID(type) type
 #endif
 
+#ifdef WXMAKINGDLL_RIBBON
+#    define WXDLLIMPEXP_RIBBON WXEXPORT
+#elif defined(WXUSINGDLL)
+#    define WXDLLIMPEXP_RIBBON WXIMPORT
+#else /* not making nor using DLL */
+#    define WXDLLIMPEXP_RIBBON
+#endif
+
 #ifdef WXMAKINGDLL_RICHTEXT
 #    define WXDLLIMPEXP_RICHTEXT WXEXPORT
 #elif defined(WXUSINGDLL)
@@ -263,6 +256,17 @@
 #    define WXDLLIMPEXP_DATA_STC(type) type
 #endif
 
+#ifdef WXMAKINGDLL_WEBVIEW
+#    define WXDLLIMPEXP_WEBVIEW WXEXPORT
+#    define WXDLLIMPEXP_DATA_WEBVIEW(type) WXEXPORT type
+#elif defined(WXUSINGDLL)
+#    define WXDLLIMPEXP_WEBVIEW WXIMPORT
+#    define WXDLLIMPEXP_DATA_WEBVIEW(type) WXIMPORT type
+#else /* not making nor using DLL */
+#    define WXDLLIMPEXP_WEBVIEW
+#    define WXDLLIMPEXP_DATA_WEBVIEW(type) type
+#endif
+
 /*
    GCC warns about using __attribute__ (and also __declspec in mingw32 case) on
    forward declarations while MSVC complains about forward declarations without
@@ -281,9 +285,11 @@
     #define WXDLLIMPEXP_FWD_XRC
     #define WXDLLIMPEXP_FWD_AUI
     #define WXDLLIMPEXP_FWD_PROPGRID
+    #define WXDLLIMPEXP_FWD_RIBBON
     #define WXDLLIMPEXP_FWD_RICHTEXT
     #define WXDLLIMPEXP_FWD_MEDIA
     #define WXDLLIMPEXP_FWD_STC
+    #define WXDLLIMPEXP_FWD_WEBVIEW
 #else
     #define WXDLLIMPEXP_FWD_BASE      WXDLLIMPEXP_BASE
     #define WXDLLIMPEXP_FWD_NET       WXDLLIMPEXP_NET
@@ -296,48 +302,15 @@
     #define WXDLLIMPEXP_FWD_XRC       WXDLLIMPEXP_XRC
     #define WXDLLIMPEXP_FWD_AUI       WXDLLIMPEXP_AUI
     #define WXDLLIMPEXP_FWD_PROPGRID  WXDLLIMPEXP_PROPGRID
+    #define WXDLLIMPEXP_FWD_RIBBON    WXDLLIMPEXP_RIBBON
     #define WXDLLIMPEXP_FWD_RICHTEXT  WXDLLIMPEXP_RICHTEXT
     #define WXDLLIMPEXP_FWD_MEDIA     WXDLLIMPEXP_MEDIA
     #define WXDLLIMPEXP_FWD_STC       WXDLLIMPEXP_STC
+    #define WXDLLIMPEXP_FWD_WEBVIEW   WXDLLIMPEXP_WEBVIEW
 #endif
 
 /* for backwards compatibility, define suffix-less versions too */
 #define WXDLLEXPORT WXDLLIMPEXP_CORE
 #define WXDLLEXPORT_DATA WXDLLIMPEXP_DATA_CORE
 
-/*
-   MSVC up to 6.0 needs to be explicitly told to export template instantiations
-   used by the DLL clients, use this macro to do it like this:
-
-       template <typename T> class Foo { ... };
-       WXDLLIMPEXP_TEMPLATE_INSTANCE_BASE( Foo<int> )
-
-   (notice that currently we only need this for wxBase and wxCore libraries)
- */
-#if defined(__VISUALC__) && (__VISUALC__ <= 1200)
-    #ifdef WXMAKINGDLL_BASE
-        #define WXDLLIMPEXP_TEMPLATE_INSTANCE_BASE(decl) \
-            template class WXDLLIMPEXP_BASE decl;
-        #define WXDLLIMPEXP_TEMPLATE_INSTANCE_CORE(decl) \
-            template class WXDLLIMPEXP_CORE decl;
-    #else
-        /*
-           We need to disable this warning when using this macro, as
-           recommended by Microsoft itself:
-
-           http://support.microsoft.com/default.aspx?scid=kb%3ben-us%3b168958
-         */
-        #pragma warning(disable:4231)
-
-        #define WXDLLIMPEXP_TEMPLATE_INSTANCE_BASE(decl) \
-            extern template class WXDLLIMPEXP_BASE decl;
-        #define WXDLLIMPEXP_TEMPLATE_INSTANCE_CORE(decl) \
-            extern template class WXDLLIMPEXP_CORE decl;
-    #endif
-#else /* not VC <= 6 */
-    #define WXDLLIMPEXP_TEMPLATE_INSTANCE_BASE(decl)
-    #define WXDLLIMPEXP_TEMPLATE_INSTANCE_CORE(decl)
-#endif /* VC6/others */
-
 #endif /* _WX_DLIMPEXP_H_ */
-
