@@ -32,12 +32,15 @@ OSStatus WXDLLIMPEXP_CORE wxMacDrawCGImage(
                                CGContextRef    inContext,
                                const CGRect *  inBounds,
                                CGImageRef      inImage) ;
-WX_NSImage WXDLLIMPEXP_CORE wxOSXGetNSImageFromCGImage( CGImageRef image, double scale = 1.0 );
+WX_NSImage WXDLLIMPEXP_CORE wxOSXGetNSImageFromCGImage( CGImageRef image, double scale = 1.0, bool isTemplate = false);
+WX_NSImage WXDLLIMPEXP_CORE wxOSXGetNSImageFromIconRef( WXHICON iconref );
 CGImageRef WXDLLIMPEXP_CORE wxOSXCreateCGImageFromNSImage( WX_NSImage nsimage, double *scale = NULL );
-CGContextRef WXDLLIMPEXP_CORE wxOSXCreateBitmapContextFromNSImage( WX_NSImage nsimage);
+CGImageRef WXDLLIMPEXP_CORE wxOSXGetCGImageFromNSImage( WX_NSImage nsimage, CGRect* r, CGContextRef cg);
+CGContextRef WXDLLIMPEXP_CORE wxOSXCreateBitmapContextFromNSImage( WX_NSImage nsimage, bool *isTemplate = NULL);
 
 wxBitmap WXDLLIMPEXP_CORE wxOSXCreateSystemBitmap(const wxString& id, const wxString &client, const wxSize& size);
 WXWindow WXDLLIMPEXP_CORE wxOSXGetMainWindow();
+WXWindow WXDLLIMPEXP_CORE wxOSXGetKeyWindow();
 
 class WXDLLIMPEXP_FWD_CORE wxDialog;
 
@@ -101,9 +104,9 @@ public :
     void                SetCursor( const wxCursor & cursor );
     void                CaptureMouse();
     void                ReleaseMouse();
-
+#if wxUSE_DRAG_AND_DROP
     void                SetDropTarget(wxDropTarget* target);
-    
+#endif
     wxInt32             GetValue() const;
     void                SetValue( wxInt32 v );
     wxBitmap            GetBitmap() const;
@@ -125,10 +128,15 @@ public :
     void                SetToolTip( wxToolTip* tooltip );
 
     void                InstallEventHandler( WXWidget control = NULL );
+    bool                EnableTouchEvents(int eventsMask);
 
+    virtual bool        ShouldHandleKeyNavigation(const wxKeyEvent &event) const;
+    bool                DoHandleKeyNavigation(const wxKeyEvent &event);
     virtual bool        DoHandleMouseEvent(NSEvent *event);
     virtual bool        DoHandleKeyEvent(NSEvent *event);
     virtual bool        DoHandleCharEvent(NSEvent *event, NSString *text);
+    virtual void        DoNotifyFocusSet();
+    virtual void        DoNotifyFocusLost();
     virtual void        DoNotifyFocusEvent(bool receivedFocus, wxWidgetImpl* otherWindow);
 
     virtual void        SetupKeyEvent(wxKeyEvent &wxevent, NSEvent * nsEvent, NSString* charString = NULL);
@@ -136,6 +144,15 @@ public :
     void                SetupCoordinates(wxCoord &x, wxCoord &y, NSEvent *nsEvent);
     virtual bool        SetupCursor(NSEvent* event);
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_10
+    virtual void        PanGestureEvent(NSPanGestureRecognizer *panGestureRecognizer);
+    virtual void        ZoomGestureEvent(NSMagnificationGestureRecognizer *magnificationGestureRecognizer);
+    virtual void        RotateGestureEvent(NSRotationGestureRecognizer *rotationGestureRecognizer);
+    virtual void        LongPressEvent(NSPressGestureRecognizer *pressGestureRecognizer);
+    virtual void        TouchesBegan(NSEvent *event);
+    virtual void        TouchesMoved(NSEvent *event);
+    virtual void        TouchesEnded(NSEvent *event);
+#endif // MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_10
 
 #if !wxOSX_USE_NATIVE_FLIPPED
     void                SetFlipped(bool flipped);
@@ -146,10 +163,12 @@ public :
     
     // cocoa thunk connected calls
 
+#if wxUSE_DRAG_AND_DROP
     virtual unsigned int        draggingEntered(void* sender, WXWidget slf, void* _cmd);
     virtual void                draggingExited(void* sender, WXWidget slf, void* _cmd);
     virtual unsigned int        draggingUpdated(void* sender, WXWidget slf, void* _cmd);
     virtual bool                performDragOperation(void* sender, WXWidget slf, void* _cmd);
+#endif
     virtual void                mouseEvent(WX_NSEvent event, WXWidget slf, void* _cmd);
     virtual void                cursorUpdate(WX_NSEvent event, WXWidget slf, void* _cmd);
     virtual void                keyEvent(WX_NSEvent event, WXWidget slf, void* _cmd);
@@ -181,7 +200,7 @@ protected:
     // events, don't resend them
     bool m_hasEditor;
 
-    DECLARE_DYNAMIC_CLASS_NO_COPY(wxWidgetCocoaImpl)
+    wxDECLARE_DYNAMIC_CLASS_NO_COPY(wxWidgetCocoaImpl);
 };
 
 DECLARE_WXCOCOA_OBJC_CLASS( wxNSWindow );
@@ -194,75 +213,86 @@ public :
 
     virtual ~wxNonOwnedWindowCocoaImpl();
 
-    virtual void WillBeDestroyed() ;
+    virtual void WillBeDestroyed() wxOVERRIDE;
     void Create( wxWindow* parent, const wxPoint& pos, const wxSize& size,
-    long style, long extraStyle, const wxString& name ) ;
+    long style, long extraStyle, const wxString& name ) wxOVERRIDE;
     void Create( wxWindow* parent, WXWindow nativeWindow );
 
-    WXWindow GetWXWindow() const;
-    void Raise();
-    void Lower();
-    bool Show(bool show);
+    WXWindow GetWXWindow() const wxOVERRIDE;
+    void Raise() wxOVERRIDE;
+    void Lower() wxOVERRIDE;
+    bool Show(bool show) wxOVERRIDE;
 
     virtual bool ShowWithEffect(bool show,
                                 wxShowEffect effect,
-                                unsigned timeout);
+                                unsigned timeout) wxOVERRIDE;
 
-    void Update();
-    bool SetTransparent(wxByte alpha);
-    bool SetBackgroundColour(const wxColour& col );
-    void SetExtraStyle( long exStyle );
-    void SetWindowStyleFlag( long style );
-    bool SetBackgroundStyle(wxBackgroundStyle style);
-    bool CanSetTransparent();
+    void Update() wxOVERRIDE;
+    bool SetTransparent(wxByte alpha) wxOVERRIDE;
+    bool SetBackgroundColour(const wxColour& col ) wxOVERRIDE;
+    void SetExtraStyle( long exStyle ) wxOVERRIDE;
+    void SetWindowStyleFlag( long style ) wxOVERRIDE;
+    bool SetBackgroundStyle(wxBackgroundStyle style) wxOVERRIDE;
+    bool CanSetTransparent() wxOVERRIDE;
 
-    void MoveWindow(int x, int y, int width, int height);
-    void GetPosition( int &x, int &y ) const;
-    void GetSize( int &width, int &height ) const;
+    void MoveWindow(int x, int y, int width, int height) wxOVERRIDE;
+    void GetPosition( int &x, int &y ) const wxOVERRIDE;
+    void GetSize( int &width, int &height ) const wxOVERRIDE;
 
-    void GetContentArea( int &left , int &top , int &width , int &height ) const;
-    bool SetShape(const wxRegion& region);
+    void GetContentArea( int &left , int &top , int &width , int &height ) const wxOVERRIDE;
+    bool SetShape(const wxRegion& region) wxOVERRIDE;
 
-    virtual void SetTitle( const wxString& title, wxFontEncoding encoding ) ;
+    virtual void SetTitle( const wxString& title, wxFontEncoding encoding ) wxOVERRIDE;
 
-    virtual bool IsMaximized() const;
+    virtual bool EnableCloseButton(bool enable) wxOVERRIDE;
+    virtual bool EnableMaximizeButton(bool enable) wxOVERRIDE;
+    virtual bool EnableMinimizeButton(bool enable) wxOVERRIDE;
 
-    virtual bool IsIconized() const;
+    virtual bool IsMaximized() const wxOVERRIDE;
 
-    virtual void Iconize( bool iconize );
+    virtual bool IsIconized() const wxOVERRIDE;
 
-    virtual void Maximize(bool maximize);
+    virtual void Iconize( bool iconize ) wxOVERRIDE;
 
-    virtual bool IsFullScreen() const;
+    virtual void Maximize(bool maximize) wxOVERRIDE;
 
-    virtual bool ShowFullScreen(bool show, long style);
+    virtual bool IsFullScreen() const wxOVERRIDE;
 
-    virtual void ShowWithoutActivating();
+    bool EnableFullScreenView(bool enable) wxOVERRIDE;
 
-    virtual void RequestUserAttention(int flags);
+    virtual bool ShowFullScreen(bool show, long style) wxOVERRIDE;
 
-    virtual void ScreenToWindow( int *x, int *y );
+    virtual void ShowWithoutActivating() wxOVERRIDE;
 
-    virtual void WindowToScreen( int *x, int *y );
+    virtual void RequestUserAttention(int flags) wxOVERRIDE;
 
-    virtual bool IsActive();
+    virtual void ScreenToWindow( int *x, int *y ) wxOVERRIDE;
 
-    virtual void SetModified(bool modified);
-    virtual bool IsModified() const;
+    virtual void WindowToScreen( int *x, int *y ) wxOVERRIDE;
 
-    virtual void SetRepresentedFilename(const wxString& filename);
+    virtual bool IsActive() wxOVERRIDE;
+
+    virtual void SetModified(bool modified) wxOVERRIDE;
+    virtual bool IsModified() const wxOVERRIDE;
+
+    virtual void SetRepresentedFilename(const wxString& filename) wxOVERRIDE;
 
     wxNonOwnedWindow*   GetWXPeer() { return m_wxPeer; }
     
-    CGWindowLevel   GetWindowLevel() const { return m_macWindowLevel; }
-    void            RestoreWindowLevel();
+    CGWindowLevel   GetWindowLevel() const wxOVERRIDE { return m_macWindowLevel; }
+    void            RestoreWindowLevel() wxOVERRIDE;
     
     static WX_NSResponder GetNextFirstResponder() ;
+    static WX_NSResponder GetFormerFirstResponder() ;
 protected :
     CGWindowLevel   m_macWindowLevel;
     WXWindow        m_macWindow;
     void *          m_macFullScreenData ;
-    DECLARE_DYNAMIC_CLASS_NO_COPY(wxNonOwnedWindowCocoaImpl)
+    
+private:
+    void SetUpForModalParent();
+    
+    wxDECLARE_DYNAMIC_CLASS_NO_COPY(wxNonOwnedWindowCocoaImpl);
 };
 
 DECLARE_WXCOCOA_OBJC_CLASS( wxNSButton );
@@ -284,6 +314,8 @@ public:
 };
 
 #ifdef __OBJC__
+
+    typedef NSRect WXRect;
     typedef void (*wxOSX_TextEventHandlerPtr)(NSView* self, SEL _cmd, NSString *event);
     typedef void (*wxOSX_EventHandlerPtr)(NSView* self, SEL _cmd, NSEvent *event);
     typedef BOOL (*wxOSX_PerformKeyEventHandlerPtr)(NSView* self, SEL _cmd, NSEvent *event);
@@ -319,11 +351,13 @@ public:
     @interface wxNSTextFieldEditor : NSTextView
     {
         NSEvent* lastKeyDownEvent;
+        NSTextField* textField;
     }
 
+    - (void) setTextField:(NSTextField*) field;
     @end
 
-    @interface wxNSTextField : NSTextField wxOSX_10_6_AND_LATER(<NSTextFieldDelegate>)
+    @interface wxNSTextField : NSTextField <NSTextFieldDelegate>
     {
         wxNSTextFieldEditor* fieldEditor;
     }
@@ -333,14 +367,14 @@ public:
 
     @end
 
-    @interface wxNSSecureTextField : NSSecureTextField wxOSX_10_6_AND_LATER(<NSTextFieldDelegate>)
+    @interface wxNSSecureTextField : NSSecureTextField <NSTextFieldDelegate>
     {
     }
 
     @end
 
 
-    @interface wxNSTextView : NSTextView wxOSX_10_6_AND_LATER(<NSTextViewDelegate>)
+    @interface wxNSTextView : NSTextView <NSTextViewDelegate>
     {
     }
 
@@ -419,7 +453,7 @@ public:
 #ifdef __LP64__
     WXEXPORT
 #endif // 64 bit builds
-    @interface wxNSAppController : NSObject wxOSX_10_6_AND_LATER(<NSApplicationDelegate>)
+    @interface wxNSAppController : NSObject <NSApplicationDelegate>
     {
     }
 
@@ -466,7 +500,7 @@ extern ClassicCursor gMacCursors[];
 
 extern NSLayoutManager* gNSLayoutManager;
 
-#endif
+#endif // wxUSE_GUI
 
 #endif
     // _WX_PRIVATE_COCOA_H_
