@@ -1,8 +1,7 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        zipstrm.h
+// Name:        wx/zipstrm.h
 // Purpose:     Streams for Zip files
 // Author:      Mike Wetherell
-// RCS-ID:      $Id: zipstrm.h,v 1.19 2005/04/11 23:26:38 MW Exp $
 // Copyright:   (c) Mike Wetherell
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -10,16 +9,11 @@
 #ifndef _WX_WXZIPSTREAM_H__
 #define _WX_WXZIPSTREAM_H__
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma interface "zipstrm.h"
-#endif
-
 #include "wx/defs.h"
 
-#if wxUSE_ZLIB && wxUSE_STREAMS && wxUSE_ZIPSTREAM
+#if wxUSE_ZIPSTREAM
 
 #include "wx/archive.h"
-#include "wx/hashmap.h"
 #include "wx/filename.h"
 
 // some methods from wxZipInputStream and wxZipOutputStream stream do not get
@@ -30,7 +24,7 @@
        || !defined __GNUC_MINOR__ \
        || !defined __GNUC_PATCHLEVEL__ \
        || __GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__ < 30402)
-#define WXZIPFIX WXDLLIMPEXP_BASE 
+#define WXZIPFIX WXDLLIMPEXP_BASE
 #else
 #define WXZIPFIX
 #endif
@@ -57,7 +51,7 @@ enum wxZipMethod
 };
 
 // Originating File-System.
-// 
+//
 // These are Pkware's values. Note that Info-zip disagree on some of them,
 // most notably NTFS.
 //
@@ -111,14 +105,23 @@ enum wxZipFlags
     wxZIP_ENHANCED          = 0x0010,
     wxZIP_PATCH             = 0x0020,
     wxZIP_STRONG_ENC        = 0x0040,
+    wxZIP_LANG_ENC_UTF8     = 0x0800,   // filename and comment are UTF8
     wxZIP_UNUSED            = 0x0F80,
     wxZIP_RESERVED          = 0xF000
 };
 
+enum wxZipArchiveFormat
+{
+    /// Default zip format
+    wxZIP_FORMAT_DEFAULT,
+    /// ZIP64 format
+    wxZIP_FORMAT_ZIP64
+};
+
 // Forward decls
 //
-class WXDLLIMPEXP_BASE wxZipEntry;
-class WXDLLIMPEXP_BASE wxZipInputStream;
+class WXDLLIMPEXP_FWD_BASE wxZipEntry;
+class WXDLLIMPEXP_FWD_BASE wxZipInputStream;
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -136,6 +139,8 @@ public:
 /////////////////////////////////////////////////////////////////////////////
 // Zip Entry - holds the meta data for a file in the zip
 
+class wxDataOutputStream;
+
 class WXDLLIMPEXP_BASE wxZipEntry : public wxArchiveEntry
 {
 public:
@@ -148,10 +153,10 @@ public:
     wxZipEntry& operator=(const wxZipEntry& entry);
 
     // Get accessors
-    wxDateTime   GetDateTime() const            { return m_DateTime; }
-    wxFileOffset GetSize() const                { return m_Size; }
-    wxFileOffset GetOffset() const              { return m_Offset; }
-    wxString     GetInternalName() const        { return m_Name; }
+    wxDateTime   GetDateTime() const wxOVERRIDE            { return m_DateTime; }
+    wxFileOffset GetSize() const wxOVERRIDE                { return m_Size; }
+    wxFileOffset GetOffset() const wxOVERRIDE              { return m_Offset; }
+    wxString     GetInternalName() const wxOVERRIDE        { return m_Name; }
     int          GetMethod() const              { return m_Method; }
     int          GetFlags() const               { return m_Flags; }
     wxUint32     GetCrc() const                 { return m_Crc; }
@@ -159,23 +164,23 @@ public:
     int          GetSystemMadeBy() const        { return m_SystemMadeBy; }
     wxString     GetComment() const             { return m_Comment; }
     wxUint32     GetExternalAttributes() const  { return m_ExternalAttributes; }
-    wxPathFormat GetInternalFormat() const      { return wxPATH_UNIX; }
+    wxPathFormat GetInternalFormat() const wxOVERRIDE      { return wxPATH_UNIX; }
     int          GetMode() const;
     const char  *GetLocalExtra() const;
     size_t       GetLocalExtraLen() const;
     const char  *GetExtra() const;
     size_t       GetExtraLen() const;
-    wxString     GetName(wxPathFormat format = wxPATH_NATIVE) const;
+    wxString     GetName(wxPathFormat format = wxPATH_NATIVE) const wxOVERRIDE;
 
     // is accessors
-    inline bool IsDir() const;
+    inline bool IsDir() const wxOVERRIDE;
     inline bool IsText() const;
-    inline bool IsReadOnly() const;
+    inline bool IsReadOnly() const wxOVERRIDE;
     inline bool IsMadeByUnix() const;
 
     // set accessors
-    void SetDateTime(const wxDateTime& dt)      { m_DateTime = dt; }
-    void SetSize(wxFileOffset size)             { m_Size = size; }
+    void SetDateTime(const wxDateTime& dt) wxOVERRIDE      { m_DateTime = dt; }
+    void SetSize(wxFileOffset size) wxOVERRIDE             { m_Size = size; }
     void SetMethod(int method)                  { m_Method = (wxUint16)method; }
     void SetComment(const wxString& comment)    { m_Comment = comment; }
     void SetExternalAttributes(wxUint32 attr )  { m_ExternalAttributes = attr; }
@@ -185,21 +190,21 @@ public:
     void SetLocalExtra(const char *extra, size_t len);
 
     inline void SetName(const wxString& name,
-                        wxPathFormat format = wxPATH_NATIVE);
+                        wxPathFormat format = wxPATH_NATIVE) wxOVERRIDE;
 
     static wxString GetInternalName(const wxString& name,
                                     wxPathFormat format = wxPATH_NATIVE,
                                     bool *pIsDir = NULL);
 
     // set is accessors
-    void SetIsDir(bool isDir = true);
-    inline void SetIsReadOnly(bool isReadOnly = true);
+    void SetIsDir(bool isDir = true) wxOVERRIDE;
+    inline void SetIsReadOnly(bool isReadOnly = true) wxOVERRIDE;
     inline void SetIsText(bool isText = true);
 
     wxZipEntry *Clone() const                   { return ZipClone(); }
 
     void SetNotifier(wxZipNotifier& notifier);
-    void UnsetNotifier();
+    void UnsetNotifier() wxOVERRIDE;
 
 protected:
     // Internal attributes
@@ -213,7 +218,7 @@ protected:
     int GetInternalAttributes() const           { return m_InternalAttributes; }
 
     void SetVersionNeeded(int version)          { m_VersionNeeded = (wxUint16)version; }
-    void SetOffset(wxFileOffset offset)         { m_Offset = offset; }
+    void SetOffset(wxFileOffset offset) wxOVERRIDE         { m_Offset = offset; }
     void SetFlags(int flags)                    { m_Flags = (wxUint16)flags; }
     void SetVersionMadeBy(int version)          { m_VersionMadeBy = (wxUint8)version; }
     void SetCrc(wxUint32 crc)                   { m_Crc = crc; }
@@ -227,10 +232,10 @@ protected:
     void Notify();
 
 private:
-    wxArchiveEntry* DoClone() const             { return ZipClone(); }
+    wxArchiveEntry* DoClone() const wxOVERRIDE             { return ZipClone(); }
 
     size_t ReadLocal(wxInputStream& stream, wxMBConv& conv);
-    size_t WriteLocal(wxOutputStream& stream, wxMBConv& conv) const;
+    size_t WriteLocal(wxOutputStream& stream, wxMBConv& conv, wxZipArchiveFormat zipFormat);
 
     size_t ReadCentral(wxInputStream& stream, wxMBConv& conv);
     size_t WriteCentral(wxOutputStream& stream, wxMBConv& conv) const;
@@ -239,6 +244,13 @@ private:
     size_t WriteDescriptor(wxOutputStream& stream, wxUint32 crc,
                            wxFileOffset compressedSize, wxFileOffset size);
 
+    void WriteLocalFileSizes(wxDataOutputStream& ds) const;
+    void WriteLocalZip64ExtraInfo(wxOutputStream& stream) const;
+
+    bool LoadExtraInfo(const char* extraData, wxUint16 extraLen, bool localInfo);
+
+    wxUint16 GetInternalFlags(bool checkForUTF8) const;
+
     wxUint8      m_SystemMadeBy;       // one of enum wxZipSystem
     wxUint8      m_VersionMadeBy;      // major * 10 + minor
 
@@ -257,6 +269,7 @@ private:
     wxUint16     m_DiskStart;          // for multidisk archives, not unsupported
     wxUint16     m_InternalAttributes; // bit 0 set for text files
     wxUint32     m_ExternalAttributes; // system specific depends on SystemMadeBy
+    wxUint16     m_z64infoOffset;      // Offset of ZIP64 local extra data for file sizes
 
     class wxZipMemory *m_Extra;
     class wxZipMemory *m_LocalExtra;
@@ -267,47 +280,53 @@ private:
     friend class wxZipInputStream;
     friend class wxZipOutputStream;
 
-    DECLARE_DYNAMIC_CLASS(wxZipEntry)
+    wxDECLARE_DYNAMIC_CLASS(wxZipEntry);
 };
 
 
 /////////////////////////////////////////////////////////////////////////////
-// wxZipOutputStream 
+// wxZipOutputStream
 
-WX_DECLARE_LIST_WITH_DECL(wxZipEntry, wx__ZipEntryList, class WXDLLIMPEXP_BASE);
+WX_DECLARE_LIST_WITH_DECL(wxZipEntry, wxZipEntryList_, class WXDLLIMPEXP_BASE);
 
 class WXDLLIMPEXP_BASE wxZipOutputStream : public wxArchiveOutputStream
 {
 public:
     wxZipOutputStream(wxOutputStream& stream,
                       int level = -1,
-                      wxMBConv& conv = wxConvLocal);
+                      wxMBConv& conv = wxConvUTF8);
+    wxZipOutputStream(wxOutputStream *stream,
+                      int level = -1,
+                      wxMBConv& conv = wxConvUTF8);
     virtual WXZIPFIX ~wxZipOutputStream();
 
     bool PutNextEntry(wxZipEntry *entry)        { return DoCreate(entry); }
 
     bool WXZIPFIX PutNextEntry(const wxString& name,
                                const wxDateTime& dt = wxDateTime::Now(),
-                               wxFileOffset size = wxInvalidOffset);
+                               wxFileOffset size = wxInvalidOffset) wxOVERRIDE;
 
     bool WXZIPFIX PutNextDirEntry(const wxString& name,
-                                  const wxDateTime& dt = wxDateTime::Now());
+                                  const wxDateTime& dt = wxDateTime::Now()) wxOVERRIDE;
 
     bool WXZIPFIX CopyEntry(wxZipEntry *entry, wxZipInputStream& inputStream);
     bool WXZIPFIX CopyArchiveMetaData(wxZipInputStream& inputStream);
 
-    void WXZIPFIX Sync();
-    bool WXZIPFIX CloseEntry();
-    bool WXZIPFIX Close();
+    void WXZIPFIX Sync() wxOVERRIDE;
+    bool WXZIPFIX CloseEntry() wxOVERRIDE;
+    bool WXZIPFIX Close() wxOVERRIDE;
 
     void SetComment(const wxString& comment)    { m_Comment = comment; }
 
     int  GetLevel() const                       { return m_level; }
     void WXZIPFIX SetLevel(int level);
-    
+
+    void SetFormat(wxZipArchiveFormat format)   { m_format = format; };
+    wxZipArchiveFormat GetFormat() const        { return m_format; };
+
 protected:
-    virtual size_t WXZIPFIX OnSysWrite(const void *buffer, size_t size);
-    virtual wxFileOffset OnSysTell() const      { return m_entrySize; }
+    virtual size_t WXZIPFIX OnSysWrite(const void *buffer, size_t size) wxOVERRIDE;
+    virtual wxFileOffset OnSysTell() const wxOVERRIDE      { return m_entrySize; }
 
     // this protected interface isn't yet finalised
     struct Buffer { const char *m_data; size_t m_size; };
@@ -320,9 +339,11 @@ protected:
         { return m_offsetAdjustment != wxInvalidOffset; }
 
 private:
-    bool WXZIPFIX PutNextEntry(wxArchiveEntry *entry);
-    bool WXZIPFIX CopyEntry(wxArchiveEntry *entry, wxArchiveInputStream& stream);
-    bool WXZIPFIX CopyArchiveMetaData(wxArchiveInputStream& stream);
+    void Init(int level);
+
+    bool WXZIPFIX PutNextEntry(wxArchiveEntry *entry) wxOVERRIDE;
+    bool WXZIPFIX CopyEntry(wxArchiveEntry *entry, wxArchiveInputStream& stream) wxOVERRIDE;
+    bool WXZIPFIX CopyArchiveMetaData(wxArchiveInputStream& stream) wxOVERRIDE;
 
     bool IsOpened() const { return m_comp || m_pending; }
 
@@ -333,7 +354,7 @@ private:
     class wxStoredOutputStream *m_store;
     class wxZlibOutputStream2 *m_deflate;
     class wxZipStreamLink *m_backlink;
-    wx__ZipEntryList m_entries;
+    wxZipEntryList_ m_entries;
     char *m_initialData;
     size_t m_initialSize;
     wxZipEntry *m_pending;
@@ -346,13 +367,15 @@ private:
     int m_level;
     wxFileOffset m_offsetAdjustment;
     wxString m_Comment;
+    bool m_endrecWritten;
+    wxZipArchiveFormat m_format;
 
-    DECLARE_NO_COPY_CLASS(wxZipOutputStream)
+    wxDECLARE_NO_COPY_CLASS(wxZipOutputStream);
 };
 
 
 /////////////////////////////////////////////////////////////////////////////
-// wxZipInputStream 
+// wxZipInputStream
 
 class WXDLLIMPEXP_BASE wxZipInputStream : public wxArchiveInputStream
 {
@@ -360,31 +383,23 @@ public:
     typedef wxZipEntry entry_type;
 
     wxZipInputStream(wxInputStream& stream, wxMBConv& conv = wxConvLocal);
-
-#if 1 //WXWIN_COMPATIBILITY_2_6
-    wxZipInputStream(const wxString& archive, const wxString& file)
-     : wxArchiveInputStream(OpenFile(archive), wxConvLocal) { Init(file); }
-#endif
+    wxZipInputStream(wxInputStream *stream, wxMBConv& conv = wxConvLocal);
 
     virtual WXZIPFIX ~wxZipInputStream();
 
     bool OpenEntry(wxZipEntry& entry)   { return DoOpen(&entry); }
-    bool WXZIPFIX CloseEntry();
+    bool WXZIPFIX CloseEntry() wxOVERRIDE;
 
     wxZipEntry *GetNextEntry();
 
     wxString WXZIPFIX GetComment();
     int WXZIPFIX GetTotalEntries();
 
-    virtual wxFileOffset GetLength() const { return m_entry.GetSize(); }
+    virtual wxFileOffset GetLength() const wxOVERRIDE { return m_entry.GetSize(); }
 
 protected:
-    size_t WXZIPFIX OnSysRead(void *buffer, size_t size);
-    wxFileOffset OnSysTell() const { return m_decomp ? m_decomp->TellI() : 0; }
-
-#if 1 //WXWIN_COMPATIBILITY_2_6
-    wxFileOffset WXZIPFIX OnSysSeek(wxFileOffset seek, wxSeekMode mode);
-#endif
+    size_t WXZIPFIX OnSysRead(void *buffer, size_t size) wxOVERRIDE;
+    wxFileOffset OnSysTell() const wxOVERRIDE { return m_decomp ? m_decomp->TellI() : 0; }
 
     // this protected interface isn't yet finalised
     virtual wxInputStream* WXZIPFIX OpenDecompressor(wxInputStream& stream);
@@ -393,11 +408,10 @@ protected:
 private:
     void Init();
     void Init(const wxString& file);
-    wxInputStream& OpenFile(const wxString& archive);
 
-    wxArchiveEntry *DoGetNextEntry()    { return GetNextEntry(); }
+    wxArchiveEntry *DoGetNextEntry() wxOVERRIDE    { return GetNextEntry(); }
 
-    bool WXZIPFIX OpenEntry(wxArchiveEntry& entry);
+    bool WXZIPFIX OpenEntry(wxArchiveEntry& entry) wxOVERRIDE;
 
     wxStreamError ReadLocal(bool readEndRec = false);
     wxStreamError ReadCentral();
@@ -405,7 +419,7 @@ private:
     wxUint32 ReadSignature();
     bool FindEndRecord();
     bool LoadEndRecord();
-    
+
     bool AtHeader() const       { return m_headerSize == 0; }
     bool AfterHeader() const    { return m_headerSize > 0 && !m_decomp; }
     bool IsOpened() const       { return m_decomp != NULL; }
@@ -418,7 +432,6 @@ private:
     class wxStoredInputStream *m_store;
     class wxZlibInputStream2 *m_inflate;
     class wxRawInputStream *m_rawin;
-    class wxFFileInputStream *m_ffile;
     wxZipEntry m_entry;
     bool m_raw;
     size_t m_headerSize;
@@ -438,12 +451,7 @@ private:
     friend bool wxZipOutputStream::CopyArchiveMetaData(
                     wxZipInputStream& inputStream);
 
-#if 1 //WXWIN_COMPATIBILITY_2_6
-    bool m_allowSeeking;
-    friend class wxZipFSInputStream;
-#endif
-
-    DECLARE_NO_COPY_CLASS(wxZipInputStream)
+    wxDECLARE_NO_COPY_CLASS(wxZipInputStream);
 };
 
 
@@ -472,27 +480,40 @@ public:
     typedef wxZipPairIter     pairiter_type;
 #endif
 
+    wxZipClassFactory();
+
     wxZipEntry *NewEntry() const
         { return new wxZipEntry; }
     wxZipInputStream *NewStream(wxInputStream& stream) const
         { return new wxZipInputStream(stream, GetConv()); }
     wxZipOutputStream *NewStream(wxOutputStream& stream) const
         { return new wxZipOutputStream(stream, -1, GetConv()); }
+    wxZipInputStream *NewStream(wxInputStream *stream) const
+        { return new wxZipInputStream(stream, GetConv()); }
+    wxZipOutputStream *NewStream(wxOutputStream *stream) const
+        { return new wxZipOutputStream(stream, -1, GetConv()); }
 
     wxString GetInternalName(const wxString& name,
-                             wxPathFormat format = wxPATH_NATIVE) const
+                             wxPathFormat format = wxPATH_NATIVE) const wxOVERRIDE
         { return wxZipEntry::GetInternalName(name, format); }
 
+    const wxChar * const *GetProtocols(wxStreamProtocolType type
+                                       = wxSTREAM_PROTOCOL) const wxOVERRIDE;
+
 protected:
-    wxArchiveEntry *DoNewEntry() const
+    wxArchiveEntry *DoNewEntry() const wxOVERRIDE
         { return NewEntry(); }
-    wxArchiveInputStream *DoNewStream(wxInputStream& stream) const
+    wxArchiveInputStream *DoNewStream(wxInputStream& stream) const wxOVERRIDE
         { return NewStream(stream); }
-    wxArchiveOutputStream *DoNewStream(wxOutputStream& stream) const
+    wxArchiveOutputStream *DoNewStream(wxOutputStream& stream) const wxOVERRIDE
+        { return NewStream(stream); }
+    wxArchiveInputStream *DoNewStream(wxInputStream *stream) const wxOVERRIDE
+        { return NewStream(stream); }
+    wxArchiveOutputStream *DoNewStream(wxOutputStream *stream) const wxOVERRIDE
         { return NewStream(stream); }
 
 private:
-    DECLARE_DYNAMIC_CLASS(wxZipClassFactory)
+    wxDECLARE_DYNAMIC_CLASS(wxZipClassFactory);
 };
 
 
@@ -516,17 +537,37 @@ inline bool wxZipEntry::IsReadOnly() const
 
 inline bool wxZipEntry::IsMadeByUnix() const
 {
-    const int pattern =
-        (1 << wxZIP_SYSTEM_OPENVMS) |
-        (1 << wxZIP_SYSTEM_UNIX) |
-        (1 << wxZIP_SYSTEM_ATARI_ST) |
-        (1 << wxZIP_SYSTEM_ACORN_RISC) |
-        (1 << wxZIP_SYSTEM_BEOS) | (1 << wxZIP_SYSTEM_TANDEM);
+    switch ( m_SystemMadeBy )
+    {
+        case wxZIP_SYSTEM_MSDOS:
+            // note: some unix zippers put madeby = dos
+            return (m_ExternalAttributes & ~0xFFFF) != 0;
 
-    // note: some unix zippers put madeby = dos
-    return (m_SystemMadeBy == wxZIP_SYSTEM_MSDOS
-            && (m_ExternalAttributes & ~0xFFFF))
-        || ((pattern >> m_SystemMadeBy) & 1);
+        case wxZIP_SYSTEM_OPENVMS:
+        case wxZIP_SYSTEM_UNIX:
+        case wxZIP_SYSTEM_ATARI_ST:
+        case wxZIP_SYSTEM_ACORN_RISC:
+        case wxZIP_SYSTEM_BEOS:
+        case wxZIP_SYSTEM_TANDEM:
+            return true;
+
+        case wxZIP_SYSTEM_AMIGA:
+        case wxZIP_SYSTEM_VM_CMS:
+        case wxZIP_SYSTEM_OS2_HPFS:
+        case wxZIP_SYSTEM_MACINTOSH:
+        case wxZIP_SYSTEM_Z_SYSTEM:
+        case wxZIP_SYSTEM_CPM:
+        case wxZIP_SYSTEM_WINDOWS_NTFS:
+        case wxZIP_SYSTEM_MVS:
+        case wxZIP_SYSTEM_VSE:
+        case wxZIP_SYSTEM_VFAT:
+        case wxZIP_SYSTEM_ALTERNATE_MVS:
+        case wxZIP_SYSTEM_OS_400:
+            return false;
+    }
+
+    // Unknown system, assume not Unix.
+    return false;
 }
 
 inline void wxZipEntry::SetIsText(bool isText)
@@ -554,6 +595,6 @@ inline void wxZipEntry::SetName(const wxString& name,
 }
 
 
-#endif // wxUSE_ZLIB && wxUSE_STREAMS && wxUSE_ZIPSTREAM
+#endif // wxUSE_ZIPSTREAM
 
 #endif // _WX_WXZIPSTREAM_H__
