@@ -2,7 +2,6 @@
 // Name:        strconv.h
 // Purpose:     interface of wxMBConvUTF7
 // Author:      wxWidgets team
-// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -69,6 +68,19 @@ public:
     */
     static size_t GetMaxMBNulLen();
 
+    /**
+        Return true if the converter's charset is UTF-8.
+
+        This is provided to optimize creating wxStrings from the @c char*
+        strings returned by this converter, as they can be directly used with
+        wxString::FromUTF8() or even wxString::FromUTF8Unchecked() when this
+        method returns @true.
+
+        This function is universally available since wxWidgets 3.1.1 (it was
+        previously only available in some of the build configurations).
+    */
+    virtual bool IsUTF8() const;
+
     /**
         Convert multibyte string to a wide character one.
 
@@ -122,7 +134,7 @@ public:
             including the terminating @c NUL character(s).
 
         @return
-            The number of character written (or which would have been written
+            The number of characters written (or which would have been written
             if it were non-@NULL) to @a dst or @c wxCONV_FAILED on error.
     */
     virtual size_t ToWChar(wchar_t* dst, size_t dstLen, const char* src,
@@ -149,8 +161,13 @@ public:
             including the terminating @c NUL character.
 
         @return
-            The number of character written (or which would have been written
-            if it were non-@NULL) to @a dst or @c wxCONV_FAILED on error.
+            If @dst is non-@NULL, the number of characters actually written to
+            it. If @dst is @NULL, the returned value is at least equal to the
+            number of characters that would have been written out if it were
+            non-@NULL, but can be larger than it under the platforms using
+            UTF-16 as @c wchar_t encoding (this allows a useful optimization in
+            the implementation of this function for UTF-32). In any case,
+            @c wxCONV_FAILED is returned on conversion error.
     */
     virtual size_t FromWChar(char* dst, size_t dstLen, const wchar_t* src,
                              size_t srcLen = wxNO_LEN) const;
@@ -173,7 +190,7 @@ public:
         invalid and @a outLen is set to 0 (and not @c wxCONV_FAILED for
         compatibility concerns).
     */
-    const wxWCharBuffer cMB2WC(const char* in,
+    wxWCharBuffer cMB2WC(const char* in,
                                size_t inLen,
                                size_t *outLen) const;
 
@@ -192,7 +209,7 @@ public:
 
         @since 2.9.1
      */
-    const wxWCharBuffer cMB2WC(const wxCharBuffer& buf) const;
+    wxWCharBuffer cMB2WC(const wxCharBuffer& buf) const;
 
     //@{
     /**
@@ -204,7 +221,7 @@ public:
         is defined as the correct return type (without const).
     */
     const char* cMB2WX(const char* psz) const;
-    const wxWCharBuffer cMB2WX(const char* psz) const;
+    wxWCharBuffer cMB2WX(const char* psz) const;
     //@}
 
     /**
@@ -217,7 +234,7 @@ public:
         Its parameters have the same meaning as the corresponding parameters of
         FromWChar(), please see the description of cMB2WC() for more details.
     */
-    const wxCharBuffer cWC2MB(const wchar_t* in,
+    wxCharBuffer cWC2MB(const wchar_t* in,
                               size_t inLen,
                               size_t *outLen) const;
 
@@ -236,7 +253,7 @@ public:
 
         @since 2.9.1
      */
-    const wxCharBuffer cWC2MB(const wxWCharBuffer& buf) const;
+    wxCharBuffer cWC2MB(const wxWCharBuffer& buf) const;
 
     //@{
     /**
@@ -247,7 +264,7 @@ public:
         defined as the correct return type (without const).
     */
     const wchar_t* cWC2WX(const wchar_t* psz) const;
-    const wxCharBuffer cWC2WX(const wchar_t* psz) const;
+    wxCharBuffer cWC2WX(const wchar_t* psz) const;
     //@}
 
     //@{
@@ -259,7 +276,7 @@ public:
         is defined as the correct return type (without const).
     */
     const char* cWX2MB(const wxChar* psz) const;
-    const wxCharBuffer cWX2MB(const wxChar* psz) const;
+    wxCharBuffer cWX2MB(const wxChar* psz) const;
     //@}
 
     //@{
@@ -271,13 +288,13 @@ public:
         defined as the correct return type (without const).
     */
     const wchar_t* cWX2WC(const wxChar* psz) const;
-    const wxWCharBuffer cWX2WC(const wxChar* psz) const;
+    wxWCharBuffer cWX2WC(const wxChar* psz) const;
     //@}
 
     /**
         @deprecated This function is deprecated, please use ToWChar() instead.
 
-        Converts from a string @a in in multibyte encoding to Unicode putting up to
+        Converts from a string @a in multibyte encoding to Unicode putting up to
         @a outLen characters into the buffer @e out.
 
         If @a out is @NULL, only the length of the string which would result
@@ -479,25 +496,46 @@ public:
     bool IsOk() const;
 };
 
+/**
+    Conversion object always producing non-empty output for non-empty input.
+
+    Conversions done using this object never lose data, at the cost of possibly
+    producing the output in an unwanted encoding or misinterpreting input
+    encoding.
+
+    To be precise, converting Unicode to multibyte strings using this object
+    tries to use the current locale encoding first but if this doesn't work, it
+    falls back to using UTF-8. In the other direction, UTF-8 is tried first,
+    then the current locale encoding and if this fails too, input is
+    interpreted as using ISO 8859-1, which never fails.
+
+    It is almost always @e wrong to use this converter for multibyte-to-Unicode
+    direction as the program should know which encoding the input data is
+    supposed to use and use the appropriate converter instead. However it may
+    be useful in the Unicode-to-multibyte direction if the goal is to produce
+    the output in the current locale encoding if possible, but still output
+    something, instead of nothing at all, even if the Unicode string is not
+    representable in this encoding.
+
+    @since 3.1.0
+ */
+extern wxMBConv& wxConvWhateverWorks;
 
 
 /**
-    @class wxMBConvFile
-
-    This class used to define the class instance @b wxConvFileName, but
-    nowadays @b wxConvFileName is either of type wxConvLibc (on most platforms)
-    or wxConvUTF8 (on MacOS X).
+    Conversion object used for converting file names from their external
+    representation to the one used inside the program.
 
     @b wxConvFileName converts filenames between filesystem multibyte encoding
     and Unicode. @b wxConvFileName can also be set to a something else at
-    run-time which is used e.g. by wxGTK to use a class which checks the
+    run-time which is used e.g. by wxGTK to use an object which checks the
     environment variable @b G_FILESYSTEM_ENCODING indicating that filenames
     should not be interpreted as UTF8 and also for converting invalid UTF8
     characters (e.g. if there is a filename in iso8859_1) to strings with octal
     values.
 
     Since some platforms (such as Win32) use Unicode in the filenames,
-    and others (such as Unix) use multibyte encodings, this class should only
+    and others (such as Unix) use multibyte encodings, this object should only
     be used directly if wxMBFILES is defined to 1. A convenience macro,
     @c wxFNCONV, is defined to @c wxConvFileName->cWX2MB in this case. You
     could use it like this:
@@ -508,7 +546,7 @@ public:
     @endcode
 
     (although it would be better to just use wxFopen(name, "r") in this
-    particular case, you only need to use this class for functions taking file
+    particular case, you only need to use this object for functions taking file
     names not wrapped by wxWidgets.)
 
     @library{wxbase}
@@ -516,7 +554,4 @@ public:
 
     @see @ref overview_mbconv
 */
-class wxMBConvFile : public wxMBConv
-{
-public:
-};
+extern wxMBConv* wxConvFileName;
