@@ -2,7 +2,6 @@
 // Name:        toplevel.h
 // Purpose:     interface of wxTopLevelWindow
 // Author:      wxWidgets team
-// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -34,6 +33,14 @@ enum
                           wxFULLSCREEN_NOCAPTION
 };
 
+#define wxDEFAULT_FRAME_STYLE (wxSYSTEM_MENU |          \
+                               wxRESIZE_BORDER |        \
+                               wxMINIMIZE_BOX |         \
+                               wxMAXIMIZE_BOX |         \
+                               wxCLOSE_BOX |            \
+                               wxCAPTION |              \
+                               wxCLIP_CHILDREN)
+
 /**
     @class wxTopLevelWindow
 
@@ -46,7 +53,7 @@ enum
     internal top level window list.
 
     @beginEventEmissionTable
-    @event{EVT_ACTIVATE(id, func)}
+    @event{EVT_MAXIMIZE(id, func)}
         Process a @c wxEVT_MAXIMIZE event. See wxMaximizeEvent.
     @event{EVT_MOVE(func)}
         Process a @c wxEVT_MOVE event, which is generated when a window is moved.
@@ -59,6 +66,8 @@ enum
         Process a @c wxEVT_MOVE_END event, which is generated when the user stops
         moving or sizing a window. wxMSW only.
         See wxMoveEvent.
+    @event{EVT_SHOW(func)}
+        Process a @c wxEVT_SHOW event. See wxShowEvent.
     @endEventTable
 
     @library{wxcore}
@@ -66,7 +75,7 @@ enum
 
     @see wxDialog, wxFrame
 */
-class wxTopLevelWindow : public wxWindow
+class wxTopLevelWindow : public wxNonOwnedWindow
 {
 public:
     /**
@@ -78,7 +87,7 @@ public:
         Constructor creating the top level window.
     */
     wxTopLevelWindow(wxWindow *parent,
-                    wxWindowID winid,
+                    wxWindowID id,
                     const wxString& title,
                     const wxPoint& pos = wxDefaultPosition,
                     const wxSize& size = wxDefaultSize,
@@ -115,7 +124,7 @@ public:
     /**
         A synonym for CentreOnScreen().
     */
-    void CenterOnScreen(int direction);
+    void CenterOnScreen(int direction = wxBOTH);
 
     /**
         Centres the window on screen.
@@ -133,14 +142,45 @@ public:
         corner of a dialog) and the Close entry of the system menu (most often
         in the left upper corner of the dialog).
 
-        Currently only implemented for wxMSW and wxGTK.
-
         Returns @true if operation was successful. This may be wrong on X11
         (including GTK+) where the window manager may not support this operation
         and there is no way to find out.
     */
     virtual bool EnableCloseButton(bool enable = true);
 
+    /**
+        Enables or disables the Maximize button (in the right or left upper
+        corner of a frame or dialog).
+
+        Currently only implemented for wxMSW and wxOSX.
+
+        The window style must contain wxMAXIMIZE_BOX.
+
+        Returns @true if operation was successful. Note that a successful
+        operation does not change the window style flags.
+
+        @since 3.1.0
+    */
+    virtual bool EnableMaximizeButton(bool enable = true);
+
+    /**
+        Enables or disables the Minimize button (in the right or left upper
+        corner of a frame or dialog).
+
+        Currently only implemented for wxMSW and wxOSX.
+
+        The window style must contain wxMINIMIZE_BOX.
+
+        Note that in wxMSW a successful operation will change the window
+        style flags.
+
+        Returns @true if operation was successful. Note that a successful
+        operation does not change the window style flags.
+
+        @since 3.1.0
+    */
+    virtual bool EnableMinimizeButton(bool enable = true);
+
     /**
         Returns a pointer to the button which is the default for this window, or
         @c @NULL. The default button is the one activated by pressing the Enter
@@ -148,6 +188,19 @@ public:
     */
     wxWindow* GetDefaultItem() const;
 
+    /**
+        Get the default size for a new top level window.
+
+        This is used internally by wxWidgets on some platforms to determine the
+        default size for a window created using ::wxDefaultSize so it is not
+        necessary to use it when creating a wxTopLevelWindow, however it may be
+        useful if a rough estimation of the window size is needed for some
+        other reason.
+
+        @since 2.9.2
+     */
+    static wxSize GetDefaultSize();
+
     /**
         Returns the standard icon of the window. The icon will be invalid if it
         hadn't been previously set by SetIcon().
@@ -172,26 +225,18 @@ public:
     */
     virtual wxString GetTitle() const;
 
-    /**
-        Unique to the wxWinCE port. Responds to showing/hiding SIP (soft input
-        panel) area and resize window accordingly. Override this if you want to
-        avoid resizing or do additional operations.
-    */
-    virtual bool HandleSettingChange(WXWPARAM wParam,
-                                     WXLPARAM lParam);
-
     /**
         Iconizes or restores the window.
 
         @param iconize
             If @true, iconizes the window; if @false, shows and restores it.
 
-        @see IsIconized(), Maximize(), wxIconizeEvent.
+        @see IsIconized(), Restore()(), wxIconizeEvent.
     */
     virtual void Iconize(bool iconize = true);
 
     /**
-        Returns @true if this window is currently active, i.e. if the user is
+        Returns @true if this window is currently active, i.e.\ if the user is
         currently working with it.
     */
     virtual bool IsActive();
@@ -243,10 +288,37 @@ public:
         @param maximize
             If @true, maximizes the window, otherwise it restores it.
 
-        @see Iconize()
+        @see Restore(), Iconize()
     */
     virtual void Maximize(bool maximize = true);
 
+    /**
+        MSW-specific function for accessing the system menu.
+
+        Returns a wxMenu pointer representing the system menu of the window
+        under MSW. The returned wxMenu may be used, if non-@c NULL, to add
+        extra items to the system menu. The usual @c wxEVT_MENU
+        events (that can be processed using @c EVT_MENU event table macro) will
+        then be generated for them. All the other wxMenu methods may be used as
+        well but notice that they won't allow you to access any standard system
+        menu items (e.g. they can't be deleted or modified in any way
+        currently).
+
+        Notice that because of the native system limitations the identifiers of
+        the items added to the system menu must be multiples of 16, otherwise
+        no events will be generated for them.
+
+        The returned pointer must @em not be deleted, it is owned by the window
+        and will be only deleted when the window itself is destroyed.
+
+        This function is not available in the other ports by design, any
+        occurrences of it in the portable code must be guarded by
+        @code #ifdef __WXMSW__ @endcode preprocessor guards.
+
+        @since 2.9.3
+     */
+    wxMenu *MSWGetSystemMenu() const;
+
     /**
         Use a system-dependent way to attract users attention to the window when
         it is in background.
@@ -266,6 +338,18 @@ public:
     */
     virtual void RequestUserAttention(int flags = wxUSER_ATTENTION_INFO);
 
+    /**
+        Restore a previously iconized or maximized window to its normal state.
+
+        In wxGTK this method currently doesn't return the maximized window to
+        its normal state and you must use Maximize() with @false argument
+        explicitly for this. In the other ports, it both unmaximizes the
+        maximized windows and uniconizes the iconized ones.
+
+        @see Iconize(), Maximize()
+     */
+    void Restore();
+
     /**
         Changes the default item for the panel, usually @a win is a button.
 
@@ -273,6 +357,11 @@ public:
     */
     wxWindow* SetDefaultItem(wxWindow* win);
 
+    
+    wxWindow*  SetTmpDefaultItem(wxWindow * win);
+    wxWindow* GetTmpDefaultItem() const;
+
+    
     /**
         Sets the icon for this window.
 
@@ -305,24 +394,6 @@ public:
     */
     virtual void SetIcons(const wxIconBundle& icons);
 
-    /**
-        Sets action or menu activated by pressing left hardware button on the
-        smart phones. Unavailable on full keyboard machines.
-
-        @param id
-            Identifier for this button.
-        @param label
-            Text to be displayed on the screen area dedicated to this hardware
-            button.
-        @param subMenu
-            The menu to be opened after pressing this hardware button.
-
-        @see SetRightMenu().
-    */
-    void SetLeftMenu(int id = wxID_ANY,
-                     const wxString& label = wxEmptyString,
-                     wxMenu* subMenu = NULL);
-
     /**
         A simpler interface for setting the size hints than SetSizeHints().
     */
@@ -333,33 +404,6 @@ public:
     */
     virtual void SetMinSize(const wxSize& size);
 
-    /**
-        Sets action or menu activated by pressing right hardware button on the
-        smart phones. Unavailable on full keyboard machines.
-
-        @param id
-            Identifier for this button.
-        @param label
-            Text to be displayed on the screen area dedicated to this hardware
-            button.
-        @param subMenu
-            The menu to be opened after pressing this hardware button.
-
-        @see SetLeftMenu().
-    */
-    void SetRightMenu(int id = wxID_ANY,
-                      const wxString& label = wxEmptyString,
-                      wxMenu* subMenu = NULL);
-
-    /**
-        If the platform supports it, sets the shape of the window to that
-        depicted by @a region. The system will not display or respond to any
-        mouse event for the pixels that lie outside of the region. To reset the
-        window to the normal rectangular shape simply call SetShape() again with
-        an empty wxRegion. Returns @true if the operation is successful.
-    */
-    virtual bool SetShape(const wxRegion& region);
-
     /**
         Allows specification of minimum and maximum window sizes, and window
         size increments. If a pair of values is not set (or set to -1), no
@@ -425,7 +469,7 @@ public:
 
         @param alpha
             Determines how opaque or transparent the window will be, if the
-            platform supports the opreration. A value of 0 sets the window to be
+            platform supports the operation. A value of 0 sets the window to be
             fully transparent, and a value of 255 sets the window to be fully
             opaque.
     */
@@ -457,6 +501,56 @@ public:
     */
     virtual bool OSXIsModified() const;
 
+    /**
+        Sets the file name represented by this wxTopLevelWindow.
+
+        Under OS X, this file name is used to set the "proxy icon", which
+        appears in the window title bar near its title, corresponding to this
+        file name. Under other platforms it currently doesn't do anything but
+        it is harmless to call it now and it might be implemented to do
+        something useful in the future so you're encouraged to use it for any
+        window representing a file-based document.
+
+        @since 2.9.4
+    */
+    virtual void SetRepresentedFilename(const wxString& filename);
+
+    /**
+        Show the wxTopLevelWindow, but do not give it keyboard focus. This can be
+        used for pop up or notification windows that should not steal the current
+        focus.
+     */
+    virtual void ShowWithoutActivating();
+
+    /**
+        Enables the maximize button to toggle full screen mode. Prior to 
+        OS X 10.10 a full screen button is added to the right upper corner
+        of a window's title bar.
+
+        Currently only available for wxOSX/Cocoa.
+
+        @param enable
+            If @true (default) adds the full screen button in the title bar;
+            if @false the button is removed.
+
+        @return @true if the button was added or removed, @false if running
+        under another OS.
+
+        @note Having the button is also required to let ShowFullScreen()
+        make use of the full screen API available since OS X 10.7: a full
+        screen window gets its own space and entering and exiting the mode
+        is animated.
+        If the button is not present the old way of switching to full screen
+        is used.
+
+        @onlyfor{wxosx}
+
+        @see ShowFullScreen()
+
+        @since 3.1.0
+    */
+    virtual bool EnableFullScreenView(bool enable = true);
+
     /**
         Depending on the value of @a show parameter the window is either shown
         full screen or restored to its normal state. @a style is a bit list
@@ -475,7 +569,7 @@ public:
         @note Showing a window full screen also actually @ref wxWindow::Show()
               "Show()"s the window if it isn't shown.
 
-        @see IsFullScreen()
+        @see EnableFullScreenView(), IsFullScreen()
     */
     virtual bool ShowFullScreen(bool show, long style = wxFULLSCREEN_ALL);
 
