@@ -4,7 +4,6 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     2005-01-20
-// RCS-ID:      $Id$
 // Copyright:   (c) 2005 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -18,14 +17,17 @@
 #include "wx/frame.h"
 #include "wx/icon.h"
 #include "wx/menu.h"
+#include "wx/choicdlg.h"
 #include "wx/msgdlg.h"
 #include "wx/button.h"
 #include "wx/dcclient.h"
+#include "wx/timer.h"
 
 #include "wx/datetime.h"
 #include "wx/ffile.h"
 #include "wx/filename.h"
 #include "wx/debugrpt.h"
+#include "wx/dynlib.h"
 
 #if !wxUSE_DEBUGREPORT
     #error "This sample can't be built without wxUSE_DEBUGREPORT"
@@ -35,7 +37,7 @@
     #error "This sample can't be built without wxUSE_ON_FATAL_EXCEPTION"
 #endif // wxUSE_ON_FATAL_EXCEPTION
 
-#if !defined(__WXMSW__) && !defined(__WXPM__)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -62,7 +64,7 @@ protected:
     // this is called with the contents of the server response: you will
     // probably want to parse the XML document in OnServerReply() instead of
     // just dumping it as I do
-    virtual bool OnServerReply(const wxArrayString& reply)
+    virtual bool OnServerReply(const wxArrayString& reply) wxOVERRIDE
     {
         if ( reply.IsEmpty() )
         {
@@ -142,10 +144,12 @@ void foo(int n)
 
 enum
 {
+    DebugRpt_ListLoadedDLLs = 50,
     DebugRpt_Quit = wxID_EXIT,
     DebugRpt_Crash = 100,
     DebugRpt_Current,
     DebugRpt_Paint,
+    DebugRpt_Timer,
     DebugRpt_Upload,
     DebugRpt_About = wxID_ABOUT
 };
@@ -156,21 +160,34 @@ public:
     MyFrame();
 
 private:
+    void OnListLoadedDLLs(wxCommandEvent& event);
     void OnQuit(wxCommandEvent& event);
     void OnReportForCrash(wxCommandEvent& event);
     void OnReportForCurrent(wxCommandEvent& event);
     void OnReportPaint(wxCommandEvent& event);
+    void OnReportTimer(wxCommandEvent& event);
     void OnReportUpload(wxCommandEvent& event);
     void OnAbout(wxCommandEvent& event);
 
     void OnPaint(wxPaintEvent& event);
 
+    // a timer whose only purpose in life is to crash as soon as it's started
+    class BadTimer : public wxTimer
+    {
+    public:
+        BadTimer() { }
+
+        virtual void Notify() wxOVERRIDE
+        {
+            foo(8);
+        }
+    } m_badTimer;
 
     // number of lines drawn in OnPaint()
     int m_numLines;
 
     wxDECLARE_NO_COPY_CLASS(MyFrame);
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 // ----------------------------------------------------------------------------
@@ -188,10 +205,10 @@ public:
     MyApp();
 
     // create our main window here
-    virtual bool OnInit();
+    virtual bool OnInit() wxOVERRIDE;
 
     // called when a crash occurs in this application
-    virtual void OnFatalException();
+    virtual void OnFatalException() wxOVERRIDE;
 
     // this is where we really generate the debug report
     void GenerateReport(wxDebugReport::Context ctx);
@@ -207,7 +224,7 @@ private:
     wxDECLARE_NO_COPY_CLASS(MyApp);
 };
 
-IMPLEMENT_APP(MyApp)
+wxIMPLEMENT_APP(MyApp);
 
 // ============================================================================
 // implementation
@@ -217,16 +234,18 @@ IMPLEMENT_APP(MyApp)
 // MyFrame
 // ----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(MyFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
+    EVT_MENU(DebugRpt_ListLoadedDLLs, MyFrame::OnListLoadedDLLs)
     EVT_MENU(DebugRpt_Quit, MyFrame::OnQuit)
     EVT_MENU(DebugRpt_Crash, MyFrame::OnReportForCrash)
     EVT_MENU(DebugRpt_Current, MyFrame::OnReportForCurrent)
     EVT_MENU(DebugRpt_Paint, MyFrame::OnReportPaint)
+    EVT_MENU(DebugRpt_Timer, MyFrame::OnReportTimer)
     EVT_MENU(DebugRpt_Upload, MyFrame::OnReportUpload)
     EVT_MENU(DebugRpt_About, MyFrame::OnAbout)
 
     EVT_PAINT(MyFrame::OnPaint)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 MyFrame::MyFrame()
        : wxFrame(NULL, wxID_ANY, wxT("wxWidgets Debug Report Sample"),
@@ -237,6 +256,8 @@ MyFrame::MyFrame()
     SetIcon(wxICON(sample));
 
     wxMenu *menuFile = new wxMenu;
+    menuFile->Append(DebugRpt_ListLoadedDLLs, wxT("&List loaded DLLs...\tCtrl-L"));
+    menuFile->AppendSeparator();
     menuFile->Append(DebugRpt_Quit, wxT("E&xit\tAlt-X"));
 
     wxMenu *menuReport = new wxMenu;
@@ -246,12 +267,14 @@ MyFrame::MyFrame()
                        wxT("Create report for the current program context"));
     menuReport->Append(DebugRpt_Paint, wxT("Report for &paint handler\tCtrl-P"),
                        wxT("Provoke a repeatable crash in wxEVT_PAINT handler"));
+    menuReport->Append(DebugRpt_Timer, wxT("Report for &timer handler\tCtrl-T"),
+                       wxT("Provoke a crash in wxEVT_TIMER handler"));
     menuReport->AppendSeparator();
     menuReport->AppendCheckItem(DebugRpt_Upload, wxT("Up&load debug report"),
                        wxT("You need to configure a web server accepting debug report uploads to use this function"));
 
     wxMenu *menuHelp = new wxMenu;
-    menuHelp->Append(DebugRpt_About, wxT("&About...\tF1"));
+    menuHelp->Append(DebugRpt_About, wxT("&About\tF1"));
 
     wxMenuBar *mbar = new wxMenuBar();
     mbar->Append(menuFile, wxT("&File"));
@@ -264,6 +287,47 @@ MyFrame::MyFrame()
     Show();
 }
 
+void MyFrame::OnListLoadedDLLs(wxCommandEvent& WXUNUSED(event))
+{
+    const wxDynamicLibraryDetailsArray loaded = wxDynamicLibrary::ListLoaded();
+    const size_t count = loaded.size();
+    if ( !count )
+    {
+        wxLogError("Failed to get the list of loaded dynamic libraries.");
+        return;
+    }
+
+    wxArrayString names;
+    names.reserve(count);
+    for ( size_t n = 0; n < count; n++ )
+    {
+        names.push_back(loaded[n].GetName());
+    }
+
+    for ( ;; )
+    {
+        const int sel = wxGetSingleChoiceIndex
+                        (
+                            "Choose a library to show more information about it",
+                            "List of loaded dynamic libraries",
+                            names,
+                            this
+                        );
+        if ( sel == wxNOT_FOUND )
+            return;
+
+        const wxDynamicLibraryDetails& det = loaded[sel];
+        void *addr = 0;
+        size_t len = 0;
+        det.GetAddress(&addr, &len);
+        wxLogMessage("Full path is \"%s\", memory range %p:%p, version \"%s\"",
+                     det.GetPath(),
+                     addr,
+                     static_cast<void*>(static_cast<char*>(addr) + len),
+                     det.GetVersion());
+    }
+}
+
 void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
 {
     Close(true);
@@ -292,6 +356,12 @@ void MyFrame::OnReportPaint(wxCommandEvent& WXUNUSED(event))
     Refresh();
 }
 
+void MyFrame::OnReportTimer(wxCommandEvent& WXUNUSED(event))
+{
+    // this will result in a crash in BadTimer::OnNotify() soon
+    m_badTimer.StartOnce(100);
+}
+
 void MyFrame::OnReportUpload(wxCommandEvent& event)
 {
     wxGetApp().UploadReport(event.IsChecked());
@@ -322,7 +392,7 @@ void MyFrame::OnPaint(wxPaintEvent& WXUNUSED(event))
 
 MyApp::MyApp()
 {
-    // user needs to explicitely enable this
+    // user needs to explicitly enable this
     m_uploadReport = false;
 
     // call this to tell the library to call our OnFatalException()
@@ -370,10 +440,20 @@ void MyApp::GenerateReport(wxDebugReport::Context ctx)
     // can also add an existing file directly, it will be copied
     // automatically
 #ifdef __WXMSW__
-    report->AddFile(wxT("c:\\autoexec.bat"), wxT("DOS startup file"));
-#else
-    report->AddFile(wxT("/etc/motd"), wxT("Message of the day"));
-#endif
+    wxString windir;
+    if ( !wxGetEnv("WINDIR", &windir) )
+        windir = "C:\\Windows";
+    fn.AssignDir(windir);
+    fn.AppendDir("system32");
+    fn.AppendDir("drivers");
+    fn.AppendDir("etc");
+#else // !__WXMSW__
+    fn.AssignDir("/etc");
+#endif // __WXMSW__/!__WXMSW__
+    fn.SetFullName("hosts");
+
+    if ( fn.FileExists() )
+        report->AddFile(fn.GetFullPath(), "Local hosts file");
 
     // calling Show() is not mandatory, but is more polite
     if ( wxDebugReportPreviewStd().Show(*report) )
