@@ -3,7 +3,6 @@
 // Purpose:     Common dialogs demo
 // Author:      Julian Smart, Vadim Zeitlin, ABX
 // Created:     04/01/98
-// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 //              (c) 2004 ABX
 //              (c) Vadim Zeitlin
@@ -29,19 +28,7 @@ of MSW, MAC and OS2
     #define USE_WXUNIVERSAL 0
 #endif
 
-#ifdef WXUSINGDLL
-    #define USE_DLL 1
-#else
-    #define USE_DLL 0
-#endif
-
-#if defined(__WXWINCE__)
-    #define USE_WXWINCE 1
-#else
-    #define USE_WXWINCE 0
-#endif
-
-#if defined(__WXMSW__) && !USE_WXWINCE
+#if defined(__WXMSW__)
     #define USE_WXMSW 1
 #else
     #define USE_WXMSW 0
@@ -53,7 +40,7 @@ of MSW, MAC and OS2
     #define USE_WXMAC 0
 #endif
 
-#if defined(__WXMAC_OSX__) && ( MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2) && USE_NATIVE_FONT_DIALOG_FOR_MACOSX
+#if USE_NATIVE_FONT_DIALOG_FOR_MACOSX
     #define USE_WXMACFONTDLG 1
 #else
     #define USE_WXMACFONTDLG 0
@@ -65,27 +52,18 @@ of MSW, MAC and OS2
     #define USE_WXGTK 0
 #endif
 
-#ifdef __WXPM__
-    #define USE_WXPM 1
-#else
-    #define USE_WXPM 0
-#endif
-
-#define USE_GENERIC_DIALOGS (!USE_WXUNIVERSAL && !USE_DLL)
-
 #define USE_COLOURDLG_GENERIC \
-    ((USE_WXMSW || USE_WXMAC) && USE_GENERIC_DIALOGS && wxUSE_COLOURDLG)
+    ((USE_WXMSW || USE_WXMAC) && USE_WXUNIVERSAL && wxUSE_COLOURDLG)
 #define USE_DIRDLG_GENERIC \
-    ((USE_WXMSW || USE_WXMAC) && USE_GENERIC_DIALOGS && wxUSE_DIRDLG)
+    ((USE_WXMSW || USE_WXMAC) && USE_WXUNIVERSAL && wxUSE_DIRDLG)
 #define USE_FILEDLG_GENERIC \
-    ((((USE_WXMSW || USE_WXMAC || USE_WXPM || USE_WXGTK) \
-                    && USE_GENERIC_DIALOGS) || USE_WXWINCE) && wxUSE_FILEDLG)
+    ((USE_WXMSW || USE_WXMAC) && USE_WXUNIVERSAL  && wxUSE_FILEDLG)
 #define USE_FONTDLG_GENERIC \
-    ((USE_WXMSW || USE_WXMACFONTDLG || USE_WXPM) && USE_GENERIC_DIALOGS && wxUSE_FONTDLG)
+    ((USE_WXMSW || USE_WXMACFONTDLG) && USE_WXUNIVERSAL && wxUSE_FONTDLG)
 
 // Turn USE_MODAL_PRESENTATION to 0 if there is any reason for not presenting difference
 // between modal and modeless dialogs (ie. not implemented it in your port yet)
-#if defined(__SMARTPHONE__) || !wxUSE_BOOKCTRL
+#if !wxUSE_BOOKCTRL
     #define USE_MODAL_PRESENTATION 0
 #else
     #define USE_MODAL_PRESENTATION 1
@@ -106,7 +84,7 @@ of MSW, MAC and OS2
 class MyAppTraits : public wxGUIAppTraits
 {
 public:
-    virtual wxLog *CreateLogTarget();
+    virtual wxLog *CreateLogTarget() wxOVERRIDE;
 };
 
 #endif // wxUSE_LOG
@@ -115,12 +93,26 @@ public:
 class MyApp: public wxApp
 {
 public:
-    virtual bool OnInit();
+    MyApp() { m_startupProgressStyle = -1; }
+
+    virtual bool OnInit() wxOVERRIDE;
+
+#if wxUSE_CMDLINE_PARSER
+    virtual void OnInitCmdLine(wxCmdLineParser& parser) wxOVERRIDE;
+    virtual bool OnCmdLineParsed(wxCmdLineParser& parser) wxOVERRIDE;
+#endif // wxUSE_CMDLINE_PARSER
 
 protected:
 #if wxUSE_LOG
-    virtual wxAppTraits *CreateTraits() { return new MyAppTraits; }
+    virtual wxAppTraits *CreateTraits() wxOVERRIDE { return new MyAppTraits; }
 #endif // wxUSE_LOG
+
+private:
+    // Flag set to a valid value if command line option "progress" is used,
+    // this allows testing of wxProgressDialog before the main event loop is
+    // started. If this option is not specified it is set to -1 by default
+    // meaning that progress dialog shouldn't be shown at all.
+    long m_startupProgressStyle;
 };
 
 #if USE_MODAL_PRESENTATION
@@ -135,7 +127,7 @@ public:
     void OnClose(wxCloseEvent& event);
 
 private:
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 // A custom modal dialog
@@ -151,7 +143,7 @@ private:
              *m_btnModeless,
              *m_btnDelete;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 #endif // USE_MODAL_PRESENTATION
@@ -182,7 +174,7 @@ private:
 
     wxSizer *m_buttonsSizer;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 // Test harness for wxMessageDialog.
@@ -191,18 +183,33 @@ class TestMessageBoxDialog : public wxDialog
 public:
     TestMessageBoxDialog(wxWindow *parent);
 
-private:
+    bool Create();
+
+protected:
+    wxString GetBoxTitle() { return m_textTitle->GetValue(); }
+    wxString GetMessage() { return m_textMsg->GetValue(); }
+    long GetStyle();
+
+    void PrepareMessageDialog(wxMessageDialogBase &dlg);
+
+    virtual void AddAdditionalTextOptions(wxSizer *WXUNUSED(sizer)) { }
+    virtual void AddAdditionalFlags(wxSizer *WXUNUSED(sizer)) { }
+
+    void ShowResult(int res);
+
     void OnApply(wxCommandEvent& event);
     void OnClose(wxCommandEvent& event);
     void OnUpdateLabelUI(wxUpdateUIEvent& event);
     void OnUpdateNoDefaultUI(wxUpdateUIEvent& event);
 
+private:
     enum
     {
         Btn_Yes,
         Btn_No,
         Btn_Ok,
         Btn_Cancel,
+        Btn_Help,
         Btn_Max
     };
 
@@ -222,10 +229,12 @@ private:
         MsgDlgIcon_Question,
         MsgDlgIcon_Warning,
         MsgDlgIcon_Error,
+        MsgDlgIcon_AuthNeeded,
         MsgDlgIcon_Max
     };
 
-    wxTextCtrl *m_textMsg,
+    wxTextCtrl *m_textTitle,
+               *m_textMsg,
                *m_textExtMsg;
 
     wxCheckBox *m_buttons[Btn_Max];
@@ -236,33 +245,84 @@ private:
     wxCheckBox *m_chkNoDefault,
                *m_chkCentre;
 
-    DECLARE_EVENT_TABLE()
+    wxStaticText *m_labelResult;
+
+    wxDECLARE_EVENT_TABLE();
     wxDECLARE_NO_COPY_CLASS(TestMessageBoxDialog);
 };
 
+#if wxUSE_RICHMSGDLG
+class TestRichMessageDialog : public TestMessageBoxDialog
+{
+public:
+    TestRichMessageDialog(wxWindow *parent);
+
+protected:
+    // overrides method in base class
+    virtual void AddAdditionalTextOptions(wxSizer *sizer) wxOVERRIDE;
+    virtual void AddAdditionalFlags(wxSizer *sizer) wxOVERRIDE;
+
+    void OnApply(wxCommandEvent& event);
+
+private:
+    wxTextCtrl *m_textCheckBox;
+    wxCheckBox *m_initialValueCheckBox;
+    wxTextCtrl *m_textDetailed;
+    wxTextCtrl *m_textFooter;
+    wxChoice *m_iconsFooter;
+
+    wxDECLARE_EVENT_TABLE();
+};
+#endif // wxUSE_RICHMSGDLG
+
 class TestDefaultActionDialog: public wxDialog
 {
 public:
     TestDefaultActionDialog( wxWindow *parent );
 
     void OnListBoxDClick(wxCommandEvent& event);
+    void OnDisableOK(wxCommandEvent& event);
+    void OnDisableCancel(wxCommandEvent& event);
     void OnCatchListBoxDClick(wxCommandEvent& event);
+    void OnTextEnter(wxCommandEvent& event);
 
 private:
     bool   m_catchListBoxDClick;
 
 private:
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 
 #if USE_SETTINGS_DIALOG
+
+// Struct containing properties edited by SettingsDialog.
+struct SettingsData
+{
+    SettingsData() :
+        m_loadLastOnStartup(false),
+        m_autoSaveInterval(1),
+        m_showToolTips(false),
+        m_applyTo(0),
+        m_bgStyle(0),
+        m_titleFontSize(10)
+    {
+    }
+
+    bool m_loadLastOnStartup;
+    int m_autoSaveInterval;
+    bool m_showToolTips;
+    int m_applyTo;
+    int m_bgStyle;
+    int m_titleFontSize;
+};
+
 // Property sheet dialog
 class SettingsDialog: public wxPropertySheetDialog
 {
-DECLARE_CLASS(SettingsDialog)
+    wxDECLARE_CLASS(SettingsDialog);
 public:
-    SettingsDialog(wxWindow* parent, int dialogType);
+    SettingsDialog(wxWindow* parent, SettingsData& settingsData, int dialogType);
     ~SettingsDialog();
 
     wxPanel* CreateGeneralSettingsPage(wxWindow* parent);
@@ -283,7 +343,9 @@ protected:
 
     wxImageList*    m_imageList;
 
-DECLARE_EVENT_TABLE()
+    SettingsData& m_settingsData;
+
+    wxDECLARE_EVENT_TABLE();
 };
 
 #endif // USE_SETTINGS_DIALOG
@@ -302,6 +364,9 @@ public:
     void MessageBoxWindowModal(wxCommandEvent& event);
     void MessageBoxWindowModalClosed(wxWindowModalDialogEvent& event);
 #endif // wxUSE_MSGDLG
+#if wxUSE_RICHMSGDLG
+    void RichMessageDialog(wxCommandEvent& event);
+#endif // wxUSE_RICHMSGDLG
 
 #if wxUSE_COLOURDLG
     void ChooseColour(wxCommandEvent& event);
@@ -326,9 +391,16 @@ public:
     void MultiChoice(wxCommandEvent& event);
 #endif // wxUSE_CHOICEDLG
 
+#if wxUSE_REARRANGECTRL
     void Rearrange(wxCommandEvent& event);
+#endif // wxUSE_REARRANGECTRL
+
+#if wxUSE_ADDREMOVECTRL
+    void AddRemove(wxCommandEvent& event);
+#endif // wxUSE_ADDREMOVECTRL
 
 #if wxUSE_TEXTDLG
+    void LineEntry(wxCommandEvent& event);
     void TextEntry(wxCommandEvent& event);
     void PasswordEntry(wxCommandEvent& event);
 #endif // wxUSE_TEXTDLG
@@ -374,7 +446,12 @@ public:
 
 #if wxUSE_PROGRESSDLG
     void ShowProgress(wxCommandEvent& event);
+#ifdef wxHAS_NATIVE_PROGRESSDIALOG
+    void ShowProgressGeneric(wxCommandEvent& event);
+#endif // wxHAS_NATIVE_PROGRESSDIALOG
+    void DoShowProgress(wxGenericProgressDialog& dialog);
 #endif // wxUSE_PROGRESSDLG
+    void ShowAppProgress(wxCommandEvent& event);
 
 #if wxUSE_ABOUTDLG
     void ShowSimpleAboutDialog(wxCommandEvent& event);
@@ -385,6 +462,7 @@ public:
 
 #if wxUSE_BUSYINFO
     void ShowBusyInfo(wxCommandEvent& event);
+    void ShowRichBusyInfo(wxCommandEvent& event);
 #endif // wxUSE_BUSYINFO
 
 #if wxUSE_FINDREPLDLG
@@ -405,14 +483,17 @@ public:
 
     void OnRequestUserAttention(wxCommandEvent& event);
 #if wxUSE_NOTIFICATION_MESSAGE
-    void OnNotifMsgAuto(wxCommandEvent& event);
-    void OnNotifMsgShow(wxCommandEvent& event);
-    void OnNotifMsgHide(wxCommandEvent& event);
+    void OnNotifMsg(wxCommandEvent& event);
 #endif // wxUSE_NOTIFICATION_MESSAGE
 
+#if wxUSE_RICHTOOLTIP
+    void OnRichTipDialog(wxCommandEvent& event);
+#endif // wxUSE_RICHTOOLTIP
+
     void OnStandardButtonsSizerDialog(wxCommandEvent& event);
 
     void OnTestDefaultActionDialog(wxCommandEvent& event);
+    void OnModalHook(wxCommandEvent& event);
 
     void OnExit(wxCommandEvent& event);
 
@@ -432,10 +513,6 @@ private:
                         *m_dlgReplace;
 #endif // wxUSE_FINDREPLDLG
 
-#if wxUSE_NOTIFICATION_MESSAGE
-    wxNotificationMessage *m_notifMsg;
-#endif // wxUSE_NOTIFICATION_MESSAGE
-
     wxColourData m_clrData;
 
     // just a window which we use to show the effect of font/colours selection
@@ -448,7 +525,11 @@ private:
               *m_infoBarAdvanced;
 #endif // wxUSE_INFOBAR
 
-    DECLARE_EVENT_TABLE()
+#if USE_SETTINGS_DIALOG
+    SettingsData m_settingsData;
+#endif // USE_SETTINGS_DIALOG
+
+    wxDECLARE_EVENT_TABLE();
 };
 
 class MyCanvas: public wxScrolledWindow
@@ -464,7 +545,7 @@ public:
 private:
     void OnPaint(wxPaintEvent& event);
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 
@@ -472,17 +553,22 @@ private:
 enum
 {
     DIALOGS_CHOOSE_COLOUR = wxID_HIGHEST,
+    DIALOGS_CHOOSE_COLOUR_ALPHA,
     DIALOGS_GET_COLOUR,
     DIALOGS_CHOOSE_COLOUR_GENERIC,
+    DIALOGS_CHOOSE_COLOUR_GENERIC_ALPHA,
     DIALOGS_CHOOSE_FONT,
     DIALOGS_CHOOSE_FONT_GENERIC,
     DIALOGS_MESSAGE_BOX,
     DIALOGS_MESSAGE_BOX_WINDOW_MODAL,
     DIALOGS_MESSAGE_DIALOG,
     DIALOGS_MESSAGE_BOX_WXINFO,
+    DIALOGS_RICH_MESSAGE_DIALOG,
     DIALOGS_SINGLE_CHOICE,
     DIALOGS_MULTI_CHOICE,
     DIALOGS_REARRANGE,
+    DIALOGS_ADDREMOVE,
+    DIALOGS_LINE_ENTRY,
     DIALOGS_TEXT_ENTRY,
     DIALOGS_PASSWORD_ENTRY,
     DIALOGS_FILE_OPEN,
@@ -508,22 +594,25 @@ enum
     DIALOGS_ONTOP,
     DIALOGS_MODELESS_BTN,
     DIALOGS_PROGRESS,
+    DIALOGS_PROGRESS_GENERIC,
+    DIALOGS_APP_PROGRESS,
     DIALOGS_ABOUTDLG_SIMPLE,
     DIALOGS_ABOUTDLG_FANCY,
     DIALOGS_ABOUTDLG_FULL,
     DIALOGS_ABOUTDLG_CUSTOM,
     DIALOGS_BUSYINFO,
+    DIALOGS_BUSYINFO_RICH,
     DIALOGS_FIND,
     DIALOGS_REPLACE,
     DIALOGS_REQUEST,
-    DIALOGS_NOTIFY_AUTO,
-    DIALOGS_NOTIFY_SHOW,
-    DIALOGS_NOTIFY_HIDE,
+    DIALOGS_NOTIFY_MSG,
+    DIALOGS_RICHTIP_DIALOG,
     DIALOGS_PROPERTY_SHEET,
     DIALOGS_PROPERTY_SHEET_TOOLBOOK,
     DIALOGS_PROPERTY_SHEET_BUTTONTOOLBOOK,
     DIALOGS_STANDARD_BUTTON_SIZER_DIALOG,
-    DIALOGS_TEST_DEFAULT_ACTION
+    DIALOGS_TEST_DEFAULT_ACTION,
+    DIALOGS_MODAL_HOOK
 };
 
 #endif
