@@ -178,6 +178,7 @@ wxBEGIN_EVENT_TABLE( GridFrame, wxFrame )
     EVT_MENU( ID_DELETEROW, GridFrame::DeleteSelectedRows )
     EVT_MENU( ID_DELETECOL, GridFrame::DeleteSelectedCols )
     EVT_MENU( ID_CLEARGRID, GridFrame::ClearGrid )
+    EVT_MENU( ID_SHOWSEL,   GridFrame::ShowSelection )
     EVT_MENU( ID_SELCELLS,  GridFrame::SelectCells )
     EVT_MENU( ID_SELROWS,  GridFrame::SelectRows )
     EVT_MENU( ID_SELCOLS,  GridFrame::SelectCols )
@@ -366,7 +367,9 @@ GridFrame::GridFrame()
     selectMenu->Append( ID_DESELECT_ROW, wxT("Deselect row 2"));
     selectMenu->Append( ID_DESELECT_COL, wxT("Deselect col 2"));
     selectMenu->Append( ID_DESELECT_CELL, wxT("Deselect cell (3, 1)"));
+    selectMenu->AppendSeparator();
     wxMenu *selectionMenu = new wxMenu;
+    selectMenu->Append( ID_SHOWSEL, "&Show current selection\tCtrl-S" );
     selectMenu->Append( ID_CHANGESEL, wxT("Change &selection mode"),
                       selectionMenu,
                       wxT("Change selection mode") );
@@ -1040,6 +1043,36 @@ void GridFrame::ClearGrid( wxCommandEvent& WXUNUSED(ev) )
     grid->ClearGrid();
 }
 
+void GridFrame::ShowSelection( wxCommandEvent& WXUNUSED(ev) )
+{
+    switch ( grid->GetSelectionMode() )
+    {
+        case wxGrid::wxGridSelectCells:
+            wxLogMessage("%zu individual cells and "
+                         "%zu blocks of contiguous cells selected",
+                         grid->GetSelectedCells().size(),
+                         grid->GetSelectionBlockTopLeft().size());
+            return;
+
+        case wxGrid::wxGridSelectRows:
+        case wxGrid::wxGridSelectColumns:
+        case wxGrid::wxGridSelectRowsOrColumns:
+            const wxArrayInt& rows = grid->GetSelectedRows();
+            if ( !rows.empty() )
+                wxLogMessage("%zu rows selected", rows.size());
+
+            const wxArrayInt& cols = grid->GetSelectedCols();
+            if ( !cols.empty() )
+                wxLogMessage("%zu columns selected", rows.size());
+
+            if ( rows.empty() && cols.empty() )
+                wxLogMessage("No selection");
+            return;
+    }
+
+    wxLogError("Unknown grid selection mode.");
+}
+
 void GridFrame::SelectCells( wxCommandEvent& WXUNUSED(ev) )
 {
     grid->SetSelectionMode( wxGrid::wxGridSelectCells );
@@ -1342,7 +1375,7 @@ void GridFrame::About(  wxCommandEvent& WXUNUSED(ev) )
     // work with it for some reason) is moved over it.
     aboutInfo.SetWebSite(wxT("http://www.wxwidgets.org"));
 
-    wxAboutBox(aboutInfo);
+    wxAboutBox(aboutInfo, this);
 }
 
 
@@ -2199,7 +2232,7 @@ void GridFrame::OnGridRender( wxCommandEvent& event )
     int styleRender = 0, i;
     bool useLometric = false, defSize = false;
     double zoom = 1;
-    wxSize sizeMargin( 0, 0 );
+    wxSize sizeOffset( 0, 0 );
     wxPoint pointOrigin( 0, 0 );
 
     wxMenu* menu = GetMenuBar()->GetMenu( 0 );
@@ -2222,7 +2255,7 @@ void GridFrame::OnGridRender( wxCommandEvent& event )
     {
         pointOrigin.x += 50;
         pointOrigin.y += 50;
-        sizeMargin.IncBy( 50 );
+        sizeOffset.IncBy( 50 );
     }
     if ( menu->FindItem( ID_RENDER_ZOOM )->IsChecked() )
         zoom = 1.25;
@@ -2277,26 +2310,19 @@ void GridFrame::OnGridRender( wxCommandEvent& event )
     sizeRender.y *= zoom;
 
     // delete any existing render frame and create new one
-    wxWindow* win = FindWindow( "frameRender" );
+    wxWindow* win = FindWindowByName( "frameRender" );
     if ( win )
         win->Destroy();
 
+    // create a frame large enough for the rendered bitmap
     wxFrame* frame = new wxFrame( this, wxID_ANY, "Grid Render" );
-    frame->SetClientSize( 780, 400 );
+    frame->SetClientSize( sizeRender + sizeOffset * 2 );
     frame->SetName( "frameRender" );
 
     wxPanel* canvas = new wxPanel( frame, wxID_ANY );
 
-    // make bitmap large enough for margins if any
-    if ( !defSize )
-        sizeRender.IncBy( sizeMargin * 2 );
-    else
-        sizeRender.IncBy( sizeMargin );
-
-    wxBitmap bmp( sizeRender );
-    // don't leave it larger or drawing will be scaled
-    sizeRender.DecBy( sizeMargin * 2 );
-
+    // make a bitmap large enough for any top/left offset
+    wxBitmap bmp( sizeRender + sizeOffset );
     wxMemoryDC memDc(bmp);
 
     // default row labels have no background colour so set background
