@@ -4,7 +4,6 @@
 // Author:      Karsten Ballueder
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id$
 // Copyright:   (c) Karsten Ballueder, Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -76,10 +75,10 @@
 #endif
 
 // ----------------------------------------------------------------------------
-// ressources
+// resources
 // ----------------------------------------------------------------------------
 // the application icon
-#if !defined(__WXMSW__) && !defined(__WXPM__)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -97,10 +96,10 @@ public:
     // this one is called on application startup and is a good place for the app
     // initialization (doing it here and not in the ctor allows to have an error
     // return: if OnInit() returns false, the application terminates)
-    virtual bool OnInit();
+    virtual bool OnInit() wxOVERRIDE;
 
     // do some clean up here
-    virtual int OnExit();
+    virtual int OnExit() wxOVERRIDE;
 };
 
 // Define a new frame type: this is going to be our main frame
@@ -163,7 +162,7 @@ private:
 #endif
 
     // any class wishing to process wxWidgets events must use this macro
-   DECLARE_EVENT_TABLE()
+   wxDECLARE_EVENT_TABLE();
 };
 
 // A custom modal dialog
@@ -174,7 +173,7 @@ public:
 
 private:
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 // ----------------------------------------------------------------------------
@@ -233,7 +232,7 @@ enum
 // the event tables connect the wxWidgets events with the functions (event
 // handlers) which process them. It can be also done at run-time, but for the
 // simple menu events like this the static method is much simpler.
-BEGIN_EVENT_TABLE(MyFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(HelpDemo_Quit,  MyFrame::OnQuit)
     EVT_MENU(HelpDemo_Help_Index, MyFrame::OnHelp)
     EVT_MENU(HelpDemo_Help_Classes, MyFrame::OnHelp)
@@ -267,14 +266,14 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(HelpDemo_Help_KDE, MyFrame::OnHelp)
     EVT_MENU(HelpDemo_Help_GNOME, MyFrame::OnHelp)
     EVT_MENU(HelpDemo_Help_Netscape, MyFrame::OnHelp)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // Create a new application object: this macro will allow wxWidgets to create
 // the application object during program execution (it's better than using a
 // static object for many reasons) and also declares the accessor function
 // wxGetApp() which will return the reference of the right type (i.e. MyApp and
 // not wxApp)
-IMPLEMENT_APP(MyApp)
+wxIMPLEMENT_APP(MyApp);
 
 // ============================================================================
 // implementation
@@ -324,7 +323,6 @@ bool MyApp::OnInit()
 #endif // !USE_SIMPLE_HELP_PROVIDER
 
     frame->Show(true);
-    SetTopWindow(frame);
 
     // initialise the help system: this means that we'll use doc.hlp file under
     // Windows and that the HTML docs are in the subdirectory doc for platforms
@@ -461,10 +459,10 @@ MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
     m_embeddedHelpWindow = new wxHtmlHelpWindow;
     // m_embeddedHtmlHelp.UseConfig(config, rootPath); // Can set your own config object here
     m_embeddedHtmlHelp.SetHelpWindow(m_embeddedHelpWindow);
-    
+
     m_embeddedHelpWindow->Create(this,
         wxID_ANY, wxDefaultPosition, GetClientSize(), wxTAB_TRAVERSAL|wxNO_BORDER, wxHF_DEFAULT_STYLE);
-        
+
     m_embeddedHtmlHelp.AddBook(wxFileName(wxT("doc.zip")));
     m_embeddedHtmlHelp.Display(wxT("Introduction"));
 #else
@@ -533,7 +531,7 @@ void MyFrame::OnBestHelp(wxCommandEvent& event)
 #if USE_HTML_HELP
 void MyFrame::OnModalHtmlHelp(wxCommandEvent& WXUNUSED(event))
 {
-    wxHtmlModalHelp modalHelp(this, wxT("doc.zip"), wxT("Introduction"));
+    wxHtmlModalHelp(this, wxT("doc.zip"), wxT("Introduction"));
 }
 #endif
 
@@ -676,8 +674,8 @@ void MyFrame::ShowHelp(int commandId, wxHelpControllerBase& helpController)
 // Demonstrates context-sensitive help
 // ----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(MyModalDialog, wxDialog)
-END_EVENT_TABLE()
+wxBEGIN_EVENT_TABLE(MyModalDialog, wxDialog)
+wxEND_EVENT_TABLE()
 
 MyModalDialog::MyModalDialog(wxWindow *parent)
              : wxDialog(parent, wxID_ANY, wxString(wxT("Modal dialog")))
@@ -708,8 +706,8 @@ MyModalDialog::MyModalDialog(wxWindow *parent)
                                       wxDefaultPosition, wxSize(300, 100),
                                       wxTE_MULTILINE);
     text->SetHelpText(_("Type text here if you have got nothing more interesting to do"));
-    sizerTop->Add(text, 0, wxEXPAND|wxALIGN_CENTER_VERTICAL|wxALL, 5 );
-    sizerTop->Add(sizerRow, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5 );
+    sizerTop->Add(text, 0, wxEXPAND|wxALL, 5 );
+    sizerTop->Add(sizerRow, 0, wxALIGN_RIGHT|wxALL, 5 );
 
     SetSizerAndFit(sizerTop);
 
