@@ -4,7 +4,6 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     04.08.03
-// RCS-ID:      $Id$
 // Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwidgets.org>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -46,8 +45,8 @@
 // resources
 // ----------------------------------------------------------------------------
 
-// the application icon (under Windows and OS/2 it is in resources)
-#if defined(__WXGTK__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__) || defined(__WXX11__)
+// the application icon (under Windows it is in resources)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -67,7 +66,7 @@ public:
                                   int WXUNUSED(flags) = 0,
                                   wxHeaderSortIconType WXUNUSED(sortArrow)
                                     = wxHDR_SORT_ICON_NONE,
-                                  wxHeaderButtonParams* params = NULL)
+                                  wxHeaderButtonParams* params = NULL) wxOVERRIDE
     {
         wxDCBrushChanger setBrush(dc, *wxBLUE_BRUSH);
         wxDCTextColourChanger setFgCol(dc, *wxWHITE);
@@ -87,7 +86,7 @@ public:
 // cases, but we show this here just for completeness)
 class MyTraits : public wxGUIAppTraits
 {
-    virtual wxRendererNative *CreateRenderer()
+    virtual wxRendererNative *CreateRenderer() wxOVERRIDE
     {
         // it will be deleted on program shutdown by wxWidgets itself
         return new MyRenderer;
@@ -98,10 +97,10 @@ class MyTraits : public wxGUIAppTraits
 class MyApp : public wxApp
 {
 public:
-    virtual bool OnInit();
+    virtual bool OnInit() wxOVERRIDE;
 
     // if we want MyTraits to be used we must override CreateTraits()
-    virtual wxAppTraits *CreateTraits() { return new MyTraits; }
+    virtual wxAppTraits *CreateTraits() wxOVERRIDE { return new MyTraits; }
 };
 
 // Define a new frame type: this is going to be our main frame
@@ -124,6 +123,10 @@ private:
         { OnToggleDrawFlag(event, wxCONTROL_CHECKED); }
     void OnDrawHot(wxCommandEvent& event)
         { OnToggleDrawFlag(event, wxCONTROL_CURRENT); }
+    void OnDrawUndetermined(wxCommandEvent &event)
+        { OnToggleDrawFlag(event, wxCONTROL_UNDETERMINED); }
+    void OnDrawSpecial(wxCommandEvent &event)
+        { OnToggleDrawFlag(event, wxCONTROL_SPECIAL); }
 
     void OnAlignLeft(wxCommandEvent& WXUNUSED(event))
         { OnChangeAlign(wxALIGN_LEFT); }
@@ -135,6 +138,7 @@ private:
     void OnUseIcon(wxCommandEvent& event);
     void OnUseBitmap(wxCommandEvent& event);
 
+    void OnUseGeneric(wxCommandEvent& event);
 #if wxUSE_DYNLIB_CLASS
     void OnLoad(wxCommandEvent& event);
     void OnUnload(wxCommandEvent& event);
@@ -148,7 +152,7 @@ private:
     class MyPanel *m_panel;
 
     // any class wishing to process wxWidgets events must use this macro
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 // a very simple class just to have something to draw on
@@ -160,7 +164,8 @@ public:
         m_flags = 0;
         m_align = wxALIGN_LEFT;
         m_useIcon =
-        m_useBitmap = false;
+        m_useBitmap =
+        m_useGeneric = false;
     }
 
     int GetFlags() const { return m_flags; }
@@ -169,13 +174,15 @@ public:
     void SetAlignment(int align) { m_align = align; }
     void SetUseIcon(bool useIcon) { m_useIcon = useIcon; }
     void SetUseBitmap(bool useBitmap) { m_useBitmap = useBitmap; }
+    void SetUseGeneric(bool useGeneric) { m_useGeneric = useGeneric; }
 
 private:
     void OnPaint(wxPaintEvent&)
     {
         wxPaintDC dc(this);
 
-        wxRendererNative& renderer = wxRendererNative::Get();
+        wxRendererNative& renderer = m_useGeneric ? wxRendererNative::GetGeneric()
+                                                  : wxRendererNative::Get();
 
         int x1 = 10,    // text offset
             x2 = 300,   // drawing offset
@@ -195,7 +202,11 @@ private:
             flagsString += "wxCONTROL_CURRENT ";
         if ( m_flags & wxCONTROL_CHECKED )
             flagsString += "wxCONTROL_CHECKED ";
-        if ( flagsString.empty() )
+        if ( m_flags & wxCONTROL_UNDETERMINED )
+            flagsString += "wxCONTROL_UNDETERMINED ";
+        if (m_flags & wxCONTROL_SPECIAL)
+            flagsString += "wxCONTROL_SPECIAL ";
+        if (flagsString.empty())
             flagsString = "(none)";
         dc.DrawText("Using flags: " + flagsString, x1, y);
         y += lineHeight*3;
@@ -216,6 +227,8 @@ private:
                                                                wxART_LIST);
         }
 
+        // Note that we need to use GetDefault() explicitly to show the default
+        // implementation.
         dc.DrawText("DrawHeaderButton() (default)", x1, y);
         wxRendererNative::GetDefault().DrawHeaderButton(this, dc,
                                   wxRect(x2, y, widthHdr, heightHdr), m_flags,
@@ -239,6 +252,12 @@ private:
                                  wxRect(wxPoint(x2, y), sizeCheck), m_flags);
         y += lineHeight + sizeCheck.y;
 
+        dc.DrawText("DrawCollapseButton()", x1, y);
+        const wxSize sizeCollapse = renderer.GetCollapseButtonSize(this, dc);
+        renderer.DrawCollapseButton(this, dc,
+                                    wxRect(wxPoint(x2, y), sizeCollapse), m_flags);
+        y += lineHeight + sizeCollapse.y;
+
         dc.DrawText("DrawTreeItemButton()", x1, y);
         renderer.DrawTreeItemButton(this, dc,
                                     wxRect(x2, y, 20, 20), m_flags);
@@ -264,19 +283,46 @@ private:
 
         y += lineHeight + rBtn.height;
 #endif // wxHAS_DRAW_TITLE_BAR_BITMAP
+
+        // The meanings of those are reversed for the vertical gauge below.
+        const wxCoord heightGauge = 24;
+        const wxCoord widthGauge = 180;
+
+        dc.DrawText("DrawGauge()", x1, y);
+        renderer.DrawGauge(this, dc,
+            wxRect(x2, y, widthGauge, heightGauge), 25, 100, m_flags);
+        renderer.DrawGauge(this, dc,
+            wxRect(x2 + widthGauge + 30, y + heightGauge - widthGauge,
+                   heightGauge, widthGauge),
+            25, 100, m_flags | wxCONTROL_SPECIAL);
+
+        y += lineHeight + heightGauge;
+
+        const wxCoord heightListItem = 48;
+        const wxCoord widthListItem = 260;
+
+        dc.DrawText("DrawItemSelectionRect()", x1, y);
+        renderer.DrawItemSelectionRect(this, dc,
+            wxRect(x2, y, widthListItem, heightListItem), m_flags | wxCONTROL_SELECTED);
+
+        renderer.DrawItemText(this, dc, "DrawItemText()",
+            wxRect(x2, y, widthListItem, heightListItem).Inflate(-2, -2), m_align, m_flags | wxCONTROL_SELECTED);
+
+        y += lineHeight + heightListItem;
     }
 
     int m_flags;
     int m_align;
     bool m_useIcon,
-         m_useBitmap;
+         m_useBitmap,
+         m_useGeneric;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
-BEGIN_EVENT_TABLE(MyPanel, wxPanel)
+wxBEGIN_EVENT_TABLE(MyPanel, wxPanel)
     EVT_PAINT(MyPanel::OnPaint)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // ----------------------------------------------------------------------------
 // constants
@@ -291,6 +337,8 @@ enum
     Render_DrawPressed,
     Render_DrawChecked,
     Render_DrawHot,
+    Render_DrawUndetermined,
+    Render_DrawSpecial,
 
     Render_AlignLeft,
     Render_AlignCentre,
@@ -299,6 +347,7 @@ enum
     Render_UseIcon,
     Render_UseBitmap,
 
+    Render_UseGeneric,
 #if wxUSE_DYNLIB_CLASS
     Render_Load,
     Render_Unload,
@@ -320,13 +369,14 @@ enum
 // the event tables connect the wxWidgets events with the functions (event
 // handlers) which process them. It can be also done at run-time, but for the
 // simple menu events like this the static method is much simpler.
-BEGIN_EVENT_TABLE(MyFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(Render_DrawDisabled, MyFrame::OnDrawDisabled)
     EVT_MENU(Render_DrawFocused, MyFrame::OnDrawFocused)
     EVT_MENU(Render_DrawPressed, MyFrame::OnDrawPressed)
     EVT_MENU(Render_DrawChecked, MyFrame::OnDrawChecked)
     EVT_MENU(Render_DrawHot, MyFrame::OnDrawHot)
-
+    EVT_MENU(Render_DrawUndetermined, MyFrame::OnDrawUndetermined)
+    EVT_MENU(Render_DrawSpecial, MyFrame::OnDrawSpecial)
     EVT_MENU(Render_AlignLeft, MyFrame::OnAlignLeft)
     EVT_MENU(Render_AlignCentre, MyFrame::OnAlignCentre)
     EVT_MENU(Render_AlignRight, MyFrame::OnAlignRight)
@@ -334,6 +384,7 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(Render_UseIcon, MyFrame::OnUseIcon)
     EVT_MENU(Render_UseBitmap, MyFrame::OnUseBitmap)
 
+    EVT_MENU(Render_UseGeneric, MyFrame::OnUseGeneric)
 #if wxUSE_DYNLIB_CLASS
     EVT_MENU(Render_Load,  MyFrame::OnLoad)
     EVT_MENU(Render_Unload,MyFrame::OnUnload)
@@ -341,14 +392,14 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(Render_Quit,  MyFrame::OnQuit)
 
     EVT_MENU(Render_About, MyFrame::OnAbout)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // Create a new application object: this macro will allow wxWidgets to create
 // the application object during program execution (it's better than using a
 // static object for many reasons) and also implements the accessor function
 // wxGetApp() which will return the reference of the right type (i.e. MyApp and
 // not wxApp)
-IMPLEMENT_APP(MyApp)
+wxIMPLEMENT_APP(MyApp);
 
 // ============================================================================
 // implementation
@@ -385,9 +436,7 @@ bool MyApp::OnInit()
 MyFrame::MyFrame()
        : wxFrame(NULL,
                  wxID_ANY,
-                 wxT("Render wxWidgets Sample"),
-                 wxPoint(50, 50),
-                 wxSize(450, 340))
+                 wxT("Render wxWidgets Sample"))
 {
     // set the frame icon
     SetIcon(wxICON(sample));
@@ -405,6 +454,10 @@ MyFrame::MyFrame()
                               "Draw in &checked state\tCtrl-C");
     menuFile->AppendCheckItem(Render_DrawHot,
                               "Draw in &hot state\tCtrl-H");
+    menuFile->AppendCheckItem(Render_DrawUndetermined,
+                              "Draw in unde&termined state\tCtrl-T");
+    menuFile->AppendCheckItem(Render_DrawSpecial,
+                              "Draw in &special state\tCtrl-S");
     menuFile->AppendSeparator();
 
     menuFile->AppendRadioItem(Render_AlignLeft, "&Left align\tCtrl-1");
@@ -416,6 +469,7 @@ MyFrame::MyFrame()
     menuFile->AppendCheckItem(Render_UseBitmap, "Draw &bitmap\tCtrl-B");
     menuFile->AppendSeparator();
 
+    menuFile->AppendCheckItem(Render_UseGeneric, "Use &generic renderer\tCtrl-G");
 #if wxUSE_DYNLIB_CLASS
     menuFile->Append(Render_Load, wxT("&Load renderer...\tCtrl-L"));
     menuFile->Append(Render_Unload, wxT("&Unload renderer\tCtrl-U"));
@@ -438,6 +492,8 @@ MyFrame::MyFrame()
 
     m_panel = new MyPanel(this);
 
+    SetClientSize(600, 600);
+
 #if wxUSE_STATUSBAR
     // create a status bar just for fun (by default with 1 pane only)
     CreateStatusBar(2);
@@ -485,6 +541,12 @@ void MyFrame::OnUseBitmap(wxCommandEvent& event)
     m_panel->Refresh();
 }
 
+void MyFrame::OnUseGeneric(wxCommandEvent& event)
+{
+    m_panel->SetUseGeneric(event.IsChecked());
+    m_panel->Refresh();
+}
+
 #if wxUSE_DYNLIB_CLASS
 
 void MyFrame::OnLoad(wxCommandEvent& WXUNUSED(event))
