@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -25,7 +24,7 @@
 
 #include "typetest.h"
 
-#if !defined(__WXMSW__) && !defined(__WXPM__)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -47,11 +46,11 @@
 #include "wx/mstream.h"
 
 // Create a new application object
-IMPLEMENT_APP    (MyApp)
+wxIMPLEMENT_APP(MyApp);
 
-IMPLEMENT_DYNAMIC_CLASS    (MyApp, wxApp)
+wxIMPLEMENT_DYNAMIC_CLASS(MyApp, wxApp);
 
-BEGIN_EVENT_TABLE(MyApp, wxApp)
+wxBEGIN_EVENT_TABLE(MyApp, wxApp)
     EVT_MENU(TYPES_VARIANT,   MyApp::DoVariantDemo)
     EVT_MENU(TYPES_BYTEORDER, MyApp::DoByteOrderDemo)
 #if wxUSE_UNICODE
@@ -65,7 +64,7 @@ BEGIN_EVENT_TABLE(MyApp, wxApp)
     EVT_MENU(TYPES_STREAM6, MyApp::DoStreamDemo6)
     EVT_MENU(TYPES_STREAM7, MyApp::DoStreamDemo7)
     EVT_MENU(TYPES_MIME, MyApp::DoMIMEDemo)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 wxString file_name = wxT("test_wx.dat");
 wxString file_name2 = wxString(wxT("test_wx2.dat"));
@@ -116,8 +115,6 @@ bool MyApp::OnInit()
     // Show the frame
     frame->Show(true);
 
-    SetTopWindow(frame);
-
     return true;
 }
 
@@ -894,10 +891,10 @@ void MyApp::DoUnicodeDemo(wxCommandEvent& WXUNUSED(event))
 
     printf( "\n\nConversion with wxConvLocal:\n" );
     wxConvCurrent = &wxConvLocal;
-    printf( (const char*) str.mbc_str() );
+    puts( str.mbc_str() );
     printf( "\n\nConversion with wxConvLibc:\n" );
     wxConvCurrent = &wxConvLibc;
-    printf( (const char*) str.mbc_str() );
+    puts( str.mbc_str() );
 
 }
 #endif
@@ -948,8 +945,11 @@ void MyApp::DoMIMEDemo(wxCommandEvent& WXUNUSED(event))
         else
         {
             wxString type, desc, open;
+            wxIconLocation loc;
+
             filetype->GetMimeType(&type);
             filetype->GetDescription(&desc);
+            filetype->GetIcon(&loc);
 
             wxString filename = wxT("filename");
             filename << wxT(".") << ext;
@@ -961,6 +961,8 @@ void MyApp::DoMIMEDemo(wxCommandEvent& WXUNUSED(event))
                      << wxT("\tDescription: ") << ( !desc ? wxString(wxEmptyString) : desc )
                         << wxT('\n')
                      << wxT("\tCommand to open: ") << ( !open ? wxString("no") : open )
+                        << wxT('\n')
+                     << wxT("\tIcon: ") << ( loc.IsOk() ? loc.GetFileName() : wxString("no") )
                         << wxT('\n');
 
             delete filetype;
@@ -1047,7 +1049,8 @@ void MyApp::DoVariantDemo(wxCommandEvent& WXUNUSED(event) )
         textCtrl << wxT("var1[") << (int) i << wxT("] (type ") << var1[i].GetType() << wxT(") = ") << var1[i].MakeString() << wxT("\n");
     }
 
-    var1 = wxVariant(new wxFont(wxSystemSettings::GetFont(wxSYS_OEM_FIXED_FONT)));
+    wxFont* sysFont = new wxFont(wxSystemSettings::GetFont(wxSYS_OEM_FIXED_FONT));
+    var1 = wxVariant(sysFont);
     textCtrl << wxT("var1 = (wxfont)\"");
     wxFont* font = wxGetVariantCast(var1,wxFont);
     if (font)
@@ -1058,12 +1061,14 @@ void MyApp::DoVariantDemo(wxCommandEvent& WXUNUSED(event) )
     {
         textCtrl << wxT("(null)\"\n");
     }
+
+    delete sysFont;
 }
 
-BEGIN_EVENT_TABLE(MyFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(TYPES_QUIT, MyFrame::OnQuit)
     EVT_MENU(TYPES_ABOUT, MyFrame::OnAbout)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // My frame constructor
 MyFrame::MyFrame(wxFrame *parent, const wxString& title,
