@@ -4,7 +4,6 @@
 // Purpose:     Part of the widgets sample showing various static controls
 // Author:      Vadim Zeitlin
 // Created:     11.04.01
-// Id:          $Id$
 // Copyright:   (c) 2001 Vadim Zeitlin
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -41,6 +40,7 @@
 
 #include "wx/statline.h"
 #include "wx/generic/stattextg.h"
+#include "wx/wupdlock.h"
 
 #include "widgets.h"
 #include "icons/statbox.xpm"
@@ -93,20 +93,39 @@ public:
     StaticWidgetsPage(WidgetsBookCtrl *book, wxImageList *imaglist);
     virtual ~StaticWidgetsPage(){};
 
-    virtual wxControl *GetWidget() const { return m_statText; }
-    virtual void RecreateWidget() { CreateStatic(); }
+    virtual wxWindow *GetWidget() const wxOVERRIDE { return m_statText; }
+    virtual Widgets GetWidgets() const wxOVERRIDE
+    {
+        Widgets widgets;
+        widgets.push_back(m_sizerStatBox->GetStaticBox());
+        widgets.push_back(m_statText);
+#if wxUSE_MARKUP
+        widgets.push_back(m_statMarkup);
+#endif // wxUSE_MARKUP
+#if wxUSE_STATLINE
+        widgets.push_back(m_statLine);
+#endif // wxUSE_STATLINE
+
+        return widgets;
+    }
+    virtual void RecreateWidget() wxOVERRIDE { CreateStatic(); }
 
     // lazy creation of the content
-    virtual void CreateContent();
+    virtual void CreateContent() wxOVERRIDE;
 
 protected:
     // event handlers
     void OnCheckOrRadioBox(wxCommandEvent& event);
+#ifdef wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+    void OnBoxCheckBox(wxCommandEvent& event);
+#endif // wxHAS_WINDOW_LABEL_IN_STATIC_BOX
 
     void OnButtonReset(wxCommandEvent& event);
     void OnButtonBoxText(wxCommandEvent& event);
     void OnButtonLabelText(wxCommandEvent& event);
+#if wxUSE_MARKUP
     void OnButtonLabelWithMarkupText(wxCommandEvent& event);
+#endif // wxUSE_MARKUP
     void OnMouseEvent(wxMouseEvent& event);
 
     // reset all parameters
@@ -121,10 +140,16 @@ protected:
     // the check/radio boxes for styles
     wxCheckBox *m_chkVert,
                *m_chkGeneric,
+#ifdef wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+               *m_chkBoxWithCheck,
+#endif // wxHAS_WINDOW_LABEL_IN_STATIC_BOX
                *m_chkAutoResize,
-               *m_chkEllipsize,
-               *m_chkMarkup,
+               *m_chkEllipsize;
+
+#if wxUSE_MARKUP
+    wxCheckBox *m_chkMarkup,
                *m_chkGreen;
+#endif // wxUSE_MARKUP
 
     wxRadioBox *m_radioHAlign,
                *m_radioVAlign,
@@ -132,8 +157,12 @@ protected:
 
     // the controls and the sizer containing them
     wxStaticBoxSizer *m_sizerStatBox;
-    wxStaticTextBase *m_statText,
-                     *m_statMarkup;
+    wxStaticTextBase *m_statText;
+
+#if wxUSE_MARKUP
+    wxStaticTextBase *m_statMarkup;
+#endif // wxUSE_MARKUP
+
 #if wxUSE_STATLINE
     wxStaticLine *m_statLine;
 #endif // wxUSE_STATLINE
@@ -141,11 +170,14 @@ protected:
 
     // the text entries for command parameters
     wxTextCtrl *m_textBox,
-               *m_textLabel,
-               *m_textLabelWithMarkup;
+               *m_textLabel;
+
+#if wxUSE_MARKUP
+    wxTextCtrl *m_textLabelWithMarkup;
+#endif // wxUSE_MARKUP
 
 private:
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
     DECLARE_WIDGETS_PAGE(StaticWidgetsPage)
 };
 
@@ -153,15 +185,17 @@ private:
 // event tables
 // ----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(StaticWidgetsPage, WidgetsPage)
+wxBEGIN_EVENT_TABLE(StaticWidgetsPage, WidgetsPage)
     EVT_BUTTON(StaticPage_Reset, StaticWidgetsPage::OnButtonReset)
     EVT_BUTTON(StaticPage_LabelText, StaticWidgetsPage::OnButtonLabelText)
+#if wxUSE_MARKUP
     EVT_BUTTON(StaticPage_LabelTextWithMarkup, StaticWidgetsPage::OnButtonLabelWithMarkupText)
+#endif // wxUSE_MARKUP
     EVT_BUTTON(StaticPage_BoxText, StaticWidgetsPage::OnButtonBoxText)
 
     EVT_CHECKBOX(wxID_ANY, StaticWidgetsPage::OnCheckOrRadioBox)
     EVT_RADIOBOX(wxID_ANY, StaticWidgetsPage::OnCheckOrRadioBox)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // ============================================================================
 // implementation
@@ -177,22 +211,36 @@ StaticWidgetsPage::StaticWidgetsPage(WidgetsBookCtrl *book,
 {
     // init everything
     m_chkVert =
-    m_chkAutoResize = (wxCheckBox *)NULL;
-    m_chkGeneric = NULL;
-    m_chkGreen = NULL;
+    m_chkAutoResize =
+    m_chkGeneric =
+#ifdef wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+    m_chkBoxWithCheck =
+#endif // wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+#if wxUSE_MARKUP
+    m_chkGreen =
+#endif // wxUSE_MARKUP
+                NULL;
 
     m_radioHAlign =
     m_radioVAlign = (wxRadioBox *)NULL;
 
+    m_statText = NULL;
 #if wxUSE_STATLINE
     m_statLine = (wxStaticLine *)NULL;
 #endif // wxUSE_STATLINE
-    m_statText = m_statMarkup = NULL;
+#if wxUSE_MARKUP
+    m_statMarkup = NULL;
+#endif // wxUSE_MARKUP
 
     m_sizerStatBox = (wxStaticBoxSizer *)NULL;
     m_sizerStatic = (wxSizer *)NULL;
 
-    m_textBox = m_textLabel = m_textLabelWithMarkup = NULL;
+    m_textBox =
+    m_textLabel =
+#if wxUSE_MARKUP
+    m_textLabelWithMarkup =
+#endif // wxUSE_MARKUP
+                            NULL;
 }
 
 void StaticWidgetsPage::CreateContent()
@@ -204,7 +252,9 @@ void StaticWidgetsPage::CreateContent()
 
     m_chkGeneric = CreateCheckBoxAndAddToSizer(sizerLeft,
                                                "&Generic wxStaticText");
-    m_chkMarkup = CreateCheckBoxAndAddToSizer(sizerLeft, "Support &markup");
+#ifdef wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+    m_chkBoxWithCheck = CreateCheckBoxAndAddToSizer(sizerLeft, "Checkable &box");
+#endif // wxHAS_WINDOW_LABEL_IN_STATIC_BOX
     m_chkVert = CreateCheckBoxAndAddToSizer(sizerLeft, "&Vertical line");
     m_chkAutoResize = CreateCheckBoxAndAddToSizer(sizerLeft, "&Fit to text");
     sizerLeft->Add(5, 5, 0, wxGROW | wxALL, 5); // spacer
@@ -261,7 +311,7 @@ void StaticWidgetsPage::CreateContent()
 
     m_textBox = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
     wxButton *b1 = new wxButton(this, wxID_ANY, "Change &box label");
-    b1->Connect(wxEVT_COMMAND_BUTTON_CLICKED,
+    b1->Connect(wxEVT_BUTTON,
                 wxCommandEventHandler(StaticWidgetsPage::OnButtonBoxText),
                 NULL, this);
     sizerMiddle->Add(m_textBox, 0, wxEXPAND|wxALL, 5);
@@ -271,38 +321,41 @@ void StaticWidgetsPage::CreateContent()
                                  wxDefaultPosition, wxDefaultSize,
                                  wxTE_MULTILINE|wxHSCROLL);
     wxButton *b2 = new wxButton(this, wxID_ANY, "Change &text label");
-    b2->Connect(wxEVT_COMMAND_BUTTON_CLICKED,
+    b2->Connect(wxEVT_BUTTON,
                 wxCommandEventHandler(StaticWidgetsPage::OnButtonLabelText),
                 NULL, this);
     sizerMiddle->Add(m_textLabel, 0, wxEXPAND|wxALL, 5);
     sizerMiddle->Add(b2, 0, wxLEFT|wxBOTTOM, 5);
 
+#if wxUSE_MARKUP
     m_textLabelWithMarkup = new wxTextCtrl(this, wxID_ANY, wxEmptyString,
                                            wxDefaultPosition, wxDefaultSize,
                                            wxTE_MULTILINE|wxHSCROLL);
 
     wxButton *b3 = new wxButton(this, wxID_ANY, "Change decorated text label");
-    b3->Connect(wxEVT_COMMAND_BUTTON_CLICKED,
+    b3->Connect(wxEVT_BUTTON,
                 wxCommandEventHandler(StaticWidgetsPage::OnButtonLabelWithMarkupText),
                 NULL, this);
     sizerMiddle->Add(m_textLabelWithMarkup, 0, wxEXPAND|wxALL, 5);
     sizerMiddle->Add(b3, 0, wxLEFT|wxBOTTOM, 5);
 
-    m_chkGreen = CreateCheckBoxAndAddToSizer(sizerLeft,
+    m_chkGreen = CreateCheckBoxAndAddToSizer(sizerMiddle,
                                              "Decorated label on g&reen");
-    sizerMiddle->Add(m_chkGreen, 0, wxALL, 5);
+#endif // wxUSE_MARKUP
 
     // final initializations
     // NB: must be done _before_ calling CreateStatic()
     Reset();
 
-    m_textBox->SetValue(wxT("This is a box"));
+    m_textBox->SetValue(wxT("This is a &box"));
     m_textLabel->SetValue(wxT("And this is a\n\tlabel inside the box with a &mnemonic.\n")
                           wxT("Only this text is affected by the ellipsize settings."));
+#if wxUSE_MARKUP
     m_textLabelWithMarkup->SetValue(wxT("Another label, this time <b>decorated</b> ")
                                     wxT("with <u>markup</u>; here you need entities ")
                                     wxT("for the symbols: &lt; &gt; &amp; &apos; &quot; ")
                                     wxT(" but you can still place &mnemonics..."));
+#endif // wxUSE_MARKUP
 
     // right pane
     wxSizer *sizerRight = new wxBoxSizer(wxHORIZONTAL);
@@ -326,10 +379,12 @@ void StaticWidgetsPage::CreateContent()
 void StaticWidgetsPage::Reset()
 {
     m_chkGeneric->SetValue(false);
+#ifdef wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+    m_chkBoxWithCheck->SetValue(false);
+#endif // wxHAS_WINDOW_LABEL_IN_STATIC_BOX
     m_chkVert->SetValue(false);
     m_chkAutoResize->SetValue(true);
     m_chkEllipsize->SetValue(true);
-    m_chkMarkup->SetValue(true);
 
     m_radioHAlign->SetSelection(StaticHAlign_Left);
     m_radioVAlign->SetSelection(StaticVAlign_Top);
@@ -337,6 +392,8 @@ void StaticWidgetsPage::Reset()
 
 void StaticWidgetsPage::CreateStatic()
 {
+    wxWindowUpdateLocker lock(this);
+
     bool isVert = m_chkVert->GetValue();
 
     if ( m_sizerStatBox )
@@ -344,15 +401,17 @@ void StaticWidgetsPage::CreateStatic()
         // delete m_sizerStatBox; -- deleted by Remove()
         m_sizerStatic->Remove(m_sizerStatBox);
         delete m_statText;
+#if wxUSE_MARKUP
         delete m_statMarkup;
+#endif // wxUSE_MARKUP
 #if wxUSE_STATLINE
         delete m_statLine;
 #endif // wxUSE_STATLINE
     }
 
     int flagsBox = 0,
-        flagsText = ms_defaultFlags,
-        flagsDummyText = ms_defaultFlags;
+        flagsText = GetAttrs().m_defaultFlags,
+        flagsDummyText = GetAttrs().m_defaultFlags;
 
     if ( !m_chkAutoResize->GetValue() )
     {
@@ -360,12 +419,6 @@ void StaticWidgetsPage::CreateStatic()
         flagsDummyText |= wxST_NO_AUTORESIZE;
     }
 
-    if ( m_chkMarkup->GetValue() )
-    {
-        flagsText |= wxST_MARKUP;
-        flagsDummyText |= wxST_MARKUP;
-    }
-
     int align = 0;
     switch ( m_radioHAlign->GetSelection() )
     {
@@ -431,50 +484,82 @@ void StaticWidgetsPage::CreateStatic()
     flagsText |= align;
     flagsBox |= align;
 
-    wxStaticBox *staticBox = new wxStaticBox(this, wxID_ANY,
-                                             m_textBox->GetValue(),
-                                             wxDefaultPosition, wxDefaultSize,
-                                             flagsBox);
+    wxStaticBox *staticBox;
+#ifdef wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+    if ( m_chkBoxWithCheck->GetValue() )
+    {
+        wxCheckBox* const label = new wxCheckBox(this, wxID_ANY,
+                                                 m_textBox->GetValue());
+        label->Bind(wxEVT_CHECKBOX, &StaticWidgetsPage::OnBoxCheckBox, this);
+
+        staticBox = new wxStaticBox(this, wxID_ANY,
+                                    label,
+                                    wxDefaultPosition, wxDefaultSize,
+                                    flagsBox);
+    }
+    else // normal static box
+#endif // wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+    {
+        staticBox = new wxStaticBox(this, wxID_ANY,
+                                    m_textBox->GetValue(),
+                                    wxDefaultPosition, wxDefaultSize,
+                                    flagsBox);
+    }
+
     m_sizerStatBox = new wxStaticBoxSizer(staticBox, isVert ? wxHORIZONTAL
                                                             : wxVERTICAL);
 
     if ( m_chkGeneric->GetValue() )
     {
-        m_statText = new wxGenericStaticText(this, wxID_ANY,
+        m_statText = new wxGenericStaticText(staticBox, wxID_ANY,
                                              m_textLabel->GetValue(),
                                              wxDefaultPosition, wxDefaultSize,
                                              flagsDummyText);
-        m_statMarkup = new wxGenericStaticText(this, wxID_ANY,
-                                             m_textLabelWithMarkup->GetValue(),
+#if wxUSE_MARKUP
+        m_statMarkup = new wxGenericStaticText(staticBox, wxID_ANY,
+                                             wxString(),
                                              wxDefaultPosition, wxDefaultSize,
                                              flagsText);
+#endif // wxUSE_MARKUP
     }
     else // use native versions
     {
-        m_statText = new wxStaticText(this, wxID_ANY,
+        m_statText = new wxStaticText(staticBox, wxID_ANY,
                                       m_textLabel->GetValue(),
                                       wxDefaultPosition, wxDefaultSize,
                                       flagsDummyText);
-        m_statMarkup = new wxStaticText(this, wxID_ANY,
-                                        m_textLabelWithMarkup->GetValue(),
+#if wxUSE_MARKUP
+        m_statMarkup = new wxStaticText(staticBox, wxID_ANY,
+                                        wxString(),
                                         wxDefaultPosition, wxDefaultSize,
                                         flagsText);
+#endif // wxUSE_MARKUP
     }
+
+    m_statText->SetToolTip("Tooltip for a label inside the box");
+
+#if wxUSE_MARKUP
+    m_statMarkup->SetLabelMarkup(m_textLabelWithMarkup->GetValue());
+
     if ( m_chkGreen->GetValue() )
         m_statMarkup->SetBackgroundColour(*wxGREEN);
+#endif // wxUSE_MARKUP
+
 #if wxUSE_STATLINE
-    m_statLine = new wxStaticLine(this, wxID_ANY,
+    m_statLine = new wxStaticLine(staticBox, wxID_ANY,
                                   wxDefaultPosition, wxDefaultSize,
                                   isVert ? wxLI_VERTICAL : wxLI_HORIZONTAL);
 #endif // wxUSE_STATLINE
 
-    m_sizerStatBox->Add(m_statText, 1, wxGROW | wxALL, 5);
+    m_sizerStatBox->Add(m_statText, 0, wxGROW);
 #if wxUSE_STATLINE
-    m_sizerStatBox->Add(m_statLine, 0, wxGROW | wxALL, 5);
+    m_sizerStatBox->Add(m_statLine, 0, wxGROW | wxTOP | wxBOTTOM, 10);
 #endif // wxUSE_STATLINE
-    m_sizerStatBox->Add(m_statMarkup, 1, wxGROW | wxALL, 5);
+#if wxUSE_MARKUP
+    m_sizerStatBox->Add(m_statMarkup);
+#endif // wxUSE_MARKUP
 
-    m_sizerStatic->Add(m_sizerStatBox, 1, wxGROW);
+    m_sizerStatic->Add(m_sizerStatBox, 0, wxGROW);
 
     m_sizerStatic->Layout();
 
@@ -484,6 +569,8 @@ void StaticWidgetsPage::CreateStatic()
     staticBox->Connect(wxEVT_LEFT_UP,
                        wxMouseEventHandler(StaticWidgetsPage::OnMouseEvent),
                        NULL, this);
+
+    SetUpWidget();
 }
 
 // ----------------------------------------------------------------------------
@@ -507,6 +594,14 @@ void StaticWidgetsPage::OnCheckOrRadioBox(wxCommandEvent& event)
     CreateStatic();
 }
 
+#ifdef wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+void StaticWidgetsPage::OnBoxCheckBox(wxCommandEvent& event)
+{
+    wxLogMessage("Box check box has been %schecked",
+                 event.IsChecked() ? "": "un");
+}
+#endif // wxHAS_WINDOW_LABEL_IN_STATIC_BOX
+
 void StaticWidgetsPage::OnButtonBoxText(wxCommandEvent& WXUNUSED(event))
 {
     m_sizerStatBox->GetStaticBox()->SetLabel(m_textBox->GetValue());
@@ -525,9 +620,10 @@ void StaticWidgetsPage::OnButtonLabelText(wxCommandEvent& WXUNUSED(event))
                  m_statText->GetLabelText());
 }
 
+#if wxUSE_MARKUP
 void StaticWidgetsPage::OnButtonLabelWithMarkupText(wxCommandEvent& WXUNUSED(event))
 {
-    m_statMarkup->SetLabel(m_textLabelWithMarkup->GetValue());
+    m_statMarkup->SetLabelMarkup(m_textLabelWithMarkup->GetValue());
 
     // test GetLabel() and GetLabelText(); the first should return the
     // label as it is written in the relative text control; the second should
@@ -537,6 +633,7 @@ void StaticWidgetsPage::OnButtonLabelWithMarkupText(wxCommandEvent& WXUNUSED(eve
     wxLogMessage(wxT("The label text is '%s'"),
                  m_statMarkup->GetLabelText());
 }
+#endif // wxUSE_MARKUP
 
 void StaticWidgetsPage::OnMouseEvent(wxMouseEvent& event)
 {
