@@ -4,9 +4,8 @@
 // Purpose:     part of the widgets sample showing wxTextCtrl
 // Author:      Vadim Zeitlin
 // Created:     27.03.01
-// Id:          $Id: textctrl.cpp,v 1.23 2005/04/16 20:47:24 MW Exp $
 // Copyright:   (c) 2001 Vadim Zeitlin
-// License:     wxWindows license
+// Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 // ============================================================================
@@ -53,7 +52,7 @@
 // control ids
 enum
 {
-    TextPage_Reset = 100,
+    TextPage_Reset = wxID_HIGHEST,
 
     TextPage_Set,
     TextPage_Add,
@@ -64,6 +63,7 @@ enum
     TextPage_StreamRedirector,
 
     TextPage_Password,
+    TextPage_NoVertScrollbar,
     TextPage_WrapLines,
     TextPage_Textctrl
 };
@@ -86,6 +86,14 @@ enum WrapStyle
     WrapStyle_Max
 };
 
+// Alignment style radio box
+enum AlignmentStyle
+{
+    Align_Left,
+    Align_Center,
+    Align_Right,
+};
+
 #ifdef __WXMSW__
 
 // textctrl kind values
@@ -106,8 +114,13 @@ static const struct ControlValues
 
     bool password;
     bool readonly;
+    bool processEnter;
+    bool processTab;
+    bool filename;
+    bool noVertScrollbar;
 
     WrapStyle wrapStyle;
+    AlignmentStyle alignmentStyle;
 
 #ifdef __WXMSW__
     TextKind textKind;
@@ -117,7 +130,12 @@ static const struct ControlValues
     TextLines_Multi,    // multiline
     false,              // not password
     false,              // not readonly
+    true,               // do process enter
+    false,              // do not process Tab
+    false,              // not filename
+    false,              // don't hide vertical scrollbar
     WrapStyle_Word,     // wrap on word boundaries
+    Align_Left,         // leading-alignment
 #ifdef __WXMSW__
     TextKind_Plain      // plain EDIT control
 #endif // __WXMSW__
@@ -132,10 +150,15 @@ class TextWidgetsPage : public WidgetsPage
 {
 public:
     // ctor(s) and dtor
-    TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist);
+    TextWidgetsPage(WidgetsBookCtrl *book, wxImageList *imaglist);
     virtual ~TextWidgetsPage(){};
 
-    virtual wxControl *GetWidget() const { return m_text; }
+    virtual wxWindow *GetWidget() const wxOVERRIDE { return m_text; }
+    virtual wxTextEntryBase *GetTextEntry() const wxOVERRIDE { return m_text; }
+    virtual void RecreateWidget() wxOVERRIDE { CreateText(); }
+
+    // lazy creation of the content
+    virtual void CreateContent() wxOVERRIDE;
 
 protected:
     // create an info text contorl
@@ -162,13 +185,15 @@ protected:
 
     void OnText(wxCommandEvent& event);
     void OnTextEnter(wxCommandEvent& event);
+    void OnTextPasted(wxClipboardTextEvent& event);
 
     void OnCheckOrRadioBox(wxCommandEvent& event);
 
     void OnUpdateUIClearButton(wxUpdateUIEvent& event);
 
     void OnUpdateUIPasswordCheckbox(wxUpdateUIEvent& event);
-    void OnUpdateUIWrapLinesCheckbox(wxUpdateUIEvent& event);
+    void OnUpdateUINoVertScrollbarCheckbox(wxUpdateUIEvent& event);
+    void OnUpdateUIWrapLinesRadiobox(wxUpdateUIEvent& event);
 
     void OnUpdateUIResetButton(wxUpdateUIEvent& event);
 
@@ -195,9 +220,16 @@ protected:
     // and another one to choose the wrapping style
     wxRadioBox *m_radioWrap;
 
+    // and yet another one to choose the alignment style
+    wxRadioBox *m_radioAlign;
+
     // the checkboxes controlling text ctrl styles
     wxCheckBox *m_chkPassword,
-               *m_chkReadonly;
+               *m_chkReadonly,
+               *m_chkProcessEnter,
+               *m_chkProcessTab,
+               *m_chkFilename,
+               *m_chkNoVertScrollbar;
 
     // under MSW we test rich edit controls as well here
 #ifdef __WXMSW__
@@ -228,7 +260,7 @@ protected:
 
 private:
     // any class wishing to process wxWidgets events must use this macro
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
     DECLARE_WIDGETS_PAGE(TextWidgetsPage)
 };
 
@@ -255,45 +287,45 @@ protected:
         switch ( HitTest(event.GetPosition(), &x, &y) )
         {
             default:
-                wxFAIL_MSG( _T("unexpected HitTest() result") );
+                wxFAIL_MSG( wxT("unexpected HitTest() result") );
                 // fall through
 
             case wxTE_HT_UNKNOWN:
                 x = y = -1;
-                where = _T("nowhere near");
+                where = wxT("nowhere near");
                 break;
 
             case wxTE_HT_BEFORE:
-                where = _T("before");
+                where = wxT("before");
                 break;
 
             case wxTE_HT_BELOW:
-                where = _T("below");
+                where = wxT("below");
                 break;
 
             case wxTE_HT_BEYOND:
-                where = _T("beyond");
+                where = wxT("beyond");
                 break;
 
             case wxTE_HT_ON_TEXT:
-                where = _T("at");
+                where = wxT("at");
                 break;
         }
 
-        wxLogMessage(_T("Mouse is %s (%ld, %ld)"), where.c_str(), x, y);
+        wxLogMessage(wxT("Mouse is %s (%ld, %ld)"), where.c_str(), x, y);
 
         event.Skip();
     }
 
 private:
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 // ----------------------------------------------------------------------------
 // event tables
 // ----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(TextWidgetsPage, WidgetsPage)
+wxBEGIN_EVENT_TABLE(TextWidgetsPage, WidgetsPage)
     EVT_IDLE(TextWidgetsPage::OnIdle)
 
     EVT_BUTTON(TextPage_Reset, TextWidgetsPage::OnButtonReset)
@@ -309,45 +341,60 @@ BEGIN_EVENT_TABLE(TextWidgetsPage, WidgetsPage)
     EVT_UPDATE_UI(TextPage_Clear, TextWidgetsPage::OnUpdateUIClearButton)
 
     EVT_UPDATE_UI(TextPage_Password, TextWidgetsPage::OnUpdateUIPasswordCheckbox)
-    EVT_UPDATE_UI(TextPage_WrapLines, TextWidgetsPage::OnUpdateUIWrapLinesCheckbox)
+    EVT_UPDATE_UI(TextPage_NoVertScrollbar, TextWidgetsPage::OnUpdateUINoVertScrollbarCheckbox)
+    EVT_UPDATE_UI(TextPage_WrapLines, TextWidgetsPage::OnUpdateUIWrapLinesRadiobox)
 
     EVT_UPDATE_UI(TextPage_Reset, TextWidgetsPage::OnUpdateUIResetButton)
 
     EVT_TEXT(TextPage_Textctrl, TextWidgetsPage::OnText)
     EVT_TEXT_ENTER(TextPage_Textctrl, TextWidgetsPage::OnTextEnter)
+    EVT_TEXT_PASTE(TextPage_Textctrl, TextWidgetsPage::OnTextPasted)
 
     EVT_CHECKBOX(wxID_ANY, TextWidgetsPage::OnCheckOrRadioBox)
     EVT_RADIOBOX(wxID_ANY, TextWidgetsPage::OnCheckOrRadioBox)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
-BEGIN_EVENT_TABLE(WidgetsTextCtrl, wxTextCtrl)
+wxBEGIN_EVENT_TABLE(WidgetsTextCtrl, wxTextCtrl)
     EVT_RIGHT_UP(WidgetsTextCtrl::OnRightClick)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // ============================================================================
 // implementation
 // ============================================================================
 
-IMPLEMENT_WIDGETS_PAGE(TextWidgetsPage, _T("Text"));
+#if defined(__WXX11__)
+    #define FAMILY_CTRLS NATIVE_CTRLS
+#elif defined(__WXUNIVERSAL__)
+    #define FAMILY_CTRLS UNIVERSAL_CTRLS
+#else
+    #define FAMILY_CTRLS NATIVE_CTRLS
+#endif
+
+IMPLEMENT_WIDGETS_PAGE(TextWidgetsPage, wxT("Text"),
+                       FAMILY_CTRLS | EDITABLE_CTRLS
+                       );
 
 // ----------------------------------------------------------------------------
 // TextWidgetsPage creation
 // ----------------------------------------------------------------------------
 
-TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
-               : WidgetsPage(book)
+TextWidgetsPage::TextWidgetsPage(WidgetsBookCtrl *book, wxImageList *imaglist)
+               : WidgetsPage(book, imaglist, text_xpm)
 {
-    imaglist->Add(wxBitmap(text_xpm));
-
     // init everything
 #ifdef __WXMSW__
     m_radioKind =
 #endif // __WXMSW__
     m_radioWrap =
+    m_radioAlign =
     m_radioTextLines = (wxRadioBox *)NULL;
 
     m_chkPassword =
-    m_chkReadonly = (wxCheckBox *)NULL;
+    m_chkReadonly =
+    m_chkProcessEnter =
+    m_chkProcessTab =
+    m_chkFilename =
+    m_chkNoVertScrollbar = (wxCheckBox *)NULL;
 
     m_text =
     m_textPosCur =
@@ -365,16 +412,19 @@ TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
     m_posLast =
     m_selFrom =
     m_selTo = -2; // not -1 which means "no selection"
+}
 
+void TextWidgetsPage::CreateContent()
+{
     // left pane
     static const wxString modes[] =
     {
-        _T("single line"),
-        _T("multi line"),
+        wxT("single line"),
+        wxT("multi line"),
     };
 
-    wxStaticBox *box = new wxStaticBox(this, wxID_ANY, _T("&Set textctrl parameters"));
-    m_radioTextLines = new wxRadioBox(this, wxID_ANY, _T("&Number of lines:"),
+    wxStaticBox *box = new wxStaticBox(this, wxID_ANY, wxT("&Set textctrl parameters"));
+    m_radioTextLines = new wxRadioBox(this, wxID_ANY, wxT("&Number of lines:"),
                                       wxDefaultPosition, wxDefaultSize,
                                       WXSIZEOF(modes), modes,
                                       1, wxRA_SPECIFY_COLS);
@@ -385,36 +435,62 @@ TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
     sizerLeft->AddSpacer(5);
 
     m_chkPassword = CreateCheckBoxAndAddToSizer(
-                        sizerLeft, _T("&Password control"), TextPage_Password
+                        sizerLeft, wxT("&Password control"), TextPage_Password
                     );
     m_chkReadonly = CreateCheckBoxAndAddToSizer(
-                        sizerLeft, _T("&Read-only mode")
+                        sizerLeft, wxT("&Read-only mode")
                     );
+    m_chkProcessEnter = CreateCheckBoxAndAddToSizer(
+                        sizerLeft, wxT("Process &Enter")
+                    );
+    m_chkProcessTab = CreateCheckBoxAndAddToSizer(
+                        sizerLeft, wxT("Process &Tab")
+                    );
+    m_chkFilename = CreateCheckBoxAndAddToSizer(
+                        sizerLeft, wxT("&Filename control")
+                    );
+    m_chkNoVertScrollbar = CreateCheckBoxAndAddToSizer(
+                        sizerLeft, wxT("No &vertical scrollbar"),
+                        TextPage_NoVertScrollbar
+                    );
+    m_chkFilename->Disable(); // not implemented yet
     sizerLeft->AddSpacer(5);
 
     static const wxString wrap[] =
     {
-        _T("no wrap"),
-        _T("word wrap"),
-        _T("char wrap"),
-        _T("best wrap"),
+        wxT("no wrap"),
+        wxT("word wrap"),
+        wxT("char wrap"),
+        wxT("best wrap"),
     };
 
-    m_radioWrap = new wxRadioBox(this, wxID_ANY, _T("&Wrap style:"),
+    m_radioWrap = new wxRadioBox(this, TextPage_WrapLines, wxT("&Wrap style:"),
                                  wxDefaultPosition, wxDefaultSize,
                                  WXSIZEOF(wrap), wrap,
                                  1, wxRA_SPECIFY_COLS);
     sizerLeft->Add(m_radioWrap, 0, wxGROW | wxALL, 5);
 
+    static const wxString halign[] =
+    {
+        wxS("left"),
+        wxS("centre"),
+        wxS("right"),
+    };
+
+    m_radioAlign = new wxRadioBox(this, wxID_ANY, wxS("&Text alignment"),
+                                    wxDefaultPosition, wxDefaultSize,
+                                    WXSIZEOF(halign), halign, 1);
+    sizerLeft->Add(m_radioAlign, 0, wxGROW | wxALL, 5);
+
 #ifdef __WXMSW__
     static const wxString kinds[] =
     {
-        _T("plain edit"),
-        _T("rich edit"),
-        _T("rich edit 2.0"),
+        wxT("plain edit"),
+        wxT("rich edit"),
+        wxT("rich edit 2.0"),
     };
 
-    m_radioKind = new wxRadioBox(this, wxID_ANY, _T("Control &kind"),
+    m_radioKind = new wxRadioBox(this, wxID_ANY, wxT("Control &kind"),
                                  wxDefaultPosition, wxDefaultSize,
                                  WXSIZEOF(kinds), kinds,
                                  1, wxRA_SPECIFY_COLS);
@@ -423,33 +499,33 @@ TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
     sizerLeft->Add(m_radioKind, 0, wxGROW | wxALL, 5);
 #endif // __WXMSW__
 
-    wxButton *btn = new wxButton(this, TextPage_Reset, _T("&Reset"));
+    wxButton *btn = new wxButton(this, TextPage_Reset, wxT("&Reset"));
     sizerLeft->Add(2, 2, 0, wxGROW | wxALL, 1); // spacer
     sizerLeft->Add(btn, 0, wxALIGN_CENTRE_HORIZONTAL | wxALL, 15);
 
     // middle pane
-    wxStaticBox *box2 = new wxStaticBox(this, wxID_ANY, _T("&Change contents:"));
+    wxStaticBox *box2 = new wxStaticBox(this, wxID_ANY, wxT("&Change contents:"));
     wxSizer *sizerMiddleUp = new wxStaticBoxSizer(box2, wxVERTICAL);
 
-    btn = new wxButton(this, TextPage_Set, _T("&Set text value"));
+    btn = new wxButton(this, TextPage_Set, wxT("&Set text value"));
     sizerMiddleUp->Add(btn, 0, wxALL | wxGROW, 1);
 
-    btn = new wxButton(this, TextPage_Add, _T("&Append text"));
+    btn = new wxButton(this, TextPage_Add, wxT("&Append text"));
     sizerMiddleUp->Add(btn, 0, wxALL | wxGROW, 1);
 
-    btn = new wxButton(this, TextPage_Insert, _T("&Insert text"));
+    btn = new wxButton(this, TextPage_Insert, wxT("&Insert text"));
     sizerMiddleUp->Add(btn, 0, wxALL | wxGROW, 1);
 
-    btn = new wxButton(this, TextPage_Load, _T("&Load file"));
+    btn = new wxButton(this, TextPage_Load, wxT("&Load file"));
     sizerMiddleUp->Add(btn, 0, wxALL | wxGROW, 1);
 
-    btn = new wxButton(this, TextPage_Clear, _T("&Clear"));
+    btn = new wxButton(this, TextPage_Clear, wxT("&Clear"));
     sizerMiddleUp->Add(btn, 0, wxALL | wxGROW, 1);
 
-    btn = new wxButton(this, TextPage_StreamRedirector, _T("St&ream redirection"));
+    btn = new wxButton(this, TextPage_StreamRedirector, wxT("St&ream redirection"));
     sizerMiddleUp->Add(btn, 0, wxALL | wxGROW, 1);
 
-    wxStaticBox *box4 = new wxStaticBox(this, wxID_ANY, _T("&Info:"));
+    wxStaticBox *box4 = new wxStaticBox(this, wxID_ANY, wxT("&Info:"));
     wxSizer *sizerMiddleDown = new wxStaticBoxSizer(box4, wxVERTICAL);
 
     m_textPosCur = CreateInfoText();
@@ -459,19 +535,19 @@ TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
     wxSizer *sizerRow = new wxBoxSizer(wxHORIZONTAL);
     sizerRow->Add(CreateTextWithLabelSizer
                   (
-                    _T("Current pos:"),
+                    wxT("Current pos:"),
                     m_textPosCur
                   ),
                   0, wxRIGHT, 5);
     sizerRow->Add(CreateTextWithLabelSizer
                   (
-                    _T("Col:"),
+                    wxT("Col:"),
                     m_textColCur
                   ),
                   0, wxLEFT | wxRIGHT, 5);
     sizerRow->Add(CreateTextWithLabelSizer
                   (
-                    _T("Row:"),
+                    wxT("Row:"),
                     m_textRowCur
                   ),
                   0, wxLEFT, 5);
@@ -483,9 +559,9 @@ TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
                      (
                         CreateTextWithLabelSizer
                         (
-                          _T("Number of lines:"),
+                          wxT("Number of lines:"),
                           m_textLineLast,
-                          _T("Last position:"),
+                          wxT("Last position:"),
                           m_textPosLast
                         ),
                         0, wxALL, 5
@@ -497,9 +573,9 @@ TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
                      (
                         CreateTextWithLabelSizer
                         (
-                          _T("Selection: from"),
+                          wxT("Selection: from"),
                           m_textSelFrom,
-                          _T("to"),
+                          wxT("to"),
                           m_textSelTo
                         ),
                         0, wxALL, 5
@@ -512,7 +588,7 @@ TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
                      (
                         CreateTextWithLabelSizer
                         (
-                          _T("Range 10..20:"),
+                          wxT("Range 10..20:"),
                           m_textRange
                         ),
                         0, wxALL, 5
@@ -523,7 +599,7 @@ TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
     sizerMiddle->Add(sizerMiddleDown, 1, wxGROW | wxTOP, 5);
 
     // right pane
-    wxStaticBox *box3 = new wxStaticBox(this, wxID_ANY, _T("&Text:"));
+    wxStaticBox *box3 = new wxStaticBox(this, wxID_ANY, wxT("&Text:"));
     m_sizerText = new wxStaticBoxSizer(box3, wxHORIZONTAL);
     Reset();
     CreateText();
@@ -536,8 +612,6 @@ TextWidgetsPage::TextWidgetsPage(wxBookCtrl *book, wxImageList *imaglist)
     sizerTop->Add(m_sizerText, 1, wxGROW | (wxALL & ~wxRIGHT), 10);
 
     SetSizer(sizerTop);
-
-    sizerTop->Fit(this);
 }
 
 // ----------------------------------------------------------------------------
@@ -550,7 +624,7 @@ wxTextCtrl *TextWidgetsPage::CreateInfoText()
     if ( !s_maxWidth )
     {
         // calc it once only
-        GetTextExtent(_T("9999999"), &s_maxWidth, NULL);
+        GetTextExtent(wxT("9999999"), &s_maxWidth, NULL);
     }
 
     wxTextCtrl *text = new wxTextCtrl(this, wxID_ANY, wxEmptyString,
@@ -589,8 +663,13 @@ void TextWidgetsPage::Reset()
 
     m_chkPassword->SetValue(DEFAULTS.password);
     m_chkReadonly->SetValue(DEFAULTS.readonly);
+    m_chkProcessEnter->SetValue(DEFAULTS.processEnter);
+    m_chkProcessTab->SetValue(DEFAULTS.processTab);
+    m_chkFilename->SetValue(DEFAULTS.filename);
+    m_chkNoVertScrollbar->SetValue(DEFAULTS.noVertScrollbar);
 
     m_radioWrap->SetSelection(DEFAULTS.wrapStyle);
+    m_radioAlign->SetSelection(DEFAULTS.alignmentStyle);
 
 #ifdef __WXMSW__
     m_radioKind->SetSelection(DEFAULTS.textKind);
@@ -599,11 +678,11 @@ void TextWidgetsPage::Reset()
 
 void TextWidgetsPage::CreateText()
 {
-    int flags = 0;
+    int flags = GetAttrs().m_defaultFlags;
     switch ( m_radioTextLines->GetSelection() )
     {
         default:
-            wxFAIL_MSG( _T("unexpected lines radio box selection") );
+            wxFAIL_MSG( wxT("unexpected lines radio box selection") );
 
         case TextLines_Single:
             break;
@@ -618,11 +697,18 @@ void TextWidgetsPage::CreateText()
         flags |= wxTE_PASSWORD;
     if ( m_chkReadonly->GetValue() )
         flags |= wxTE_READONLY;
+    if ( m_chkProcessEnter->GetValue() )
+        flags |= wxTE_PROCESS_ENTER;
+    if ( m_chkProcessTab->GetValue() )
+        flags |= wxTE_PROCESS_TAB;
+    if ( m_chkNoVertScrollbar->GetValue() )
+        flags |= wxTE_NO_VSCROLL;
 
     switch ( m_radioWrap->GetSelection() )
     {
         default:
-            wxFAIL_MSG( _T("unexpected wrap style radio box selection") );
+            wxFAIL_MSG( wxT("unexpected wrap style radio box selection") );
+            wxFALLTHROUGH;
 
         case WrapStyle_None:
             flags |= wxTE_DONTWRAP; // same as wxHSCROLL
@@ -633,7 +719,7 @@ void TextWidgetsPage::CreateText()
             break;
 
         case WrapStyle_Char:
-            flags |= wxTE_LINEWRAP;
+            flags |= wxTE_CHARWRAP;
             break;
 
         case WrapStyle_Best:
@@ -642,11 +728,26 @@ void TextWidgetsPage::CreateText()
             break;
     }
 
+    switch ( m_radioAlign->GetSelection() )
+    {
+        case Align_Left:
+            flags |= wxTE_LEFT;
+            break;
+        case Align_Center:
+            flags |= wxTE_CENTER;
+            break;
+        case Align_Right:
+            flags |= wxTE_RIGHT;
+            break;
+        default:
+            wxFAIL_MSG( wxS("unexpected alignment style radio box selection") );
+    }
+
 #ifdef __WXMSW__
     switch ( m_radioKind->GetSelection() )
     {
         default:
-            wxFAIL_MSG( _T("unexpected kind radio box selection") );
+            wxFAIL_MSG( wxT("unexpected kind radio box selection") );
 
         case TextKind_Plain:
             break;
@@ -671,11 +772,16 @@ void TextWidgetsPage::CreateText()
     }
     else
     {
-        valueOld = _T("Hello, Universe!");
+        valueOld = wxT("Hello, Universe!");
     }
 
     m_text = new WidgetsTextCtrl(this, TextPage_Textctrl, valueOld, flags);
 
+#if 0
+    if ( m_chkFilename->GetValue() )
+        ;
+#endif // TODO
+
     // cast to int needed to silence gcc warning about different enums
     m_sizerText->Add(m_text, 1, wxALL |
                      (flags & wxTE_MULTILINE ? (int)wxGROW
@@ -726,7 +832,7 @@ void TextWidgetsPage::OnIdle(wxIdleEvent& WXUNUSED(event))
     if ( m_textLineLast )
     {
         m_textLineLast->SetValue(
-                wxString::Format(_T("%d"), m_text->GetNumberOfLines()) );
+                wxString::Format(wxT("%d"), m_text->GetNumberOfLines()) );
     }
 
     if ( m_textSelFrom && m_textSelTo )
@@ -771,8 +877,8 @@ void TextWidgetsPage::OnButtonReset(wxCommandEvent& WXUNUSED(event))
 void TextWidgetsPage::OnButtonSet(wxCommandEvent& WXUNUSED(event))
 {
     m_text->SetValue(m_text->GetWindowStyle() & wxTE_MULTILINE
-                        ? _T("Here,\nthere and\neverywhere")
-                        : _T("Yellow submarine"));
+                        ? wxT("Here,\nthere and\neverywhere")
+                        : wxT("Yellow submarine"));
 
     m_text->SetFocus();
 }
@@ -781,18 +887,18 @@ void TextWidgetsPage::OnButtonAdd(wxCommandEvent& WXUNUSED(event))
 {
     if ( m_text->GetWindowStyle() & wxTE_MULTILINE )
     {
-        m_text->AppendText(_T("We all live in a\n"));
+        m_text->AppendText(wxT("We all live in a\n"));
     }
 
-    m_text->AppendText(_T("Yellow submarine"));
+    m_text->AppendText(wxT("Yellow submarine"));
 }
 
 void TextWidgetsPage::OnButtonInsert(wxCommandEvent& WXUNUSED(event))
 {
-    m_text->WriteText(_T("Is there anybody going to listen to my story"));
+    m_text->WriteText(wxT("Is there anybody going to listen to my story"));
     if ( m_text->GetWindowStyle() & wxTE_MULTILINE )
     {
-        m_text->WriteText(_T("\nall about the girl who came to stay"));
+        m_text->WriteText(wxT("\nall about the girl who came to stay"));
     }
 }
 
@@ -806,14 +912,14 @@ void TextWidgetsPage::OnButtonLoad(wxCommandEvent& WXUNUSED(event))
 {
     // search for the file in several dirs where it's likely to be
     wxPathList pathlist;
-    pathlist.Add(_T("."));
-    pathlist.Add(_T(".."));
-    pathlist.Add(_T("../../../samples/widgets"));
+    pathlist.Add(wxT("."));
+    pathlist.Add(wxT(".."));
+    pathlist.Add(wxT("../../../samples/widgets"));
 
-    wxString filename = pathlist.FindValidPath(_T("textctrl.cpp"));
+    wxString filename = pathlist.FindValidPath(wxT("textctrl.cpp"));
     if ( !filename )
     {
-        wxLogError(_T("File textctrl.cpp not found."));
+        wxLogError(wxT("File textctrl.cpp not found."));
     }
     else // load it
     {
@@ -821,12 +927,12 @@ void TextWidgetsPage::OnButtonLoad(wxCommandEvent& WXUNUSED(event))
         if ( !m_text->LoadFile(filename) )
         {
             // this is not supposed to happen ...
-            wxLogError(_T("Error loading file."));
+            wxLogError(wxT("Error loading file."));
         }
         else
         {
             long elapsed = sw.Time();
-            wxLogMessage(_T("Loaded file '%s' in %lu.%us"),
+            wxLogMessage(wxT("Loaded file '%s' in %lu.%us"),
                          filename.c_str(), elapsed / 1000,
                          (unsigned int) elapsed % 1000);
         }
@@ -838,7 +944,7 @@ void TextWidgetsPage::OnUpdateUIClearButton(wxUpdateUIEvent& event)
     event.Enable(!m_text->GetValue().empty());
 }
 
-void TextWidgetsPage::OnUpdateUIWrapLinesCheckbox(wxUpdateUIEvent& event)
+void TextWidgetsPage::OnUpdateUIWrapLinesRadiobox(wxUpdateUIEvent& event)
 {
     event.Enable( !IsSingleLine() );
 }
@@ -849,14 +955,24 @@ void TextWidgetsPage::OnUpdateUIPasswordCheckbox(wxUpdateUIEvent& event)
     event.Enable( IsSingleLine() );
 }
 
+void TextWidgetsPage::OnUpdateUINoVertScrollbarCheckbox(wxUpdateUIEvent& event)
+{
+    // Vertical scrollbar creation can be blocked only in multiline control
+    event.Enable( !IsSingleLine());
+}
+
 void TextWidgetsPage::OnUpdateUIResetButton(wxUpdateUIEvent& event)
 {
     event.Enable( (m_radioTextLines->GetSelection() != DEFAULTS.textLines) ||
 #ifdef __WXMSW__
                   (m_radioKind->GetSelection() != DEFAULTS.textKind) ||
 #endif // __WXMSW__
-                  (m_chkReadonly->GetValue() != DEFAULTS.readonly) ||
                   (m_chkPassword->GetValue() != DEFAULTS.password) ||
+                  (m_chkReadonly->GetValue() != DEFAULTS.readonly) ||
+                  (m_chkProcessEnter->GetValue() != DEFAULTS.processEnter) ||
+                  (m_chkProcessTab->GetValue() != DEFAULTS.processTab) ||
+                  (m_chkFilename->GetValue() != DEFAULTS.filename) ||
+                  (m_chkNoVertScrollbar->GetValue() != DEFAULTS.noVertScrollbar) ||
                   (m_radioWrap->GetSelection() != DEFAULTS.wrapStyle) );
 }
 
@@ -872,27 +988,71 @@ void TextWidgetsPage::OnText(wxCommandEvent& WXUNUSED(event))
         return;
     }
 
-    wxLogMessage(_T("Text ctrl value changed"));
+    wxLogMessage(wxT("Text ctrl value changed"));
 }
 
 void TextWidgetsPage::OnTextEnter(wxCommandEvent& event)
 {
-    wxLogMessage(_T("Text entered: '%s'"), event.GetString().c_str());
+    wxLogMessage(wxT("Text entered: '%s'"), event.GetString().c_str());
     event.Skip();
 }
 
-void TextWidgetsPage::OnCheckOrRadioBox(wxCommandEvent& WXUNUSED(event))
+void TextWidgetsPage::OnTextPasted(wxClipboardTextEvent& event)
 {
-    CreateText();
+    wxLogMessage("Text pasted from clipboard.");
+    event.Skip();
+}
+
+void TextWidgetsPage::OnCheckOrRadioBox(wxCommandEvent& event)
+{
+#if defined(__WXMSW__) || defined(__WXGTK__) || defined(__WXOSX__)
+    // We should be able to change text alignment
+    // dynamically, without recreating the control.
+    if( event.GetEventObject() == m_radioAlign )
+    {
+        long flags = m_text->GetWindowStyle();
+        flags &= ~(wxTE_LEFT|wxTE_CENTER|wxTE_RIGHT);
+
+        switch ( event.GetSelection() )
+        {
+            case Align_Left:
+                flags |= wxTE_LEFT;
+                break;
+            case Align_Center:
+                flags |= wxTE_CENTER;
+                break;
+            case Align_Right:
+                flags |= wxTE_RIGHT;
+                break;
+            default:
+                wxFAIL_MSG( wxS("unexpected alignment style radio box selection") );
+                return;
+        }
+
+        m_text->SetWindowStyle(flags);
+        m_text->Refresh();
+
+        flags = m_text->GetWindowStyle();
+        wxLogMessage(wxString::Format("Text alignment: %s",
+               (flags & wxTE_RIGHT) ? "Right" :
+               (flags & wxTE_CENTER) ? "Center" : "Left"));
+    }
+    else
+#else
+    wxUnusedVar(event);
+#endif // WXMSW || WXGTK || WXOSX
+    {
+        CreateText();
+    }
 }
 
 void TextWidgetsPage::OnStreamRedirector(wxCommandEvent& WXUNUSED(event))
 {
 #if wxHAS_TEXT_WINDOW_STREAM
     wxStreamToTextRedirector redirect(m_text);
-    wxString str( _T("Outputed to cout, appears in wxTextCtrl!") );
+    wxString str( wxT("Outputed to cout, appears in wxTextCtrl!") );
     wxSTD cout << str << wxSTD endl;
 #else
-    wxMessageBox(_T("This wxWidgets build does not support wxStreamToTextRedirector"));
+    wxMessageBox(wxT("This wxWidgets build does not support wxStreamToTextRedirector"));
 #endif
 }
