@@ -3,7 +3,6 @@
 // Purpose:     wxFileSystem class - interface for opening files
 // Author:      Vaclav Slavik
 // Copyright:   (c) 1999 Vaclav Slavik
-// CVS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -30,6 +29,7 @@
 #include "wx/tokenzr.h"
 #include "wx/uri.h"
 #include "wx/private/fileback.h"
+#include "wx/utils.h"
 
 // ----------------------------------------------------------------------------
 // wxFSFile
@@ -50,7 +50,7 @@ const wxString& wxFSFile::GetMimeType() const
 // wxFileSystemHandler
 // ----------------------------------------------------------------------------
 
-IMPLEMENT_ABSTRACT_CLASS(wxFileSystemHandler, wxObject)
+wxIMPLEMENT_ABSTRACT_CLASS(wxFileSystemHandler, wxObject);
 
 
 /* static */
@@ -124,7 +124,7 @@ wxString wxFileSystemHandler::GetMimeTypeFromExt(const wxString& location)
         wxFileType *ft = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
         if ( !ft || !ft -> GetMimeType(&mime) )
         {
-            mime = wxEmptyString;
+            mime.clear();
         }
 
         delete ft;
@@ -153,7 +153,7 @@ wxString wxFileSystemHandler::GetMimeTypeFromExt(const wxString& location)
 /* static */
 wxString wxFileSystemHandler::GetProtocol(const wxString& location)
 {
-    wxString s = wxEmptyString;
+    wxString s;
     int i, l = location.length();
     bool fnd = false;
 
@@ -182,18 +182,53 @@ wxString wxFileSystemHandler::GetLeftLocation(const wxString& location)
 /* static */
 wxString wxFileSystemHandler::GetRightLocation(const wxString& location)
 {
-    int i, l = location.length();
-    int l2 = l + 1;
-
-    for (i = l-1;
-         (i >= 0) &&
-         ((location[i] != wxT(':')) || (i == 1) || (location[i-2] == wxT(':')));
-         i--)
+    int i, len = location.length();
+    for (i = len-1; i >= 0; i--)
     {
-        if (location[i] == wxT('#')) l2 = i + 1;
+        if (location[i] == wxT('#'))
+            len = i;
+        if (location[i] != wxT(':'))
+            continue;
+
+        // C: on Windows
+        if (i == 1)
+            continue;
+        if (i >= 2 && wxIsalpha(location[i-1]) && location[i-2] == wxT('/'))
+            continue;
+
+        // Could be the protocol
+        break;
     }
     if (i == 0) return wxEmptyString;
-    else return location.Mid(i + 1, l2 - i - 2);
+
+    const static wxString protocol(wxT("file:"));
+    if (i < (int)protocol.length() - 1 || location.compare(0, i + 1, protocol))
+        return location.Mid(i + 1, len - i - 1);
+
+    int s = ++i; // Start position
+    // Check if there are three '/'s after "file:"
+    int end = wxMin(len, s + 3);
+    while (i < end && location[i] == wxT('/'))
+        i++;
+    if (i == s + 2) // Host is specified, e.g. "file://host/path"
+        return location.Mid(s, len - s);
+    if (i > s)
+    {
+        // Remove the last '/' if it is preceding "C:/...".
+        // Otherwise, keep it.
+        if (i + 1 >= len || location[i + 1] != wxT(':'))
+            i--;
+        else if (i + 4 < len)
+        {
+            // Check if ':' was encoded
+            const static wxString colonLower(wxT("%3a"));
+            const static wxString colonUpper(wxT("%3A"));
+            wxString sub =location.Mid(i + 1, 3);
+            if (sub == colonLower || sub == colonUpper)
+                i--;
+        }
+    }
+    return location.Mid(i, len - i);
 }
 
 /* static */
@@ -207,7 +242,7 @@ wxString wxFileSystemHandler::GetAnchor(const wxString& location)
         if (c == wxT('#'))
             return location.Right(l-i-1);
         else if ((c == wxT('/')) || (c == wxT('\\')) || (c == wxT(':')))
-            return wxEmptyString;
+            break;
     }
     return wxEmptyString;
 }
@@ -255,14 +290,14 @@ wxFSFile* wxLocalFSHandler::OpenFile(wxFileSystem& WXUNUSED(fs), const wxString&
 #else
 #error One of wxUSE_FILE or wxUSE_FFILE must be set to 1 for wxFSHandler to work
 #endif
-    if ( !is->Ok() )
+    if ( !is->IsOk() )
     {
         delete is;
         return NULL;
     }
 
     return new wxFSFile(is,
-                        right,
+                        location,
                         wxEmptyString,
                         GetAnchor(location)
 #if wxUSE_DATETIME
@@ -294,8 +329,8 @@ wxString wxLocalFSHandler::FindNext()
 // wxFileSystem
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxFileSystem, wxObject)
-IMPLEMENT_ABSTRACT_CLASS(wxFSFile, wxObject)
+wxIMPLEMENT_DYNAMIC_CLASS(wxFileSystem, wxObject);
+wxIMPLEMENT_ABSTRACT_CLASS(wxFSFile, wxObject);
 
 
 wxList wxFileSystem::m_Handlers;
@@ -355,7 +390,7 @@ void wxFileSystem::ChangePathTo(const wxString& location, bool is_dir)
 
     if (is_dir)
     {
-        if (m_Path.length() > 0 && m_Path.Last() != wxT('/') && m_Path.Last() != wxT(':'))
+        if (!m_Path.empty() && m_Path.Last() != wxT('/') && m_Path.Last() != wxT(':'))
             m_Path << wxT('/');
     }
 
@@ -392,7 +427,7 @@ void wxFileSystem::ChangePathTo(const wxString& location, bool is_dir)
                 }
             }
             if (i == (int) m_Path.length())
-                m_Path = wxEmptyString;
+                m_Path.clear();
         }
         else
         {
@@ -445,7 +480,7 @@ wxFSFile* wxFileSystem::OpenFile(const wxString& location, int flags)
         }
         if (meta != 0) break;
     }
-    m_LastName = wxEmptyString;
+    m_LastName.clear();
 
     // try relative paths first :
     if (meta != wxT(':'))
@@ -635,11 +670,11 @@ wxFileName wxFileSystem::URLToFileName(const wxString& url)
 
     path = wxURI::Unescape(path);
 
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
     // file urls either start with a forward slash (local harddisk),
     // otherwise they have a servername/sharename notation,
     // which only exists on msw and corresponds to a unc
-    if ( path[0u] == wxT('/') && path [1u] != wxT('/'))
+    if ( path.length() > 1 && (path[0u] == wxT('/') && path [1u] != wxT('/')) )
     {
         path = path.Mid(1);
     }
@@ -665,21 +700,11 @@ static wxString EscapeFileNameCharsInURL(const char *in)
     {
         const unsigned char c = *p;
 
-        // notice that all colons *must* be encoded in the paths used by
-        // wxFileSystem even though this makes URLs produced by this method
-        // unusable with IE under Windows as it requires "file:///c:/foo.bar"
-        // and doesn't accept "file:///c%3a/foo.bar" -- but then we never made
-        // any guarantees about general suitability of the strings returned by
-        // this method, they must work with wxFileSystem only and not encoding
-        // the colon breaks handling of
-        // "http://wherever/whatever.zip#zip:filename.ext" URLs so we really
-        // can't do this without heavy changes to the parsing code here, in
-        // particular in GetRightLocation()
-
-        if ( c == '/' || c == '-' || c == '.' || c == '_' || c == '~' ||
-             (c >= '0' && c <= '9') ||
+        // https://tools.ietf.org/html/rfc1738#section-5
+        if ( (c >= '0' && c <= '9') ||
              (c >= 'a' && c <= 'z') ||
-             (c >= 'A' && c <= 'Z') )
+             (c >= 'A' && c <= 'Z') ||
+             strchr("/:$-_.+!*'(),", c) ) // Plus '/' and ':'
         {
             s << c;
         }
@@ -703,15 +728,11 @@ wxString wxFileSystem::FileNameToURL(const wxFileName& filename)
     // unc notation, wxMSW
     if ( url.Find(wxT("\\\\")) == 0 )
     {
-        url = wxT("//") + url.Mid(2);
+        url = url.Mid(2);
     }
     else
     {
         url = wxT("/") + url;
-#ifdef __WXMAC__
-        url = wxT("/") + url;
-#endif
-
     }
 #endif
 
@@ -719,7 +740,7 @@ wxString wxFileSystem::FileNameToURL(const wxFileName& filename)
 
     // Do wxURI- and common practice-compatible escaping: encode the string
     // into UTF-8, then escape anything non-ASCII:
-    return wxT("file:") + EscapeFileNameCharsInURL(url.utf8_str());
+    return wxT("file://") + EscapeFileNameCharsInURL(url.utf8_str());
 }
 
 
@@ -727,7 +748,7 @@ wxString wxFileSystem::FileNameToURL(const wxFileName& filename)
 
 class wxFileSystemModule : public wxModule
 {
-    DECLARE_DYNAMIC_CLASS(wxFileSystemModule)
+    wxDECLARE_DYNAMIC_CLASS(wxFileSystemModule);
 
     public:
         wxFileSystemModule() :
@@ -736,13 +757,13 @@ class wxFileSystemModule : public wxModule
         {
         }
 
-        virtual bool OnInit()
+        virtual bool OnInit() wxOVERRIDE
         {
             m_handler = new wxLocalFSHandler;
             wxFileSystem::AddHandler(m_handler);
             return true;
         }
-        virtual void OnExit()
+        virtual void OnExit() wxOVERRIDE
         {
             delete wxFileSystem::RemoveHandler(m_handler);
 
@@ -754,7 +775,31 @@ class wxFileSystemModule : public wxModule
 
 };
 
-IMPLEMENT_DYNAMIC_CLASS(wxFileSystemModule, wxModule)
+wxIMPLEMENT_DYNAMIC_CLASS(wxFileSystemModule, wxModule);
+
+//// wxFSInputStream
+
+wxFSInputStream::wxFSInputStream(const wxString& filename, int flags)
+{
+    wxFileSystem fs;
+    m_file = fs.OpenFile(filename, flags | wxFS_READ);
+
+    if ( m_file )
+    {
+        wxInputStream* const stream = m_file->GetStream();
+        if ( stream )
+        {
+            // Notice that we pass the stream by reference: it shouldn't be
+            // deleted by us as it's owned by m_file already.
+            InitParentStream(*stream);
+        }
+    }
+}
+
+wxFSInputStream::~wxFSInputStream()
+{
+    delete m_file;
+}
 
 #endif
   // wxUSE_FILESYSTEM
