@@ -4,7 +4,6 @@
 // Author:      Mattia Barbon
 // Modified by:
 // Created:     17.07.02
-// RCS-ID:      $Id$
 // Copyright:   (c) the wxWidgets team
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -118,18 +117,26 @@ void wxMessageOutputBest::Output(const wxString& str)
             return;
     }
 
-    ::MessageBox(NULL, str.t_str(), NULL, MB_ICONINFORMATION | MB_OK);
+    wxString title;
+    if ( wxTheApp )
+        title = wxTheApp->GetAppDisplayName();
+    else // Use some title to avoid default "Error"
+        title = _("Message");
+
+    ::MessageBox(NULL, str.t_str(), title.t_str(), MB_ICONINFORMATION | MB_OK);
 #else // !__WINDOWS__
+    wxUnusedVar(m_flags);
+
     // TODO: use the native message box for the other ports too
     wxMessageOutputStderr::Output(str);
 #endif // __WINDOWS__/!__WINDOWS__
 }
 
 // ----------------------------------------------------------------------------
-// wxMessageOutputStderr
+// wxMessageOutputWithConv
 // ----------------------------------------------------------------------------
 
-wxString wxMessageOutputStderr::AppendLineFeedIfNeeded(const wxString& str)
+wxString wxMessageOutputWithConv::AppendLineFeedIfNeeded(const wxString& str)
 {
     wxString strLF(str);
     if ( strLF.empty() || *strLF.rbegin() != '\n' )
@@ -138,16 +145,37 @@ wxString wxMessageOutputStderr::AppendLineFeedIfNeeded(const wxString& str)
     return strLF;
 }
 
+wxCharBuffer wxMessageOutputWithConv::PrepareForOutput(const wxString& str)
+{
+    wxString strWithLF = AppendLineFeedIfNeeded(str);
+
+#if defined(__WINDOWS__)
+    // Determine whether the encoding is UTF-16. In that case, the file
+    // should have been opened in "wb" mode, and EOL conversion must be done
+    // here as it won't be done at stdio level.
+    if ( m_conv->GetMBNulLen() == 2 )
+    {
+        strWithLF.Replace("\n", "\r\n");
+    }
+#endif // __WINDOWS__
+
+    return m_conv->cWX2MB(strWithLF.c_str());
+}
+
+// ----------------------------------------------------------------------------
+// wxMessageOutputStderr
+// ----------------------------------------------------------------------------
+
+wxMessageOutputStderr::wxMessageOutputStderr(FILE *fp, const wxMBConv& conv)
+                     : wxMessageOutputWithConv(conv),
+                       m_fp(fp)
+{
+}
+
 void wxMessageOutputStderr::Output(const wxString& str)
 {
-    const wxString strWithLF = AppendLineFeedIfNeeded(str);
-    const wxWX2MBbuf buf = strWithLF.mb_str();
-
-    if ( buf )
-        fprintf(m_fp, "%s", (const char*) buf);
-    else // print at least something
-        fprintf(m_fp, "%s", (const char*) strWithLF.ToAscii());
-
+    const wxCharBuffer& buf = PrepareForOutput(str);
+    fwrite(buf, buf.length(), 1, m_fp);
     fflush(m_fp);
 }
 
@@ -157,7 +185,7 @@ void wxMessageOutputStderr::Output(const wxString& str)
 
 void wxMessageOutputDebug::Output(const wxString& str)
 {
-#if defined(__WXMSW__) && !defined(__WXMICROWIN__)
+#if defined(__WINDOWS__)
     wxString out(AppendLineFeedIfNeeded(str));
     out.Replace(wxT("\t"), wxT("        "));
     out.Replace(wxT("\n"), wxT("\r\n"));
@@ -189,12 +217,14 @@ void wxMessageOutputLog::Output(const wxString& str)
 
 #if wxUSE_GUI && wxUSE_MSGDLG
 
+extern WXDLLEXPORT_DATA(const char) wxMessageBoxCaptionStr[] = "Message";
+
 void wxMessageOutputMessageBox::Output(const wxString& str)
 {
     wxString out(str);
 
     // the native MSW msg box understands the TABs, others don't
-#ifndef __WXMSW__
+#ifndef __WINDOWS__
     out.Replace(wxT("\t"), wxT("        "));
 #endif
 
