@@ -4,7 +4,6 @@
 // Author:      Guilhem Lavaux
 // Modified by:
 // Created:     20/07/1997
-// RCS-ID:      $Id$
 // Copyright:   (c) 1997, 1998 Guilhem Lavaux
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -30,7 +29,7 @@
 #include <string.h>
 #include <ctype.h>
 
-IMPLEMENT_CLASS(wxURL, wxURI)
+wxIMPLEMENT_CLASS(wxURL, wxURI);
 
 // Protocols list
 wxProtoInfo *wxURL::ms_protocols = NULL;
@@ -251,7 +250,7 @@ bool wxURL::FetchProtocol()
     {
         if (m_scheme == info->m_protoname)
         {
-            if (m_port.IsNull())
+            if ( m_port.empty() )
                 m_port = info->m_servname;
             m_protoinfo = info;
             m_protocol = (wxProtocol *)m_protoinfo->m_cinfo->CreateObject();
@@ -319,7 +318,7 @@ wxInputStream *wxURL::GetInputStream()
 
         addr.Service(m_port);
 
-        if (!m_protocol->Connect(addr, true)) // Watcom needs the 2nd arg for some reason
+        if (!m_protocol->Connect(addr))
         {
             m_error = wxURL_CONNERR;
             return NULL;
@@ -389,7 +388,7 @@ void wxURL::SetDefaultProxy(const wxString& url_proxy)
             ms_proxyDefault->Close();
         else
             ms_proxyDefault = new wxHTTP();
-        ms_proxyDefault->Connect(addr, true); // Watcom needs the 2nd arg for some reason
+        ms_proxyDefault->Connect(addr);
     }
 }
 
@@ -428,7 +427,7 @@ void wxURL::SetProxy(const wxString& url_proxy)
         if (m_proxy && m_proxy != ms_proxyDefault)
             delete m_proxy;
         m_proxy = new wxHTTP();
-        m_proxy->Connect(addr, true); // Watcom needs the 2nd arg for some reason
+        m_proxy->Connect(addr);
 
         CleanData();
         // Reparse url.
@@ -451,14 +450,14 @@ class wxURLModule : public wxModule
 public:
     wxURLModule();
 
-    virtual bool OnInit();
-    virtual void OnExit();
+    virtual bool OnInit() wxOVERRIDE;
+    virtual void OnExit() wxOVERRIDE;
 
 private:
-    DECLARE_DYNAMIC_CLASS(wxURLModule)
+    wxDECLARE_DYNAMIC_CLASS(wxURLModule);
 };
 
-IMPLEMENT_DYNAMIC_CLASS(wxURLModule, wxModule)
+wxIMPLEMENT_DYNAMIC_CLASS(wxURLModule, wxModule);
 
 wxURLModule::wxURLModule()
 {
