@@ -6,7 +6,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: sashwin.cpp,v 1.53 2006/06/15 17:58:07 ABX Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -35,12 +34,12 @@
 
 #include "wx/laywin.h"
 
-DEFINE_EVENT_TYPE(wxEVT_SASH_DRAGGED)
+wxDEFINE_EVENT( wxEVT_SASH_DRAGGED, wxSashEvent );
 
-IMPLEMENT_DYNAMIC_CLASS(wxSashWindow, wxWindow)
-IMPLEMENT_DYNAMIC_CLASS(wxSashEvent, wxCommandEvent)
+wxIMPLEMENT_DYNAMIC_CLASS(wxSashWindow, wxWindow);
+wxIMPLEMENT_DYNAMIC_CLASS(wxSashEvent, wxCommandEvent);
 
-BEGIN_EVENT_TABLE(wxSashWindow, wxWindow)
+wxBEGIN_EVENT_TABLE(wxSashWindow, wxWindow)
     EVT_PAINT(wxSashWindow::OnPaint)
     EVT_SIZE(wxSashWindow::OnSize)
     EVT_MOUSE_EVENTS(wxSashWindow::OnMouseEvent)
@@ -48,7 +47,7 @@ BEGIN_EVENT_TABLE(wxSashWindow, wxWindow)
     EVT_SET_CURSOR(wxSashWindow::OnSetCursor)
 #endif // __WXMSW__ || __WXMAC__
 
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 bool wxSashWindow::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos,
     const wxSize& size, long style, const wxString& name)
@@ -115,8 +114,8 @@ void wxSashWindow::OnMouseEvent(wxMouseEvent& event)
             // the area to draw on.
             wxWindow* parent = this;
 
-            while (parent && !parent->IsKindOf(CLASSINFO(wxDialog)) &&
-                             !parent->IsKindOf(CLASSINFO(wxFrame)))
+            while (parent && !wxDynamicCast(parent, wxDialog) &&
+                             !wxDynamicCast(parent, wxFrame))
               parent = parent->GetParent();
 
             wxScreenDC::StartDrawingOnTop(parent);
@@ -289,7 +288,7 @@ void wxSashWindow::OnMouseEvent(wxMouseEvent& event)
            ReleaseMouse();
         m_mouseCaptured = false;
     }
-    else if (event.Moving() && !event.Dragging())
+    else if ((event.Moving() || event.Leaving()) && !event.Dragging())
     {
         // Just change the cursor if required
         if ( sashHit != wxSASH_NONE )
@@ -426,10 +425,10 @@ void wxSashWindow::DrawBorders(wxDC& dc)
     int w, h;
     GetClientSize(&w, &h);
 
-    wxPen mediumShadowPen(m_mediumShadowColour, 1, wxSOLID);
-    wxPen darkShadowPen(m_darkShadowColour, 1, wxSOLID);
-    wxPen lightShadowPen(m_lightShadowColour, 1, wxSOLID);
-    wxPen hilightPen(m_hilightColour, 1, wxSOLID);
+    wxPen mediumShadowPen(m_mediumShadowColour, 1, wxPENSTYLE_SOLID);
+    wxPen darkShadowPen(m_darkShadowColour, 1, wxPENSTYLE_SOLID);
+    wxPen lightShadowPen(m_lightShadowColour, 1, wxPENSTYLE_SOLID);
+    wxPen hilightPen(m_hilightColour, 1, wxPENSTYLE_SOLID);
 
     if ( GetWindowStyleFlag() & wxSW_3DBORDER )
     {
@@ -475,16 +474,16 @@ void wxSashWindow::DrawSash(wxSashEdgePosition edge, wxDC& dc)
     int w, h;
     GetClientSize(&w, &h);
 
-    wxPen facePen(m_faceColour, 1, wxSOLID);
-    wxBrush faceBrush(m_faceColour, wxSOLID);
-    wxPen mediumShadowPen(m_mediumShadowColour, 1, wxSOLID);
-    wxPen darkShadowPen(m_darkShadowColour, 1, wxSOLID);
-    wxPen lightShadowPen(m_lightShadowColour, 1, wxSOLID);
-    wxPen hilightPen(m_hilightColour, 1, wxSOLID);
+    wxPen facePen(m_faceColour, 1, wxPENSTYLE_SOLID);
+    wxBrush faceBrush(m_faceColour, wxBRUSHSTYLE_SOLID);
+    wxPen mediumShadowPen(m_mediumShadowColour, 1, wxPENSTYLE_SOLID);
+    wxPen darkShadowPen(m_darkShadowColour, 1, wxPENSTYLE_SOLID);
+    wxPen lightShadowPen(m_lightShadowColour, 1, wxPENSTYLE_SOLID);
+    wxPen hilightPen(m_hilightColour, 1, wxPENSTYLE_SOLID);
     wxColour blackClr(0, 0, 0);
     wxColour whiteClr(255, 255, 255);
-    wxPen blackPen(blackClr, 1, wxSOLID);
-    wxPen whitePen(whiteClr, 1, wxSOLID);
+    wxPen blackPen(blackClr, 1, wxPENSTYLE_SOLID);
+    wxPen whitePen(whiteClr, 1, wxPENSTYLE_SOLID);
 
     if ( edge == wxSASH_LEFT || edge == wxSASH_RIGHT )
     {
@@ -587,7 +586,7 @@ void wxSashWindow::DrawSashTracker(wxSashEdgePosition edge, int x, int y)
     ClientToScreen(&x1, &y1);
     ClientToScreen(&x2, &y2);
 
-    wxPen sashTrackerPen(*wxBLACK, 2, wxSOLID);
+    wxPen sashTrackerPen(*wxBLACK, 2, wxPENSTYLE_SOLID);
 
     screenDC.SetLogicalFunction(wxINVERT);
     screenDC.SetPen(sashTrackerPen);
