@@ -43,7 +43,7 @@
 #include "wx/spinbutt.h"
 
 #ifndef wxHAS_NATIVE_TIMEPICKERCTRL
-    IMPLEMENT_DYNAMIC_CLASS(wxTimePickerCtrl, wxControl)
+wxIMPLEMENT_DYNAMIC_CLASS(wxTimePickerCtrl, wxControl);
 #endif
 
 // ----------------------------------------------------------------------------
@@ -86,7 +86,11 @@ public:
         // nice to add support to "%k" and "%l" (hours with leading blanks
         // instead of zeros) too as this is the most common unsupported case in
         // practice.
+#if wxUSE_XLOCALE
         m_useAMPM = wxLocale::GetInfo(wxLOCALE_TIME_FMT).Contains("%p");
+#else
+        m_useAMPM = false;
+#endif
 
         m_text->Connect
                 (
@@ -237,21 +241,6 @@ private:
                     AppendDigitToCurrentField(key - '0');
                 }
                 break;
-            case WXK_NUMPAD0:
-            case WXK_NUMPAD1:
-            case WXK_NUMPAD2:
-            case WXK_NUMPAD3:
-            case WXK_NUMPAD4:
-            case WXK_NUMPAD5:
-            case WXK_NUMPAD6:
-            case WXK_NUMPAD7:
-            case WXK_NUMPAD8:
-            case WXK_NUMPAD9:
-                if ( m_currentField != Field_AMPM )
-                {
-                    AppendDigitToCurrentField(key - WXK_NUMPAD0);
-                }
-                break;
 
             case 'A':
             case 'P':
@@ -425,6 +414,7 @@ private:
 
             case Field_Max:
                 wxFAIL_MSG( "Invalid field" );
+                return;
         }
 
         UpdateText();
