@@ -8,7 +8,6 @@
 //              3) Fixed ShowPage() bug on displaying bitmaps
 //              Robert Vazan (sizers)
 // Created:     15.08.99
-// RCS-ID:      $Id$
 // Copyright:   (c) 1999 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -54,10 +53,10 @@ class wxWizardSizer : public wxSizer
 public:
     wxWizardSizer(wxWizard *owner);
 
-    virtual wxSizerItem *Insert(size_t index, wxSizerItem *item);
+    virtual wxSizerItem *Insert(size_t index, wxSizerItem *item) wxOVERRIDE;
 
-    virtual void RecalcSizes();
-    virtual wxSize CalcMin();
+    virtual void RecalcSizes() wxOVERRIDE;
+    virtual wxSize CalcMin() wxOVERRIDE;
 
     // get the max size of all wizard pages
     wxSize GetMaxChildSize();
@@ -83,12 +82,13 @@ private:
 
 wxDEFINE_EVENT( wxEVT_WIZARD_PAGE_CHANGED, wxWizardEvent );
 wxDEFINE_EVENT( wxEVT_WIZARD_PAGE_CHANGING, wxWizardEvent );
+wxDEFINE_EVENT( wxEVT_WIZARD_BEFORE_PAGE_CHANGED, wxWizardEvent );
 wxDEFINE_EVENT( wxEVT_WIZARD_CANCEL, wxWizardEvent );
 wxDEFINE_EVENT( wxEVT_WIZARD_FINISHED, wxWizardEvent );
 wxDEFINE_EVENT( wxEVT_WIZARD_HELP, wxWizardEvent );
 wxDEFINE_EVENT( wxEVT_WIZARD_PAGE_SHOWN, wxWizardEvent );
 
-BEGIN_EVENT_TABLE(wxWizard, wxDialog)
+wxBEGIN_EVENT_TABLE(wxWizard, wxDialog)
     EVT_BUTTON(wxID_CANCEL, wxWizard::OnCancel)
     EVT_BUTTON(wxID_BACKWARD, wxWizard::OnBackOrNext)
     EVT_BUTTON(wxID_FORWARD, wxWizard::OnBackOrNext)
@@ -99,9 +99,9 @@ BEGIN_EVENT_TABLE(wxWizard, wxDialog)
     EVT_WIZARD_CANCEL(wxID_ANY, wxWizard::OnWizEvent)
     EVT_WIZARD_FINISHED(wxID_ANY, wxWizard::OnWizEvent)
     EVT_WIZARD_HELP(wxID_ANY, wxWizard::OnWizEvent)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
-IMPLEMENT_DYNAMIC_CLASS(wxWizard, wxDialog)
+wxIMPLEMENT_DYNAMIC_CLASS(wxWizard, wxDialog);
 
 /*
     TODO PROPERTIES :
@@ -110,9 +110,9 @@ IMPLEMENT_DYNAMIC_CLASS(wxWizard, wxDialog)
         title
 */
 
-IMPLEMENT_ABSTRACT_CLASS(wxWizardPage, wxPanel)
-IMPLEMENT_DYNAMIC_CLASS(wxWizardPageSimple, wxWizardPage)
-IMPLEMENT_DYNAMIC_CLASS(wxWizardEvent, wxNotifyEvent)
+wxIMPLEMENT_ABSTRACT_CLASS(wxWizardPage, wxPanel);
+wxIMPLEMENT_DYNAMIC_CLASS(wxWizardPageSimple, wxWizardPage);
+wxIMPLEMENT_DYNAMIC_CLASS(wxWizardEvent, wxNotifyEvent);
 
 // ============================================================================
 // implementation
@@ -271,6 +271,7 @@ void wxWizard::Init()
 {
     m_posWizard = wxDefaultPosition;
     m_page = NULL;
+    m_firstpage = NULL;
     m_btnPrev = m_btnNext = NULL;
     m_statbmp = NULL;
     m_sizerBmpAndPage = NULL;
@@ -316,7 +317,7 @@ void wxWizard::AddBitmapRow(wxBoxSizer *mainColumn)
     mainColumn->Add(
         m_sizerBmpAndPage,
         1, // Vertically stretchable
-        wxEXPAND // Horizonal stretching, no border
+        wxEXPAND // Horizontal stretching, no border
     );
     mainColumn->Add(0,5,
         0, // No vertical stretching
@@ -324,7 +325,7 @@ void wxWizard::AddBitmapRow(wxBoxSizer *mainColumn)
     );
 
 #if wxUSE_STATBMP
-    if ( m_bitmap.Ok() )
+    if ( m_bitmap.IsOk() )
     {
         wxSize bitmapSize(wxDefaultSize);
         if (GetBitmapPlacement())
@@ -373,13 +374,6 @@ void wxWizard::AddBackNextPair(wxBoxSizer *buttonRow)
                   wxT("You must create the buttons before calling ")
                   wxT("wxWizard::AddBackNextPair") );
 
-    // margin between Back and Next buttons
-#ifdef __WXMAC__
-    static const int BACKNEXT_MARGIN = 10;
-#else
-    static const int BACKNEXT_MARGIN = 0;
-#endif
-
     wxBoxSizer *backNextPair = new wxBoxSizer(wxHORIZONTAL);
     buttonRow->Add(
         backNextPair,
@@ -389,7 +383,7 @@ void wxWizard::AddBackNextPair(wxBoxSizer *buttonRow)
     );
 
     backNextPair->Add(m_btnPrev);
-    backNextPair->Add(BACKNEXT_MARGIN,0,
+    backNextPair->Add(10, 0,
         0, // No horizontal stretching
         wxEXPAND // No border, (mostly useless) vertical stretching
     );
@@ -403,7 +397,7 @@ void wxWizard::AddButtonRow(wxBoxSizer *mainColumn)
     // to activate the 'next' button first (create the next button before the back button).
     // The reason is: The user will repeatedly enter information in the wizard pages and then wants to
     // press 'next'. If a user uses mostly the keyboard, he would have to skip the 'back' button
-    // everytime. This is annoying. There is a second reason: RETURN acts as TAB. If the 'next'
+    // every time. This is annoying. There is a second reason: RETURN acts as TAB. If the 'next'
     // button comes first in the TAB order, the user can enter information very fast using the RETURN
     // key to TAB to the next entry field and page. This would not be possible, if the 'back' button
     // was created before the 'next' button.
@@ -417,7 +411,7 @@ void wxWizard::AddButtonRow(wxBoxSizer *mainColumn)
         mainColumn->Add(
             buttonRow,
             0, // Vertically unstretchable
-            wxGROW|wxALIGN_CENTRE
+            wxEXPAND
             );
     else
 #endif
@@ -436,7 +430,12 @@ void wxWizard::AddButtonRow(wxBoxSizer *mainColumn)
 #endif
 
     m_btnNext = new wxButton(this, wxID_FORWARD, _("&Next >"));
-    wxButton *btnCancel=new wxButton(this, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, buttonStyle);
+    // Avoid Cmd+C closing dialog on Mac.
+    wxString cancelLabel(_("&Cancel"));
+#ifdef __WXMAC__
+    cancelLabel.Replace("&",wxEmptyString);
+#endif
+    wxButton *btnCancel=new wxButton(this, wxID_CANCEL, cancelLabel, wxDefaultPosition, wxDefaultSize, buttonStyle);
 #ifndef __WXMAC__
     if (GetExtraStyle() & wxWIZARD_EX_HELPBUTTON)
         btnHelp=new wxButton(this, wxID_HELP, _("&Help"), wxDefaultPosition, wxDefaultSize, buttonStyle);
@@ -563,11 +562,8 @@ bool wxWizard::ShowPage(wxWizardPage *page, bool goingForward)
             m_sizerBmpAndPage->Detach(m_page);
     }
 
-    // set the new page
-    m_page = page;
-
     // is this the end?
-    if ( !m_page )
+    if ( !page )
     {
         // terminate successfully
         if ( IsModal() )
@@ -582,12 +578,18 @@ bool wxWizard::ShowPage(wxWizardPage *page, bool goingForward)
 
         // and notify the user code (this is especially useful for modeless
         // wizards)
-        wxWizardEvent event(wxEVT_WIZARD_FINISHED, GetId(), false, 0);
+        wxWizardEvent event(wxEVT_WIZARD_FINISHED, GetId(), false, m_page);
         (void)GetEventHandler()->ProcessEvent(event);
 
+        m_page = NULL;
+
         return true;
     }
 
+    // notice that we change m_page only here so that wxEVT_WIZARD_FINISHED
+    // event above could still use the correct (i.e. old) value of m_page
+    m_page = page;
+
     // position and show the new page
     (void)m_page->TransferDataToWindow();
 
@@ -608,10 +610,10 @@ bool wxWizard::ShowPage(wxWizardPage *page, bool goingForward)
     if ( m_statbmp )
     {
         bmp = m_page->GetBitmap();
-        if ( !bmp.Ok() )
+        if ( !bmp.IsOk() )
             bmp = m_bitmap;
 
-        if ( !bmpPrev.Ok() )
+        if ( !bmpPrev.IsOk() )
             bmpPrev = m_bitmap;
 
         if (!GetBitmapPlacement())
@@ -624,7 +626,7 @@ bool wxWizard::ShowPage(wxWizardPage *page, bool goingForward)
 
 
     // and update the buttons state
-    m_btnPrev->Enable(HasPrevPage(m_page));
+    m_btnPrev->Enable(m_page != m_firstpage);
 
     const bool hasNext = HasNextPage(m_page);
     const wxString label = hasNext ? _("&Next >") : _("&Finish");
@@ -691,6 +693,8 @@ bool wxWizard::RunWizard(wxWizardPage *firstPage)
 {
     wxCHECK_MSG( firstPage, false, wxT("can't run empty wizard") );
 
+    m_firstpage = firstPage;
+
     // can't return false here because there is no old page
     (void)ShowPage(firstPage, true /* forward */);
 
@@ -730,7 +734,7 @@ wxSize wxWizard::GetPageSize() const
     if ( m_statbmp )
     {
         // make the page at least as tall as the bitmap
-        pageSize.IncTo(wxSize(0, m_bitmap.GetHeight()));
+        pageSize.IncTo(wxSize(0, m_bitmap.GetScaledHeight()));
     }
 
     if ( m_usingSizer )
@@ -786,7 +790,7 @@ void wxWizard::OnBackOrNext(wxCommandEvent& event)
     wxCHECK_RET( m_page, wxT("should have a valid current page") );
 
     // ask the current page first: notice that we do it before calling
-    // GetNext/Prev() because the data transfered from the controls of the page
+    // GetNext/Prev() because the data transferred from the controls of the page
     // may change the value returned by these methods
     if ( !m_page->Validate() || !m_page->TransferDataFromWindow() )
     {
@@ -796,6 +800,13 @@ void wxWizard::OnBackOrNext(wxCommandEvent& event)
 
     bool forward = event.GetEventObject() == m_btnNext;
 
+    // Give the application a chance to set state which may influence GetNext()/GetPrev()
+    wxWizardEvent eventPreChanged(wxEVT_WIZARD_BEFORE_PAGE_CHANGED, GetId(), forward, m_page);
+    (void)m_page->GetEventHandler()->ProcessEvent(eventPreChanged);
+
+    if (!eventPreChanged.IsAllowed())
+        return;
+
     wxWizardPage *page;
     if ( forward )
     {
@@ -932,15 +943,15 @@ bool wxWizard::ResizeBitmap(wxBitmap& bmp)
     if (!GetBitmapPlacement())
         return false;
 
-    if (bmp.Ok())
+    if (bmp.IsOk())
     {
         wxSize pageSize = m_sizerPage->GetSize();
         if (pageSize == wxSize(0,0))
             pageSize = GetPageSize();
-        int bitmapWidth = wxMax(bmp.GetWidth(), GetMinimumBitmapWidth());
+        int bitmapWidth = wxMax(bmp.GetScaledWidth(), GetMinimumBitmapWidth());
         int bitmapHeight = pageSize.y;
 
-        if (!m_statbmp->GetBitmap().Ok() || m_statbmp->GetBitmap().GetHeight() != bitmapHeight)
+        if (!m_statbmp->GetBitmap().IsOk() || m_statbmp->GetBitmap().GetScaledHeight() != bitmapHeight)
         {
             wxBitmap bitmap(bitmapWidth, bitmapHeight);
             {
@@ -960,16 +971,16 @@ bool wxWizard::ResizeBitmap(wxBitmap& bmp)
                     if (GetBitmapPlacement() & wxWIZARD_HALIGN_LEFT)
                         x = 0;
                     else if (GetBitmapPlacement() & wxWIZARD_HALIGN_RIGHT)
-                        x = bitmapWidth - bmp.GetWidth();
+                        x = bitmapWidth - bmp.GetScaledWidth();
                     else
-                        x = (bitmapWidth - bmp.GetWidth())/2;
+                        x = (bitmapWidth - bmp.GetScaledWidth())/2;
 
                     if (GetBitmapPlacement() & wxWIZARD_VALIGN_TOP)
                         y = 0;
                     else if (GetBitmapPlacement() & wxWIZARD_VALIGN_BOTTOM)
-                        y = bitmapHeight - bmp.GetHeight();
+                        y = bitmapHeight - bmp.GetScaledHeight();
                     else
-                        y = (bitmapHeight - bmp.GetHeight())/2;
+                        y = (bitmapHeight - bmp.GetScaledHeight())/2;
 
                     dc.DrawBitmap(bmp, x, y, true);
                     dc.SelectObject(wxNullBitmap);
@@ -985,8 +996,8 @@ bool wxWizard::ResizeBitmap(wxBitmap& bmp)
 
 bool wxWizard::TileBitmap(const wxRect& rect, wxDC& dc, const wxBitmap& bitmap)
 {
-    int w = bitmap.GetWidth();
-    int h = bitmap.GetHeight();
+    int w = bitmap.GetScaledWidth();
+    int h = bitmap.GetScaledHeight();
 
     wxMemoryDC dcMem;
 
@@ -996,7 +1007,7 @@ bool wxWizard::TileBitmap(const wxRect& rect, wxDC& dc, const wxBitmap& bitmap)
     for (i = rect.x; i < rect.x + rect.width; i += w)
     {
         for (j = rect.y; j < rect.y + rect.height; j+= h)
-            dc.Blit(i, j, bitmap.GetWidth(), bitmap.GetHeight(), & dcMem, 0, 0);
+            dc.Blit(i, j, bitmap.GetScaledWidth(), bitmap.GetScaledHeight(), & dcMem, 0, 0);
     }
     dcMem.SelectObject(wxNullBitmap);
 
