@@ -2,7 +2,6 @@
 // Name:        src/gtk/checkbox.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -15,6 +14,8 @@
 #include "wx/checkbox.h"
 
 #include <gtk/gtk.h>
+#include "wx/gtk/private/gtk2-compat.h"
+#include "wx/gtk/private/eventsdisabler.h"
 
 //-----------------------------------------------------------------------------
 // data
@@ -29,8 +30,6 @@ extern bool           g_blockEventsOnDrag;
 extern "C" {
 static void gtk_checkbox_toggled_callback(GtkWidget *widget, wxCheckBox *cb)
 {
-    if (!cb->m_hasVMT) return;
-
     if (g_blockEventsOnDrag) return;
 
     // Transitions for 3state checkbox must be done manually, GTK's checkbox
@@ -44,10 +43,10 @@ static void gtk_checkbox_toggled_callback(GtkWidget *widget, wxCheckBox *cb)
         {
             // The 3 states cycle like this when clicked:
             // checked -> undetermined -> unchecked -> checked -> ...
-            bool active = gtk_toggle_button_get_active(toggle);
-            bool inconsistent = gtk_toggle_button_get_inconsistent(toggle);
+            bool active = gtk_toggle_button_get_active(toggle) != 0;
+            bool inconsistent = gtk_toggle_button_get_inconsistent(toggle) != 0;
 
-            cb->GTKDisableEvents();
+            wxGtkEventsDisabler<wxCheckBox> noEvents(cb);
 
             if (!active && !inconsistent)
             {
@@ -69,8 +68,6 @@ static void gtk_checkbox_toggled_callback(GtkWidget *widget, wxCheckBox *cb)
             {
                 wxFAIL_MSG(wxT("3state wxCheckBox in unexpected state!"));
             }
-
-            cb->GTKEnableEvents();
         }
         else
         {
@@ -79,7 +76,7 @@ static void gtk_checkbox_toggled_callback(GtkWidget *widget, wxCheckBox *cb)
         }
     }
 
-    wxCommandEvent event(wxEVT_COMMAND_CHECKBOX_CLICKED, cb->GetId());
+    wxCommandEvent event(wxEVT_CHECKBOX, cb->GetId());
     event.SetInt(cb->Get3StateValue());
     event.SetEventObject(cb);
     cb->HandleWindowEvent(event);
@@ -90,10 +87,15 @@ static void gtk_checkbox_toggled_callback(GtkWidget *widget, wxCheckBox *cb)
 // wxCheckBox
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxCheckBox,wxControl)
-
 wxCheckBox::wxCheckBox()
 {
+    m_widgetCheckbox = NULL;
+}
+
+wxCheckBox::~wxCheckBox()
+{
+    if (m_widgetCheckbox && m_widgetCheckbox != m_widget)
+        GTKDisconnect(m_widgetCheckbox);
 }
 
 bool wxCheckBox::Create(wxWindow *parent,
@@ -105,6 +107,7 @@ bool wxCheckBox::Create(wxWindow *parent,
                         const wxValidator& validator,
                         const wxString &name )
 {
+    WXValidateStyle( &style );
     if (!PreCreation( parent, pos, size ) ||
         !CreateBase( parent, id, pos, size, style, validator, name ))
     {
@@ -112,11 +115,6 @@ bool wxCheckBox::Create(wxWindow *parent,
         return false;
     }
 
-    wxASSERT_MSG( (style & wxCHK_ALLOW_3RD_STATE_FOR_USER) == 0 ||
-                  (style & wxCHK_3STATE) != 0,
-                  wxT("Using wxCHK_ALLOW_3RD_STATE_FOR_USER")
-                  wxT(" style flag for a 2-state checkbox is useless") );
-
     if ( style & wxALIGN_RIGHT )
     {
         // VZ: as I don't know a way to create a right aligned checkbox with
@@ -125,9 +123,15 @@ bool wxCheckBox::Create(wxWindow *parent,
         m_widgetCheckbox = gtk_check_button_new();
 
         m_widgetLabel = gtk_label_new("");
+#ifdef __WXGTK4__
+        g_object_set(m_widgetLabel, "xalign", 0.0f, NULL);
+#else
+        wxGCC_WARNING_SUPPRESS(deprecated-declarations)
         gtk_misc_set_alignment(GTK_MISC(m_widgetLabel), 0.0, 0.5);
+        wxGCC_WARNING_RESTORE()
+#endif
 
-        m_widget = gtk_hbox_new(FALSE, 0);
+        m_widget = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
         gtk_box_pack_start(GTK_BOX(m_widget), m_widgetLabel, FALSE, FALSE, 3);
         gtk_box_pack_start(GTK_BOX(m_widget), m_widgetCheckbox, FALSE, FALSE, 3);
 
@@ -137,7 +141,7 @@ bool wxCheckBox::Create(wxWindow *parent,
     else
     {
         m_widgetCheckbox = gtk_check_button_new_with_label("");
-        m_widgetLabel = GTK_BIN(m_widgetCheckbox)->child;
+        m_widgetLabel = gtk_bin_get_child(GTK_BIN(m_widgetCheckbox));
         m_widget = m_widgetCheckbox;
     }
     g_object_ref(m_widget);
@@ -172,18 +176,15 @@ void wxCheckBox::SetValue( bool state )
     if (state == GetValue())
         return;
 
-    GTKDisableEvents();
-
+    wxGtkEventsDisabler<wxCheckBox> noEvents(this);
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(m_widgetCheckbox), state );
-
-    GTKEnableEvents();
 }
 
 bool wxCheckBox::GetValue() const
 {
     wxCHECK_MSG( m_widgetCheckbox != NULL, false, wxT("invalid checkbox") );
 
-    return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_widgetCheckbox));
+    return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_widgetCheckbox)) != 0;
 }
 
 void wxCheckBox::DoSet3StateValue(wxCheckBoxState state)
@@ -209,6 +210,13 @@ void wxCheckBox::SetLabel( const wxString& label )
 {
     wxCHECK_RET( m_widgetLabel != NULL, wxT("invalid checkbox") );
 
+    // If we don't hide the empty label, in some themes a focus rectangle is
+    // still drawn around it and this looks out of place.
+    if ( label.empty() )
+        gtk_widget_hide(m_widgetLabel);
+    else
+        gtk_widget_show(m_widgetLabel);
+
     // save the label inside m_label in case user calls GetLabel() later
     wxControl::SetLabel(label);
 
@@ -230,20 +238,20 @@ bool wxCheckBox::Enable( bool enable )
 
 void wxCheckBox::DoApplyWidgetStyle(GtkRcStyle *style)
 {
-    gtk_widget_modify_style(m_widgetCheckbox, style);
-    gtk_widget_modify_style(m_widgetLabel, style);
+    GTKApplyStyle(m_widgetCheckbox, style);
+    GTKApplyStyle(m_widgetLabel, style);
 }
 
 GdkWindow *wxCheckBox::GTKGetWindow(wxArrayGdkWindows& WXUNUSED(windows)) const
 {
-    return GTK_BUTTON(m_widgetCheckbox)->event_window;
+    return gtk_button_get_event_window(GTK_BUTTON(m_widgetCheckbox));
 }
 
 // static
 wxVisualAttributes
 wxCheckBox::GetClassDefaultAttributes(wxWindowVariant WXUNUSED(variant))
 {
-    return GetDefaultAttributesFromGTKWidget(gtk_check_button_new);
+    return GetDefaultAttributesFromGTKWidget(gtk_check_button_new());
 }
 
 #endif
