@@ -99,7 +99,10 @@ wxEND_EVENT_TABLE()
 wxComboBox::~wxComboBox()
 {
     if (m_entry)
+    {
         GTKDisconnect(m_entry);
+        g_object_remove_weak_pointer(G_OBJECT(m_entry), (void**)&m_entry);
+    }
 }
 
 void wxComboBox::Init()
@@ -153,6 +156,9 @@ bool wxComboBox::Create( wxWindow *parent, wxWindowID id, const wxString& value,
                                          !HasFlag(wxTE_PROCESS_ENTER) );
 
         gtk_editable_set_editable(GTK_EDITABLE(entry), true);
+#ifdef __WXGTK3__
+        gtk_entry_set_width_chars(entry, 0);
+#endif
     }
 
     Append(n, choices);
@@ -191,9 +197,7 @@ bool wxComboBox::Create( wxWindow *parent, wxWindowID id, const wxString& value,
     g_signal_connect_after (m_widget, "changed",
                         G_CALLBACK (gtkcombobox_changed_callback), this);
 
-#ifndef __WXGTK3__
-    if ( !gtk_check_version(2,10,0) )
-#endif
+    if ( wx_is_at_least_gtk2(10) )
     {
         g_signal_connect (m_widget, "notify::popup-shown",
                           G_CALLBACK (gtkcombobox_popupshown_callback), this);
@@ -212,6 +216,7 @@ void wxComboBox::GTKCreateComboBoxWidget()
     g_object_ref(m_widget);
 
     m_entry = GTK_ENTRY(gtk_bin_get_child(GTK_BIN(m_widget)));
+    g_object_add_weak_pointer(G_OBJECT(m_entry), (void**)&m_entry);
 }
 
 GtkEditable *wxComboBox::GetEditable() const
