@@ -2,7 +2,6 @@
 // Name:        src/gtk/utilsgtk.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -21,11 +20,19 @@
 #include "wx/apptrait.h"
 #include "wx/process.h"
 #include "wx/sysopt.h"
-#include "wx/unix/execute.h"
+#include "wx/vector.h"
 
 #include "wx/gtk/private/timer.h"
 #include "wx/evtloop.h"
 
+#include <gtk/gtk.h>
+#ifdef GDK_WINDOWING_WIN32
+#include <gdk/gdkwin32.h>
+#endif
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#endif
+
 #if wxDEBUG_LEVEL
     #include "wx/gtk/assertdlg_gtk.h"
     #if wxUSE_STACKWALKER
@@ -37,61 +44,68 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/wait.h>   // for WNOHANG
+#ifdef __UNIX__
 #include <unistd.h>
-
-#include "glib.h"
-#include "gdk/gdk.h"
-#include "gtk/gtk.h"
-#include "gdk/gdkx.h"
-
-#if wxUSE_DETECT_SM
-    #include "X11/Xlib.h"
-    #include "X11/SM/SMlib.h"
 #endif
 
-//-----------------------------------------------------------------------------
-// data
-//-----------------------------------------------------------------------------
+#if wxUSE_DETECT_SM
+    #include <X11/SM/SMlib.h>
 
-extern GtkWidget *wxGetRootWindow();
+    #include "wx/unix/utilsx11.h"
+#endif
+
+#include "wx/gtk/private/gtk2-compat.h"
+
+GdkWindow* wxGetTopLevelGDK();
 
 //----------------------------------------------------------------------------
 // misc.
 //----------------------------------------------------------------------------
-#ifndef __EMX__
-// on OS/2, we use the wxBell from wxBase library
 
 void wxBell()
 {
     gdk_beep();
 }
-#endif
 
 // ----------------------------------------------------------------------------
-// display characterstics
+// display characteristics
 // ----------------------------------------------------------------------------
 
+#ifdef GDK_WINDOWING_X11
 void *wxGetDisplay()
 {
-    return GDK_DISPLAY();
+    return GDK_DISPLAY_XDISPLAY(gdk_window_get_display(wxGetTopLevelGDK()));
 }
+#endif
 
 void wxDisplaySize( int *width, int *height )
 {
+#ifdef __WXGTK4__
+    GdkMonitor* monitor = gdk_display_get_primary_monitor(gdk_display_get_default());
+    GdkRectangle rect;
+    gdk_monitor_get_geometry(monitor, &rect);
+    if (width) *width = rect.width;
+    if (height) *height = rect.height;
+#else
+    wxGCC_WARNING_SUPPRESS(deprecated-declarations)
     if (width) *width = gdk_screen_width();
     if (height) *height = gdk_screen_height();
+    wxGCC_WARNING_RESTORE()
+#endif
 }
 
 void wxDisplaySizeMM( int *width, int *height )
 {
+#ifdef __WXGTK4__
+    GdkMonitor* monitor = gdk_display_get_primary_monitor(gdk_display_get_default());
+    if (width) *width = gdk_monitor_get_width_mm(monitor);
+    if (height) *height = gdk_monitor_get_height_mm(monitor);
+#else
+    wxGCC_WARNING_SUPPRESS(deprecated-declarations)
     if (width) *width = gdk_screen_width_mm();
     if (height) *height = gdk_screen_height_mm();
-}
-
-void wxGetMousePosition( int* x, int* y )
-{
-    gdk_window_get_pointer( NULL, x, y, NULL );
+    wxGCC_WARNING_RESTORE()
+#endif
 }
 
 bool wxColourDisplay()
@@ -101,7 +115,11 @@ bool wxColourDisplay()
 
 int wxDisplayDepth()
 {
-    return gdk_drawable_get_visual( wxGetRootWindow()->window )->depth;
+#ifdef __WXGTK4__
+    return 24;
+#else
+    return gdk_visual_get_depth(gdk_window_get_visual(wxGetTopLevelGDK()));
+#endif
 }
 
 wxWindow* wxFindWindowAtPoint(const wxPoint& pt)
@@ -162,7 +180,9 @@ const gchar *wx_pango_version_check (int major, int minor, int micro)
     //       added in pango-1.4 or earlier since GTK 2.4 (our minimum requirement
     //       for GTK lib) required pango 1.4...
 
-#ifdef PANGO_VERSION_MAJOR
+#ifdef __WXGTK3__
+    return pango_version_check(major, minor, micro);
+#elif defined(PANGO_VERSION_MAJOR)
     if (!gtk_check_version (2,11,0))
     {
         // GTK+ 2.11 requires Pango >= 1.15.3 and pango_version_check
@@ -181,51 +201,29 @@ const gchar *wx_pango_version_check (int major, int minor, int micro)
 #endif
 }
 
-
-// ----------------------------------------------------------------------------
-// subprocess routines
-// ----------------------------------------------------------------------------
-
-extern "C" {
-static
-void GTK_EndProcessDetector(gpointer data, gint source,
-                            GdkInputCondition WXUNUSED(condition))
-{
-    wxEndProcessData * const
-        proc_data = static_cast<wxEndProcessData *>(data);
-
-    // child exited, end waiting
-    close(source);
-
-    // don't call us again!
-    gdk_input_remove(proc_data->tag);
-
-    wxHandleProcessTermination(proc_data);
-}
-}
-
-int wxGUIAppTraits::AddProcessCallback(wxEndProcessData *proc_data, int fd)
-{
-    int tag = gdk_input_add(fd,
-                            GDK_INPUT_READ,
-                            GTK_EndProcessDetector,
-                            (gpointer)proc_data);
-
-    return tag;
-}
-
-
-
 // ----------------------------------------------------------------------------
 // wxPlatformInfo-related
 // ----------------------------------------------------------------------------
 
-wxPortId wxGUIAppTraits::GetToolkitVersion(int *verMaj, int *verMin) const
+wxPortId wxGUIAppTraits::GetToolkitVersion(int *verMaj,
+                                           int *verMin,
+                                           int *verMicro) const
 {
+#ifdef __WXGTK3__
+    if (verMaj)
+        *verMaj = gtk_get_major_version();
+    if (verMin)
+        *verMin = gtk_get_minor_version();
+    if (verMicro)
+        *verMicro = gtk_get_micro_version();
+#else
     if ( verMaj )
         *verMaj = gtk_major_version;
     if ( verMin )
         *verMin = gtk_minor_version;
+    if ( verMicro )
+        *verMicro = gtk_micro_version;
+#endif
 
     return wxPORT_GTK;
 }
@@ -242,17 +240,7 @@ wxTimerImpl *wxGUIAppTraits::CreateTimerImpl(wxTimer *timer)
 #if wxUSE_DETECT_SM
 static wxString GetSM()
 {
-    class Dpy
-    {
-    public:
-        Dpy() { m_dpy = XOpenDisplay(NULL); }
-        ~Dpy() { if ( m_dpy ) XCloseDisplay(m_dpy); }
-
-        operator Display *() const { return m_dpy; }
-    private:
-        Display *m_dpy;
-    } dpy;
-
+    wxX11Display dpy;
     if ( !dpy )
         return wxEmptyString;
 
@@ -266,7 +254,11 @@ static wxString GetSM()
 
     if ( !smc_conn )
     {
-        wxLogDebug("Failed to connect to session manager: %s", smerr);
+        // Don't report error if there is no session manager at all
+        if (getenv("SESSION_MANAGER"))
+        {
+            wxLogDebug("Failed to connect to session manager: %s", smerr);
+        }
         return wxEmptyString;
     }
 
@@ -292,13 +284,7 @@ wxEventLoopBase *wxGUIAppTraits::CreateEventLoop()
 }
 
 
-#if wxUSE_INTL
-void wxGUIAppTraits::SetLocale()
-{
-    gtk_set_locale();
-    wxUpdateLocaleIsUtf8();
-}
-#endif
+#ifdef __UNIX__
 
 #if wxDEBUG_LEVEL && wxUSE_STACKWALKER
 
@@ -308,45 +294,68 @@ class StackDump : public wxStackWalker
 public:
     StackDump(GtkAssertDialog *dlg) { m_dlg=dlg; }
 
-protected:
-    virtual void OnStackFrame(const wxStackFrame& frame)
+    void ShowStackInDialog()
     {
-        wxString fncname = frame.GetName();
-        wxString fncargs = fncname;
+        ProcessFrames(0);
 
-        size_t n = fncname.find(wxT('('));
-        if (n != wxString::npos)
+        for ( wxVector<Frame>::const_iterator it = m_frames.begin();
+              it != m_frames.end();
+              ++it )
         {
-            // remove arguments from function name
-            fncname.erase(n);
-
-            // remove function name and brackets from arguments
-            fncargs = fncargs.substr(n+1, fncargs.length()-n-2);
-        }
-        else
-            fncargs = wxEmptyString;
-
-        // append this stack frame's info in the dialog
-        if (!frame.GetFileName().empty() || !fncname.empty())
             gtk_assert_dialog_append_stack_frame(m_dlg,
-                                                fncname.mb_str(),
-                                                fncargs.mb_str(),
-                                                frame.GetFileName().mb_str(),
-                                                frame.GetLine());
+                                                 it->name.utf8_str(),
+                                                 it->file.utf8_str(),
+                                                 it->line);
+        }
+
+        m_frames.clear();
+    }
+
+protected:
+    virtual void OnStackFrame(const wxStackFrame& frame) wxOVERRIDE
+    {
+        const wxString name = frame.GetName();
+        if ( name.StartsWith("wxOnAssert") )
+        {
+            // Ignore all frames until the wxOnAssert() one, just as we do in
+            // wxAppTraitsBase::GetAssertStackTrace().
+            m_frames.clear();
+            return;
+        }
+
+        // Also ignore frames which don't have neither the function name nor
+        // the file name, showing them in the dialog wouldn't provide any
+        // useful information.
+        if ( name.empty() && frame.GetFileName().empty() )
+            return;
+
+        m_frames.push_back(Frame(frame));
     }
 
 private:
     GtkAssertDialog *m_dlg;
+
+    struct Frame
+    {
+        explicit Frame(const wxStackFrame& f)
+            : name(f.GetName()),
+              file(f.GetFileName()),
+              line(f.GetLine())
+        {
+        }
+
+        wxString name;
+        wxString file;
+        int line;
+    };
+
+    wxVector<Frame> m_frames;
 };
 
-// the callback functions must be extern "C" to comply with GTK+ declarations
-extern "C"
+static void get_stackframe_callback(void* p)
 {
-    void get_stackframe_callback(StackDump *dump)
-    {
-        // skip over frames up to including wxOnAssert()
-        dump->ProcessFrames(3);
-    }
+    StackDump* dump = static_cast<StackDump*>(p);
+    dump->ShowStackInDialog();
 }
 
 #endif // wxDEBUG_LEVEL && wxUSE_STACKWALKER
@@ -364,6 +373,19 @@ bool wxGUIAppTraits::ShowAssertDialog(const wxString& msg)
         GtkWidget *dialog = gtk_assert_dialog_new();
         gtk_assert_dialog_set_message(GTK_ASSERT_DIALOG(dialog), msg.mb_str());
 
+        GdkDisplay* display = gtk_widget_get_display(dialog);
+#ifdef __WXGTK4__
+        gdk_seat_ungrab(gdk_display_get_default_seat(display));
+#elif defined(__WXGTK3__)
+        wxGCC_WARNING_SUPPRESS(deprecated-declarations)
+        GdkDeviceManager* manager = gdk_display_get_device_manager(display);
+        GdkDevice* device = gdk_device_manager_get_client_pointer(manager);
+        gdk_device_ungrab(device, unsigned(GDK_CURRENT_TIME));
+        wxGCC_WARNING_RESTORE()
+#else
+        gdk_display_pointer_ungrab(display, unsigned(GDK_CURRENT_TIME));
+#endif
+
 #if wxUSE_STACKWALKER
         // save the current stack ow...
         StackDump dump(GTK_ASSERT_DIALOG(dialog));
@@ -373,7 +395,7 @@ bool wxGUIAppTraits::ShowAssertDialog(const wxString& msg)
         gtk_assert_dialog_set_backtrace_callback
         (
             GTK_ASSERT_DIALOG(dialog),
-            (GtkAssertDialogStackFrameCallback)get_stackframe_callback,
+            get_stackframe_callback,
             &dump
         );
 #endif // wxUSE_STACKWALKER
@@ -383,7 +405,13 @@ bool wxGUIAppTraits::ShowAssertDialog(const wxString& msg)
         switch (result)
         {
             case GTK_ASSERT_DIALOG_STOP:
-                wxTrap();
+                // Don't call wxTrap() directly from here to avoid having the
+                // functions between the occurrence of the assert in the code
+                // and this function in the call stack. Instead, just set a
+                // flag so that inline expansion of the assert macro we are
+                // called from calls wxTrap() itself, like this the debugger
+                // would break exactly at the assert position.
+                wxTrapInAssert = true;
                 break;
             case GTK_ASSERT_DIALOG_CONTINUE:
                 // nothing to do
@@ -405,6 +433,10 @@ bool wxGUIAppTraits::ShowAssertDialog(const wxString& msg)
     return wxAppTraitsBase::ShowAssertDialog(msg);
 }
 
+#endif // __UNIX__
+
+#if defined(__UNIX__)
+
 wxString wxGUIAppTraits::GetDesktopEnvironment() const
 {
     wxString de = wxSystemOptions::GetOption(wxT("gtk.desktop"));
@@ -423,102 +455,4 @@ wxString wxGUIAppTraits::GetDesktopEnvironment() const
     return de;
 }
 
-#ifdef __WXGTK26__
-
-// see the hack below in wxCmdLineParser::GetUsageString().
-// TODO: replace this hack with a g_option_group_get_entries()
-//       call as soon as such function exists;
-//       see http://bugzilla.gnome.org/show_bug.cgi?id=431021 for the relative
-//       feature request
-struct _GOptionGroup
-{
-  gchar           *name;
-  gchar           *description;
-  gchar           *help_description;
-
-  GDestroyNotify   destroy_notify;
-  gpointer         user_data;
-
-  GTranslateFunc   translate_func;
-  GDestroyNotify   translate_notify;
-  gpointer     translate_data;
-
-  GOptionEntry    *entries;
-  gint             n_entries;
-
-  GOptionParseFunc pre_parse_func;
-  GOptionParseFunc post_parse_func;
-  GOptionErrorFunc error_func;
-};
-
-wxString wxGetNameFromGtkOptionEntry(const GOptionEntry *opt)
-{
-    wxString ret;
-
-    if (opt->short_name)
-        ret << wxT("-") << opt->short_name;
-    if (opt->long_name)
-    {
-        if (!ret.empty())
-            ret << wxT(", ");
-        ret << wxT("--") << opt->long_name;
-
-        if (opt->arg_description)
-            ret << wxT("=") << opt->arg_description;
-    }
-
-    return wxT("  ") + ret;
-}
-
-#endif // __WXGTK26__
-
-wxString
-wxGUIAppTraits::GetStandardCmdLineOptions(wxArrayString& names,
-                                          wxArrayString& desc) const
-{
-    wxString usage;
-
-#ifdef __WXGTK26__
-    if (!gtk_check_version(2,6,0))
-    {
-        // since GTK>=2.6, we can use the glib_check_version() symbol...
-
-        // check whether GLib version is greater than 2.6 but also lower than 2.19
-        // because, as we use the undocumented _GOptionGroup struct, we don't want
-        // to run this code with future versions which might change it (2.19 is the
-        // latest one at the time of this writing)
-        if (!glib_check_version(2,6,0) && glib_check_version(2,20,0))
-        {
-            usage << _("The following standard GTK+ options are also supported:\n");
-
-            // passing true here means that the function can open the default
-            // display while parsing (not really used here anyhow)
-            GOptionGroup *gtkOpts = gtk_get_option_group(true);
-
-            // WARNING: here we access the internals of GOptionGroup:
-            GOptionEntry *entries = ((_GOptionGroup*)gtkOpts)->entries;
-            unsigned int n_entries = ((_GOptionGroup*)gtkOpts)->n_entries;
-            wxArrayString namesOptions, descOptions;
-
-            for ( size_t n = 0; n < n_entries; n++ )
-            {
-                if ( entries[n].flags & G_OPTION_FLAG_HIDDEN )
-                    continue;       // skip
-
-                names.push_back(wxGetNameFromGtkOptionEntry(&entries[n]));
-
-                const gchar * const entryDesc = entries[n].description;
-                desc.push_back(wxString(entryDesc));
-            }
-
-            g_option_group_free (gtkOpts);
-        }
-    }
-#else
-    wxUnusedVar(names);
-    wxUnusedVar(desc);
-#endif // __WXGTK26__
-
-    return usage;
-}
-
+#endif // __UNIX__
