@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     17/09/98
-// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -59,11 +58,9 @@ static void wxTextWindowGainFocusProc(Widget w, XtPointer clientData, XmAnyCallb
 static void wxTextWindowLoseFocusProc(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs);
 static void wxTextWindowActivateProc(Widget w, XtPointer clientData, XmAnyCallbackStruct *ptr);
 
-    IMPLEMENT_DYNAMIC_CLASS(wxTextCtrl, wxTextCtrlBase)
-
-    BEGIN_EVENT_TABLE(wxTextCtrl, wxTextCtrlBase)
-        EVT_DROP_FILES(wxTextCtrl::OnDropFiles)
-        EVT_CHAR(wxTextCtrl::OnChar)
+wxBEGIN_EVENT_TABLE(wxTextCtrl, wxTextCtrlBase)
+    EVT_DROP_FILES(wxTextCtrl::OnDropFiles)
+    EVT_CHAR(wxTextCtrl::OnChar)
 
     EVT_MENU(wxID_CUT, wxTextCtrl::OnCut)
     EVT_MENU(wxID_COPY, wxTextCtrl::OnCopy)
@@ -77,7 +74,7 @@ static void wxTextWindowActivateProc(Widget w, XtPointer clientData, XmAnyCallba
     EVT_UPDATE_UI(wxID_UNDO, wxTextCtrl::OnUpdateUndo)
     EVT_UPDATE_UI(wxID_REDO, wxTextCtrl::OnUpdateRedo)
 
-    END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // ============================================================================
 // implementation
@@ -163,7 +160,7 @@ bool wxTextCtrl::Create(wxWindow *parent,
 #if 0
         // TODO: Is this relevant? What does it do?
         int noCols = 2;
-        if (!value.IsNull() && (value.length() > (unsigned int) noCols))
+        if (!value.empty() && (value.length() > (unsigned int) noCols))
             noCols = value.length();
         XtVaSetValues((Widget) m_mainWidget,
                       XmNcolumns, noCols,
@@ -472,7 +469,7 @@ void wxTextCtrl::DoSendEvents(void *wxcbs, long keycode)
     if ( !InSetValue() && m_processedDefault )
     {
         // Can generate a command
-        wxCommandEvent commandEvent(wxEVT_COMMAND_TEXT_UPDATED, GetId());
+        wxCommandEvent commandEvent(wxEVT_TEXT, GetId());
         commandEvent.SetEventObject(this);
         ProcessCommand(commandEvent);
     }
@@ -689,7 +686,7 @@ static void wxTextWindowActivateProc(Widget w, XtPointer clientData,
     if (tw->InSetValue())
         return;
 
-    wxCommandEvent event(wxEVT_COMMAND_TEXT_ENTER);
+    wxCommandEvent event(wxEVT_TEXT_ENTER);
     event.SetId(tw->GetId());
     event.SetEventObject(tw);
     tw->ProcessCommand(event);
