@@ -120,8 +120,8 @@ class wxDDEModule : public wxModule
 {
 public:
     wxDDEModule() {}
-    bool OnInit() { return true; }
-    void OnExit() { wxDDECleanUp(); }
+    bool OnInit() wxOVERRIDE { return true; }
+    void OnExit() wxOVERRIDE { wxDDECleanUp(); }
 
 private:
     wxDECLARE_DYNAMIC_CLASS(wxDDEModule);
@@ -884,18 +884,20 @@ _DDECallback(WORD wType,
                                                              (wxIPCFormat)wFmt);
                     if (data)
                     {
-                      if (user_size == wxNO_LEN)
-                        switch (wFmt)
+                        if (user_size == wxNO_LEN)
                         {
-                          case wxIPC_TEXT:
-                          case wxIPC_UTF8TEXT:
-                            user_size = strlen((const char*)data) + 1;  // includes final NUL
-                            break;
-                          case wxIPC_UNICODETEXT:
-                            user_size = (wcslen((const wchar_t*)data) + 1) * sizeof(wchar_t);  // includes final NUL
-                            break;
-                          default:
-                            user_size = 0;
+                            switch (wFmt)
+                            {
+                                case wxIPC_TEXT:
+                                case wxIPC_UTF8TEXT:
+                                    user_size = strlen((const char*)data) + 1;  // includes final NUL
+                                    break;
+                                case wxIPC_UNICODETEXT:
+                                    user_size = (wcslen((const wchar_t*)data) + 1) * sizeof(wchar_t);  // includes final NUL
+                                    break;
+                                default:
+                                    user_size = 0;
+                            }
                         }
 
                         HDDEDATA handle = DdeCreateDataHandle(DDEIdInst,
