@@ -1,16 +1,20 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        src/osx/carbon/utils.mm
+// Name:        src/osx/carbon/utilscocoa.mm
 // Purpose:     various cocoa mixin utility functions
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     1998-01-01
-// RCS-ID:      $Id: utilscocoa.mm 48805 2007-09-19 14:52:25Z SC $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 #include "wx/wxprec.h"
 
+#ifndef WX_PRECOMP
+#include "wx/object.h"
+#include "wx/math.h"
+#endif
+
 #if wxOSX_USE_COCOA_OR_CARBON
 #include <Cocoa/Cocoa.h>
 #else
@@ -23,21 +27,8 @@
 
 #include "wx/fontutil.h"
 
-#if wxOSX_USE_COCOA
-#include "wx/cocoa/string.h"
-#endif
-
 #ifdef __WXMAC__
 
-#if wxOSX_USE_CARBON
-bool wxMacInitCocoa()
-{
-    bool cocoaLoaded = NSApplicationLoad();
-    wxASSERT_MSG(cocoaLoaded,wxT("Couldn't load Cocoa in Carbon Environment")) ;
-    return cocoaLoaded;
-}
-#endif
-
 wxMacAutoreleasePool::wxMacAutoreleasePool()
 {
     m_pool = [[NSAutoreleasePool alloc] init];
@@ -108,7 +99,6 @@ void* wxMacCocoaRetain( void* obj )
 #if wxOSX_USE_COCOA
 wxFont::wxFont(WX_NSFont nsfont)
 {
-    [nsfont retain];
     wxNativeFontInfo info;
     SetNativeInfoFromNSFont(nsfont, &info);
     Create(info);
@@ -130,6 +120,7 @@ void wxFont::SetNativeInfoFromNSFont(WX_NSFont theFont, wxNativeFontInfo* info)
         wxFontStyle fontstyle = wxFONTSTYLE_NORMAL;
         wxFontWeight fontweight = wxFONTWEIGHT_NORMAL;
         bool underlined = false;
+        bool strikethrough = false;
 
         int size = (int) ([theFont pointSize]+0.5);
  
@@ -143,195 +134,70 @@ void wxFont::SetNativeInfoFromNSFont(WX_NSFont theFont, wxNativeFontInfo* info)
         if ( theTraits & NSItalicFontMask )
             fontstyle = wxFONTSTYLE_ITALIC ;
 
-        info->Init(size,fontFamily,fontstyle,fontweight,underlined,
-            wxStringWithNSString([theFont familyName]), wxFONTENCODING_DEFAULT);
+        info->Init(size,fontFamily,fontstyle,fontweight,underlined, strikethrough,
+                   wxCFStringRef::AsString([theFont familyName]), wxFONTENCODING_DEFAULT);
 
     }
 }
 
-WX_NSFont wxFont::OSXCreateNSFont(wxOSXSystemFont font, wxNativeFontInfo* info)
+NSFont* wxFont::OSXGetNSFont() const
 {
-    NSFont* nsfont = nil;
-    switch( font )
-    {
-        case wxOSX_SYSTEM_FONT_NORMAL:
-            nsfont = [NSFont systemFontOfSize:[NSFont systemFontSize]];
-            break;
-        case wxOSX_SYSTEM_FONT_BOLD:
-            nsfont = [NSFont boldSystemFontOfSize:[NSFont systemFontSize]];
-            break;
-        case wxOSX_SYSTEM_FONT_SMALL:
-            nsfont = [NSFont systemFontOfSize:[NSFont smallSystemFontSize]];
-            break;
-        case wxOSX_SYSTEM_FONT_SMALL_BOLD:
-            nsfont = [NSFont boldSystemFontOfSize:[NSFont smallSystemFontSize]];
-            break;
-        case wxOSX_SYSTEM_FONT_MINI:
-            nsfont = [NSFont systemFontOfSize:
-                [NSFont systemFontSizeForControlSize:NSMiniControlSize]];
-            break;
-       case wxOSX_SYSTEM_FONT_MINI_BOLD:
-            nsfont = [NSFont boldSystemFontOfSize:
-                [NSFont systemFontSizeForControlSize:NSMiniControlSize]];
-            break;
-        case wxOSX_SYSTEM_FONT_LABELS:
-            nsfont = [NSFont labelFontOfSize:[NSFont labelFontSize]];
-            break;
-       case wxOSX_SYSTEM_FONT_VIEWS:
-            nsfont = [NSFont controlContentFontOfSize:0];
-            break;
-        default:
-            break;
-    }
-    [nsfont retain];
-    SetNativeInfoFromNSFont(nsfont, info);
-    return nsfont;
-}
+    wxCHECK_MSG( m_refData != NULL , 0, wxT("invalid font") );
 
-static inline double DegToRad(double deg) { return (deg * M_PI) / 180.0; }
-static const NSAffineTransformStruct kSlantNSTransformStruct = { 1, 0, tan(DegToRad(11)), 1, 0, 0  };
+    // cast away constness otherwise lazy font resolution is not possible
+    const_cast<wxFont *>(this)->RealizeResource();
 
-WX_NSFont wxFont::OSXCreateNSFont(const wxNativeFontInfo* info)
-{
-    NSFont* nsFont;
-    int weight = 5;
-    NSFontTraitMask traits = 0;
-    if (info->m_weight == wxFONTWEIGHT_BOLD)
-    {
-        traits |= NSBoldFontMask;
-        weight = 9;
-    }
-    else if (info->m_weight == wxFONTWEIGHT_LIGHT)
-        weight = 3;
+    NSFont *font = const_cast<NSFont*>(reinterpret_cast<const NSFont*>(OSXGetCTFont()));
 
-    if (info->m_style == wxFONTSTYLE_ITALIC || info->m_style == wxFONTSTYLE_SLANT)
-        traits |= NSItalicFontMask;
-    
-    nsFont = [[NSFontManager sharedFontManager] fontWithFamily:wxCFStringRef(info->m_faceName).AsNSString() 
-        traits:traits weight:weight size:info->m_pointSize];
-    
-    if ( nsFont == nil )
+#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_12
+    // There's a bug in OS X 10.11 (but not present in 10.10 or 10.12) where a
+    // toll-free bridged font may have an attributed of private class __NSCFCharacterSet
+    // that unlike NSCharacterSet doesn't conform to NSSecureCoding. This poses
+    // a problem when such font is used in user-editable content, because some
+    // Asian input methods then crash in 10.11 when editing the string.
+    // As a workaround for this bug, don't use toll-free bridging, but
+    // re-create NSFont from the descriptor instead on buggy OS X versions.
+    int osMajor, osMinor;
+    wxGetOsVersion(&osMajor, &osMinor, NULL);
+    if (osMajor == 10 && osMinor == 11)
     {
-        NSFontTraitMask remainingTraits = traits;
-        nsFont = [[NSFontManager sharedFontManager] fontWithFamily:wxCFStringRef(info->m_faceName).AsNSString() 
-                                                            traits:0 weight:5 size:info->m_pointSize];
-        if ( nsFont == nil )
-        {
-            if ( info->m_weight == wxFONTWEIGHT_BOLD )
-            {
-                nsFont = [NSFont boldSystemFontOfSize:info->m_pointSize];
-                remainingTraits &= ~NSBoldFontMask;
-            }
-            else
-                nsFont = [NSFont systemFontOfSize:info->m_pointSize];
-        }
-        
-        // fallback - if in doubt, let go of the bold attribute
-        if ( nsFont && (remainingTraits & NSItalicFontMask) )
-        {
-            NSFont* nsFontWithTraits = nil;
-            if ( remainingTraits & NSBoldFontMask)
-            {
-                nsFontWithTraits = [[NSFontManager sharedFontManager] convertFont:nsFont toHaveTrait:NSBoldFontMask];
-                if ( nsFontWithTraits == nil )
-                {
-                    nsFontWithTraits = [[NSFontManager sharedFontManager] convertFont:nsFont toHaveTrait:NSItalicFontMask];
-                    if ( nsFontWithTraits != nil )
-                        remainingTraits &= ~NSItalicFontMask;
-                }
-                else
-                {
-                    remainingTraits &= ~NSBoldFontMask;
-                }
-            }
-            if ( remainingTraits & NSItalicFontMask)
-            {
-                if ( nsFontWithTraits == nil )
-                    nsFontWithTraits = nsFont;
-                
-                NSAffineTransform* transform = [NSAffineTransform transform];
-                [transform setTransformStruct:kSlantNSTransformStruct];
-                [transform scaleBy:info->m_pointSize];
-                NSFontDescriptor* italicDesc = [[nsFontWithTraits fontDescriptor] fontDescriptorWithMatrix:transform];
-                if ( italicDesc != nil )
-                {
-                    NSFont* f = [NSFont fontWithDescriptor:italicDesc size:(CGFloat)(info->m_pointSize)];
-                    if ( f != nil )
-                        nsFontWithTraits = f;
-                }
-            }
-            if ( nsFontWithTraits != nil )
-                nsFont = nsFontWithTraits;
-        }
+        return [NSFont fontWithDescriptor:[font fontDescriptor] size:[font pointSize]];
     }
-            
-    wxASSERT_MSG(nsFont != nil,wxT("Couldn't create nsFont")) ;
-    wxMacCocoaRetain(nsFont);
-    return nsFont;
+#endif // MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_12
+
+    return font;
 }
 
 #endif
 
 #if wxOSX_USE_IPHONE
 
-WX_UIFont wxFont::OSXCreateUIFont(wxOSXSystemFont font, wxNativeFontInfo* info)
+UIFont* wxFont::OSXGetUIFont() const
 {
-    UIFont* uifont;
-    switch( font )
-    {
-        case wxOSX_SYSTEM_FONT_NORMAL:
-            uifont = [UIFont systemFontOfSize:[UIFont systemFontSize]];
-            break;
-        case wxOSX_SYSTEM_FONT_BOLD:
-            uifont = [UIFont boldSystemFontOfSize:[UIFont systemFontSize]];
-            break;
-        case wxOSX_SYSTEM_FONT_MINI:
-        case wxOSX_SYSTEM_FONT_SMALL:
-            uifont = [UIFont systemFontOfSize:[UIFont smallSystemFontSize]];
-            break;
-        case wxOSX_SYSTEM_FONT_MINI_BOLD:
-        case wxOSX_SYSTEM_FONT_SMALL_BOLD:
-            uifont = [UIFont boldSystemFontOfSize:[UIFont smallSystemFontSize]];
-            break;
-        case wxOSX_SYSTEM_FONT_VIEWS:
-        case wxOSX_SYSTEM_FONT_LABELS:
-            uifont = [UIFont systemFontOfSize:[UIFont labelFontSize]];
-            break;
-        default:
-            break;
-    }
-    [uifont retain];
-    if ( info->m_faceName.empty())
-    {
-        wxFontStyle fontstyle = wxFONTSTYLE_NORMAL;
-        wxFontWeight fontweight = wxFONTWEIGHT_NORMAL;
-        bool underlined = false;
+    wxCHECK_MSG( m_refData != NULL , 0, wxT("invalid font") );
 
-        int size = (int) ([uifont pointSize]+0.5);
-        /*
-        NSFontSymbolicTraits traits = [desc symbolicTraits];
+    // cast away constness otherwise lazy font resolution is not possible
+    const_cast<wxFont *>(this)->RealizeResource();
 
-        if ( traits & NSFontBoldTrait )
-            fontweight = wxFONTWEIGHT_BOLD ;
-        else
-            fontweight = wxFONTWEIGHT_NORMAL ;
-        if ( traits & NSFontItalicTrait )
-            fontstyle = wxFONTSTYLE_ITALIC ;
-        */
-        wxCFStringRef fontname( wxCFRetain([uifont familyName]) );
-        info->Init(size,wxFONTFAMILY_DEFAULT,fontstyle,fontweight,underlined,
-            fontname.AsString(), wxFONTENCODING_DEFAULT);
-
-    }
-    return uifont;
+    return const_cast<UIFont*>(reinterpret_cast<const UIFont*>(OSXGetCTFont()));
 }
 
-WX_UIFont wxFont::OSXCreateUIFont(const wxNativeFontInfo* info)
+#endif
+
+// ----------------------------------------------------------------------------
+// NSWindow Utils
+// ----------------------------------------------------------------------------
+
+#if wxOSX_USE_COCOA
+
+WXWindow wxOSXGetMainWindow()
 {
-    UIFont* uiFont;
-    uiFont = [UIFont fontWithName:wxCFStringRef(info->m_faceName).AsNSString() size:info->m_pointSize];
-    wxMacCocoaRetain(uiFont);
-    return uiFont;
+    return [NSApp mainWindow];
+}
+
+WXWindow wxOSXGetKeyWindow()
+{
+    return [NSApp keyWindow];
 }
 
 #endif
@@ -343,61 +209,132 @@ WX_UIFont wxFont::OSXCreateUIFont(const wxNativeFontInfo* info)
 
 WX_UIImage  wxOSXGetUIImageFromCGImage( CGImageRef image )
 {
-    UIImage  *newImage = [UIImage imageWithCGImage:image];
-    [newImage autorelease];
-    return( newImage );
+    return  [UIImage imageWithCGImage:image];
+}
+
+wxBitmap wxOSXCreateSystemBitmap(const wxString& name, const wxString &client, const wxSize& size)
+{
+#if 1
+    // unfortunately this only accesses images in the app bundle, not the system wide globals
+    wxCFStringRef cfname(name);
+    return wxBitmap( [[UIImage imageNamed:cfname.AsNSString()] CGImage] );
+#else
+    return wxBitmap();
+#endif
+}
+
+double wxOSXGetMainScreenContentScaleFactor()
+{
+    double scale;
+  
+#if __IPHONE_OS_VERSION_MAX_ALLOWED >= 40000
+    if ([[UIScreen mainScreen] respondsToSelector:@selector(scale)])
+    {
+        scale=[[UIScreen mainScreen] scale];
+    }
+    else
+#endif
+    {
+        scale=1.0;
+    }
+    
+    return scale;
 }
 
 #endif
 
 #if wxOSX_USE_COCOA
 
-//  From "Cocoa Drawing Guide:Working with Images"
-WX_NSImage  wxOSXGetNSImageFromCGImage( CGImageRef image )
+wxBitmap wxOSXCreateSystemBitmap(const wxString& name, const wxString &WXUNUSED(client), const wxSize& WXUNUSED(size))
+{
+    wxCFStringRef cfname(name);
+    return wxBitmap( [NSImage imageNamed:cfname.AsNSString()] );
+}
+
+WX_NSImage  wxOSXGetNSImageFromCGImage( CGImageRef image, double scaleFactor, bool isTemplate )
 {
     NSRect      imageRect    = NSMakeRect(0.0, 0.0, 0.0, 0.0);
 
     // Get the image dimensions.
-    imageRect.size.height = CGImageGetHeight(image);
-    imageRect.size.width = CGImageGetWidth(image);
+    imageRect.size.height = CGImageGetHeight(image)/scaleFactor;
+    imageRect.size.width = CGImageGetWidth(image)/scaleFactor;
 
-    // Create a new image to receive the Quartz image data.
-    NSImage  *newImage = [[NSImage alloc] initWithSize:imageRect.size];
-    [newImage lockFocus];
+    NSImage* newImage = [[NSImage alloc] initWithCGImage:image size:imageRect.size];
+    
+    [newImage setTemplate:isTemplate];
 
-    // Get the Quartz context and draw.
-    CGContextRef  imageContext = (CGContextRef) [[NSGraphicsContext currentContext] graphicsPort];
-    CGContextDrawImage( imageContext, *(CGRect*)&imageRect, image );
-    [newImage unlockFocus];
-
-    /*
-        // Create a bitmap rep from the image...
-        NSBitmapImageRep *bitmapRep = [[NSBitmapImageRep alloc] initWithCGImage:cgImage];
-        // Create an NSImage and add the bitmap rep to it...
-        NSImage *image = [[NSImage alloc] init];
-        [image addRepresentation:bitmapRep];
-        [bitmapRep release];
-    */
     [newImage autorelease];
     return( newImage );
 }
 
-CGImageRef wxOSXCreateCGImageFromNSImage( WX_NSImage nsimage )
+WX_NSImage WXDLLIMPEXP_CORE wxOSXGetNSImageFromIconRef( WXHICON iconref )
+{
+    NSImage  *newImage = [[NSImage alloc] initWithIconRef:iconref];
+    [newImage autorelease];
+    return( newImage );
+}
+
+CGImageRef WXDLLIMPEXP_CORE wxOSXGetCGImageFromNSImage( WX_NSImage nsimage, CGRect* r, CGContextRef cg)
+{
+    NSRect nsRect = NSRectFromCGRect(*r);
+    return [nsimage CGImageForProposedRect:&nsRect
+                               context:[NSGraphicsContext graphicsContextWithGraphicsPort:cg flipped:YES]
+                                        hints:nil];
+}
+
+CGContextRef WXDLLIMPEXP_CORE wxOSXCreateBitmapContextFromNSImage( WX_NSImage nsimage, bool *isTemplate)
 {
     // based on http://www.mail-archive.com/cocoa-dev@lists.apple.com/msg18065.html
     
-    NSSize imageSize = [nsimage size];
-    CGColorSpaceRef genericRGB = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB); 
-    CGContextRef context = CGBitmapContextCreate(NULL, imageSize.width, imageSize.height, 8, 0, genericRGB, kCGImageAlphaPremultipliedFirst); 
-    NSGraphicsContext *nsGraphicsContext = [NSGraphicsContext graphicsContextWithGraphicsPort:context flipped:NO];
-    [NSGraphicsContext saveGraphicsState];
-    [NSGraphicsContext setCurrentContext:nsGraphicsContext];
-    [[NSColor whiteColor] setFill];
-    NSRectFill(NSMakeRect(0.0, 0.0, imageSize.width, imageSize.height));
-    [nsimage drawAtPoint:NSZeroPoint fromRect:NSZeroRect operation:NSCompositeCopy fraction:1.0];
-    [NSGraphicsContext setCurrentContext:nsGraphicsContext];
-    CGImageRef image = CGBitmapContextCreateImage(context);
-    CFRelease(context);
+    CGContextRef hbitmap = NULL;
+    if (nsimage != nil)
+    {
+        double scale = wxOSXGetMainScreenContentScaleFactor();
+        
+        NSSize imageSize = [nsimage size];
+        
+        hbitmap = CGBitmapContextCreate(NULL, imageSize.width*scale, imageSize.height*scale, 8, 0, wxMacGetGenericRGBColorSpace(), kCGImageAlphaPremultipliedFirst);
+        CGContextScaleCTM( hbitmap, scale, scale );
+    
+        NSGraphicsContext *previousContext = [NSGraphicsContext currentContext];
+        NSGraphicsContext *nsGraphicsContext = [NSGraphicsContext graphicsContextWithGraphicsPort:hbitmap flipped:NO];
+        [NSGraphicsContext saveGraphicsState];
+        [NSGraphicsContext setCurrentContext:nsGraphicsContext];
+        [[NSColor whiteColor] setFill];
+        NSRectFill(NSMakeRect(0.0, 0.0, imageSize.width, imageSize.height));
+        [nsimage drawAtPoint:NSZeroPoint fromRect:NSZeroRect operation:NSCompositeCopy fraction:1.0];
+        [NSGraphicsContext setCurrentContext:previousContext];
+
+        if (isTemplate)
+            *isTemplate = [nsimage isTemplate];
+    }
+    return hbitmap;
+}
+
+double wxOSXGetMainScreenContentScaleFactor()
+{
+    return [[NSScreen mainScreen] backingScaleFactor];
+}
+
+CGImageRef wxOSXCreateCGImageFromNSImage( WX_NSImage nsimage, double *scaleptr )
+{
+    // based on http://www.mail-archive.com/cocoa-dev@lists.apple.com/msg18065.html
+
+    CGImageRef image = NULL;
+    if (nsimage != nil)
+    {        
+        CGContextRef context = wxOSXCreateBitmapContextFromNSImage(nsimage);
+        if ( scaleptr )
+        {
+            // determine content scale
+            CGRect userrect = CGRectMake(0, 0, 10, 10);
+            CGRect devicerect;
+            devicerect = CGContextConvertRectToDeviceSpace(context, userrect);
+            *scaleptr = devicerect.size.height / userrect.size.height;
+        }
+        image = CGBitmapContextCreateImage(context);
+        CFRelease(context);
+    }
     return image;
  }
 
@@ -489,8 +426,11 @@ WX_NSCursor wxMacCocoaCreateStockCursor( int cursor_type )
 
     case wxCURSOR_WATCH:
     case wxCURSOR_WAIT:
-        // should be displayed by the system when things are running
-        cursor = [[NSCursor arrowCursor] retain];
+        // an arrow should be displayed by the system when things are running
+        // according to the HIG
+        // cursor = [[NSCursor arrowCursor] retain];
+        // but for crossplatform compatibility we display a watch cursor
+        cursor = wxGetStockCursor(kwxCursorWatch);
         break;
 
     case wxCURSOR_IBEAM:
