@@ -49,8 +49,7 @@ class WXDLLEXPORT wxBitmapRefData: public wxGDIRefData
     friend class WXDLLIMPEXP_FWD_CORE wxIcon;
     friend class WXDLLIMPEXP_FWD_CORE wxCursor;
 public:
-    wxBitmapRefData(int width , int height , int depth, double logicalscale);
-    wxBitmapRefData(int width , int height , int depth);
+    wxBitmapRefData(int width , int height , int depth, double logicalscale = 1.0);
     wxBitmapRefData(CGContextRef context);
     wxBitmapRefData(CGImageRef image, double scale);
     wxBitmapRefData();
@@ -106,8 +105,7 @@ public:
 
     int           GetBytesPerRow() const { return m_bytesPerRow; }
     private :
-    bool Create(int width , int height , int depth);
-    bool Create(int width , int height , int depth, double logicalScale);
+    bool Create(int width , int height , int depth, double logicalscale);
     bool Create( CGImageRef image, double scale );
     bool Create( CGContextRef bitmapcontext);
     void Init();
@@ -269,13 +267,7 @@ wxBitmapRefData::wxBitmapRefData()
     Init() ;
 }
 
-wxBitmapRefData::wxBitmapRefData( int w , int h , int d )
-{
-    Init() ;
-    Create( w , h , d ) ;
-}
-
-wxBitmapRefData::wxBitmapRefData(int w , int h , int d, double logicalscale)
+wxBitmapRefData::wxBitmapRefData( int w , int h , int d , double logicalscale)
 {
     Init() ;
     Create( w , h , d, logicalscale ) ;
@@ -370,11 +362,12 @@ bool wxBitmapRefData::Create(CGContextRef context)
     return m_ok ;
 }
 
-bool wxBitmapRefData::Create( int w , int h , int d )
+bool wxBitmapRefData::Create( int w , int h , int d, double logicalscale )
 {
     m_width = wxMax(1, w);
     m_height = wxMax(1, h);
     m_depth = d ;
+    m_scaleFactor = logicalscale;
     m_hBitmap = NULL ;
 
     m_bytesPerRow = GetBestBytesPerRow( m_width * 4 ) ;
@@ -395,12 +388,6 @@ bool wxBitmapRefData::Create( int w , int h , int d )
     return m_ok ;
 }
 
-bool wxBitmapRefData::Create( int w , int h , int d, double logicalScale )
-{
-    m_scaleFactor = logicalScale;
-    return Create(w*logicalScale,h*logicalScale,d);
-}
-
 void wxBitmapRefData::UseAlpha( bool use )
 {
     if ( m_hasAlpha == use )
@@ -495,11 +482,10 @@ IconRef wxBitmapRefData::GetIconRef()
         
         switch (sz)
         {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
             case 1024:
                 dataType = kIconServices1024PixelDataARGB;
                 break;
-#endif
+
             case 512:
                 dataType = kIconServices512PixelDataARGB;
                 break;
@@ -835,20 +821,23 @@ wxBitmapRefData::~wxBitmapRefData()
 
 bool wxBitmap::CopyFromIcon(const wxIcon& icon)
 {
-    bool created = false ;
     int w = icon.GetWidth() ;
     int h = icon.GetHeight() ;
 
-    Create( w , h ) ;
-    if ( !created )
+    if ( Create( w, h ) )
     {
+        // Assume 32 bpp icon has transparency values
+        UseAlpha(icon.GetDepth() == 32);
+
         wxMemoryDC dc ;
         dc.SelectObject( *this ) ;
         dc.DrawIcon( icon , 0 , 0 ) ;
         dc.SelectObject( wxNullBitmap ) ;
+
+        return true;
     }
 
-    return true;
+    return false;
 }
 
 wxBitmap::wxBitmap(const char bits[], int the_width, int the_height, int no_bits)
@@ -974,7 +963,7 @@ IconRef wxBitmap::GetIconRef() const
 IconRef wxBitmap::CreateIconRef() const
 {
     IconRef icon = GetIconRef();
-    verify_noerr( AcquireIconRef(icon) );
+    __Verify_noErr(AcquireIconRef(icon));
     return icon;
 }
 #endif
@@ -1123,7 +1112,7 @@ bool wxBitmap::CreateScaled(int w, int h, int d, double logicalScale)
     if ( d < 0 )
         d = wxDisplayDepth() ;
     
-    m_refData = new wxBitmapRefData( w , h , d, logicalScale );
+    m_refData = new wxBitmapRefData( w*logicalScale , h*logicalScale , d, logicalScale );
     
     return M_BITMAPDATA->IsOk() ;
 }
@@ -1213,10 +1202,12 @@ wxBitmap::wxBitmap(const wxImage& image, int depth, double scale)
     // width and height of the device-dependent bitmap
     int width = image.GetWidth();
     int height = image.GetHeight();
-
+    // we always use 32 bit internally here
+    depth = 32;
+    
     wxBitmapRefData* bitmapRefData;
 
-    m_refData = bitmapRefData = new wxBitmapRefData( width/scale, height/scale, depth, scale) ;
+    m_refData = bitmapRefData = new wxBitmapRefData( width, height, depth, scale) ;
 
     if ( bitmapRefData->IsOk())
     {
@@ -1868,9 +1859,9 @@ void wxBitmap::UngetRawData(wxPixelDataBase& WXUNUSED(dataBase))
     EndRawAccess() ;
 }
 
-void wxBitmap::UseAlpha()
+void wxBitmap::UseAlpha(bool use )
 {
     // remember that we are using alpha channel:
     // we'll need to create a proper mask in UngetRawData()
-    M_BITMAPDATA->UseAlpha( true );
+    M_BITMAPDATA->UseAlpha( use );
 }
