@@ -4,7 +4,6 @@
 // Author:      Jaakko Salli
 // Modified by:
 // Created:     2008-08-24
-// RCS-ID:      $Id$
 // Copyright:   (c) Jaakko Salli
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -40,15 +39,6 @@
 #include "wx/propgrid/property.h"
 #include "wx/propgrid/propgrid.h"
 
-
-const wxChar *wxPGTypeName_long = wxT("long");
-const wxChar *wxPGTypeName_bool = wxT("bool");
-const wxChar *wxPGTypeName_double = wxT("double");
-const wxChar *wxPGTypeName_wxString = wxT("string");
-const wxChar *wxPGTypeName_void = wxT("void*");
-const wxChar *wxPGTypeName_wxArrayString = wxT("arrstring");
-
-
 // ----------------------------------------------------------------------------
 // VariantDatas
 // ----------------------------------------------------------------------------
@@ -66,7 +56,7 @@ wxPGProperty* wxPGPropArgCls::GetPtr( wxPropertyGridInterface* iface ) const
 {
     if ( m_flags == IsProperty )
     {
-        wxASSERT_MSG( m_ptr.property, wxT("invalid property ptr") );
+        wxASSERT_MSG( m_ptr.property, wxS("invalid property ptr") );
         return m_ptr.property;
     }
     else if ( m_flags & IsWxString )
@@ -178,13 +168,13 @@ wxPGProperty* wxPropertyGridInterface::ReplaceProperty( wxPGPropArg id, wxPGProp
     wxPGProperty* replaced = p;
     wxCHECK_MSG( replaced && property,
                  wxNullProperty,
-                 wxT("NULL property") );
+                 wxS("NULL property") );
     wxCHECK_MSG( !replaced->IsCategory(),
                  wxNullProperty,
-                 wxT("cannot replace this type of property") );
+                 wxS("cannot replace this type of property") );
     wxCHECK_MSG( !m_pState->IsInNonCatMode(),
                  wxNullProperty,
-                 wxT("cannot replace properties in alphabetic mode") );
+                 wxS("cannot replace properties in alphabetic mode") );
 
     // Get address to the slot
     wxPGProperty* parent = replaced->GetParent();
@@ -260,7 +250,7 @@ bool wxPropertyGridInterface::EnableProperty( wxPGPropArg id, bool enable )
 
     if ( enable )
     {
-        if ( !(p->m_flags & wxPG_PROP_DISABLED) )
+        if ( !p->HasFlag(wxPG_PROP_DISABLED) )
             return false;
 
         // If active, Set active Editor.
@@ -269,7 +259,7 @@ bool wxPropertyGridInterface::EnableProperty( wxPGPropArg id, bool enable )
     }
     else
     {
-        if ( p->m_flags & wxPG_PROP_DISABLED )
+        if ( p->HasFlag(wxPG_PROP_DISABLED) )
             return false;
 
         // If active, Disable as active Editor.
@@ -277,13 +267,41 @@ bool wxPropertyGridInterface::EnableProperty( wxPGPropArg id, bool enable )
             grid->DoSelectProperty( p, wxPG_SEL_FORCE );
     }
 
-    state->DoEnableProperty(p, enable);
+    p->DoEnable(enable);
 
     RefreshProperty( p );
 
     return true;
 }
 
+void wxPropertyGridInterface::SetPropertyReadOnly( wxPGPropArg id, bool set, int flags)
+{
+    wxPG_PROP_ARG_CALL_PROLOG()
+
+    if ( flags & wxPG_RECURSE )
+    {
+        p->SetFlagRecursively(wxPG_PROP_READONLY, set);
+    }
+    else
+    {
+        // Do nothing if flag is already set as required.
+        if ( set && p->HasFlag(wxPG_PROP_READONLY) )
+            return;
+        if ( !set && !p->HasFlag(wxPG_PROP_READONLY) )
+            return;
+
+        p->ChangeFlag(wxPG_PROP_READONLY, set);
+    }
+
+    wxPropertyGridPageState* state = p->GetParentState();
+    if( state )
+    {
+        // If property is attached to the property grid
+        // then refresh the view.
+        RefreshProperty( p );
+    }
+}
+
 // -----------------------------------------------------------------------
 
 bool wxPropertyGridInterface::ExpandAll( bool doExpand )
@@ -416,7 +434,7 @@ void wxPropertyGridInterface::SetValidationFailureBehavior( int vfbFlags )
 wxPGProperty* wxPropertyGridInterface::GetPropertyByNameA( const wxString& name ) const
 {
     wxPGProperty* p = GetPropertyByName(name);
-    wxASSERT_MSG(p,wxString::Format(wxT("no property with name '%s'"),name.c_str()));
+    wxASSERT_MSG(p,wxString::Format(wxS("no property with name '%s'"),name.c_str()));
     return p;
 }
 
@@ -424,15 +442,7 @@ wxPGProperty* wxPropertyGridInterface::GetPropertyByNameA( const wxString& name
 
 wxPGProperty* wxPropertyGridInterface::GetPropertyByLabel( const wxString& label ) const
 {
-    wxPGVIterator it;
-
-    for ( it = GetVIterator( wxPG_ITERATE_PROPERTIES ); !it.AtEnd(); it.Next() )
-    {
-        if ( it.GetProperty()->GetLabel() == label )
-            return it.GetProperty();
-    }
-
-    return wxNullProperty;
+    return m_pState->BaseGetPropertyByLabel(label, NULL);
 }
 
 // ----------------------------------------------------------------------------
@@ -443,11 +453,16 @@ void wxPropertyGridInterface::DoSetPropertyAttribute( wxPGPropArg id, const wxSt
     wxPG_PROP_ARG_CALL_PROLOG()
 
     p->SetAttribute( name, value );
+    // If property is attached to the property grid
+    // then refresh the view.
+    if( p->GetParentState() )
+    {
+        RefreshProperty( p );
+    }
 
     if ( argFlags & wxPG_RECURSE )
     {
-        unsigned int i;
-        for ( i = 0; i < p->GetChildCount(); i++ )
+        for ( unsigned int i = 0; i < p->GetChildCount(); i++ )
             DoSetPropertyAttribute(p->Item(i), name, value, argFlags);
     }
 }
@@ -488,12 +503,12 @@ void wxPropertyGridInterface::GetPropertiesWithFlag( wxArrayPGProperty* targetAr
 
         if ( !inverse )
         {
-            if ( (property->GetFlags() & flags) == flags )
+            if ( property->HasFlagsExact(flags) )
                 targetArr->push_back((wxPGProperty*)property);
         }
         else
         {
-            if ( (property->GetFlags() & flags) != flags )
+            if ( !property->HasFlagsExact(flags) )
                 targetArr->push_back((wxPGProperty*)property);
         }
     }
@@ -537,8 +552,8 @@ wxPGProperty* wxPropertyGridInterface::GetPropertyByName( const wxString& name )
     if ( p )
         return p;
 
-    // Check if its "Property.SubProperty" format
-    int pos = name.Find(wxT('.'));
+    // Check if it is "Property.SubProperty" format
+    int pos = name.Find(wxS('.'));
     if ( pos <= 0 )
         return NULL;
 
@@ -552,6 +567,15 @@ bool wxPropertyGridInterface::HideProperty( wxPGPropArg id, bool hide, int flags
 {
     wxPG_PROP_ARG_CALL_PROLOG_RETVAL(false)
 
+    // Do nothing if single property is already hidden/visible as requested.
+    if ( !(flags & wxPG_RECURSE) )
+    {
+        if ( hide && p->HasFlag(wxPG_PROP_HIDDEN) )
+            return false;
+        if ( !hide && !p->HasFlag(wxPG_PROP_HIDDEN) )
+            return false;
+    }
+
     wxPropertyGrid* pg = m_pState->GetGrid();
 
     if ( pg == p->GetGrid() )
@@ -613,6 +637,9 @@ void wxPropertyGridInterface::SetPropertyLabel( wxPGPropArg id, const wxString&
 {
     wxPG_PROP_ARG_CALL_PROLOG()
 
+    if ( p->GetLabel() == newproplabel )
+        return;
+
     p->SetLabel( newproplabel );
 
     wxPropertyGridPageState* state = p->GetParentState();
@@ -624,9 +651,19 @@ void wxPropertyGridInterface::SetPropertyLabel( wxPGPropArg id, const wxString&
     if ( pg->GetState() == state )
     {
         if ( pg->HasFlag(wxPG_AUTO_SORT) )
+        {
             pg->Refresh();
+            // If any property is selected it has to
+            // be refreshed in the new location.
+            if ( pg == p->GetGrid() && pg->GetSelectedProperty() )
+            {
+                RefreshProperty(pg->GetSelectedProperty());
+            }
+        }
         else
+        {
             pg->DrawItem( p );
+        }
     }
 }
 
@@ -664,7 +701,16 @@ wxPropertyGridInterface::SetPropertyBackgroundColour( wxPGPropArg id,
 {
     wxPG_PROP_ARG_CALL_PROLOG()
     p->SetBackgroundColour(colour, flags);
-    RefreshProperty(p);
+
+    // Redraw the control
+    wxPropertyGrid* pg = m_pState->GetGrid();
+    if ( pg == p->GetGrid() )
+    {
+        if ( flags & wxPG_RECURSE )
+            pg->DrawItemAndChildren(p);
+        else
+            pg->DrawItem(p);
+    }
 }
 
 // -----------------------------------------------------------------------
@@ -675,16 +721,41 @@ void wxPropertyGridInterface::SetPropertyTextColour( wxPGPropArg id,
 {
     wxPG_PROP_ARG_CALL_PROLOG()
     p->SetTextColour(colour, flags);
-    RefreshProperty(p);
+
+    // Redraw the control
+    wxPropertyGrid* pg = m_pState->GetGrid();
+    if ( pg == p->GetGrid() )
+    {
+        if ( flags & wxPG_RECURSE )
+            pg->DrawItemAndChildren(p);
+        else
+            pg->DrawItem(p);
+    }
 }
 
 // -----------------------------------------------------------------------
 
-void wxPropertyGridInterface::SetPropertyColoursToDefault( wxPGPropArg id )
+#if WXWIN_COMPATIBILITY_3_0
+void wxPropertyGridInterface::SetPropertyColoursToDefault(wxPGPropArg id)
+{
+    SetPropertyColoursToDefault(id, wxPG_DONT_RECURSE);
+}
+#endif // WXWIN_COMPATIBILITY_3_0
+
+void wxPropertyGridInterface::SetPropertyColoursToDefault(wxPGPropArg id, int flags)
 {
     wxPG_PROP_ARG_CALL_PROLOG()
+    p->SetDefaultColours(flags);
 
-    p->m_cells.clear();
+    // Redraw the control
+    wxPropertyGrid* pg = m_pState->GetGrid();
+    if ( pg == p->GetGrid() )
+    {
+        if ( flags & wxPG_RECURSE )
+            pg->DrawItemAndChildren(p);
+        else
+            pg->DrawItem(p);
+    }
 }
 
 // -----------------------------------------------------------------------
@@ -699,7 +770,7 @@ void wxPropertyGridInterface::SetPropertyCell( wxPGPropArg id,
     wxPG_PROP_ARG_CALL_PROLOG()
 
     wxPGCell& cell = p->GetCell(column);
-    if ( text.length() && text != wxPG_LABEL )
+    if ( !text.empty() && text != wxPG_LABEL )
         cell.SetText(text);
     if ( bitmap.IsOk() )
         cell.SetBitmap(bitmap);
@@ -712,14 +783,14 @@ void wxPropertyGridInterface::SetPropertyCell( wxPGPropArg id,
 // -----------------------------------------------------------------------
 // GetPropertyValueAsXXX methods
 
-#define IMPLEMENT_GET_VALUE(T,TRET,BIGNAME,DEFRETVAL) \
+#define IMPLEMENT_GET_VALUE(TRET,PGTypeName,BIGNAME,DEFRETVAL) \
 TRET wxPropertyGridInterface::GetPropertyValueAs##BIGNAME( wxPGPropArg id ) const \
 { \
     wxPG_PROP_ARG_CALL_PROLOG_RETVAL(DEFRETVAL) \
     wxVariant value = p->GetValue(); \
-    if ( wxStrcmp(value.GetType(), wxPGTypeName_##T) != 0 ) \
+    if ( !value.IsType(PGTypeName) ) \
     { \
-        wxPGGetFailed(p,wxPGTypeName_##T); \
+        wxPGGetFailed(p, PGTypeName); \
         return (TRET)DEFRETVAL; \
     } \
     return (TRET)value.Get##BIGNAME(); \
@@ -736,20 +807,20 @@ bool wxPropertyGridInterface::GetPropertyValueAsBool( wxPGPropArg id ) const
 {
     wxPG_PROP_ARG_CALL_PROLOG_RETVAL(false)
     wxVariant value = p->GetValue();
-    if ( wxStrcmp(value.GetType(), wxPGTypeName_bool) == 0 )
+    if ( value.IsType(wxPG_VARIANT_TYPE_BOOL) )
     {
         return value.GetBool();
     }
-    if ( wxStrcmp(value.GetType(), wxPGTypeName_long) == 0 )
+    if ( value.IsType(wxPG_VARIANT_TYPE_LONG) )
     {
         return value.GetLong()?true:false;
     }
-    wxPGGetFailed(p,wxPGTypeName_bool);
+    wxPGGetFailed(p, wxPG_VARIANT_TYPE_BOOL);
     return false;
 }
 
-IMPLEMENT_GET_VALUE(long,long,Long,0)
-IMPLEMENT_GET_VALUE(double,double,Double,0.0)
+IMPLEMENT_GET_VALUE(long,wxPG_VARIANT_TYPE_LONG,Long,0)
+IMPLEMENT_GET_VALUE(double,wxPG_VARIANT_TYPE_DOUBLE,Double,0.0)
 
 bool wxPropertyGridInterface::IsPropertyExpanded( wxPGPropArg id ) const
 {
@@ -771,7 +842,7 @@ bool wxPropertyGridInterface::ChangePropertyValue( wxPGPropArg id, wxVariant new
 void wxPropertyGridInterface::BeginAddChildren( wxPGPropArg id )
 {
     wxPG_PROP_ARG_CALL_PROLOG()
-    wxCHECK_RET( p->HasFlag(wxPG_PROP_AGGREGATE), wxT("only call on properties with fixed children") );
+    wxCHECK_RET( p->HasFlag(wxPG_PROP_AGGREGATE), wxS("only call on properties with fixed children") );
     p->ClearFlag(wxPG_PROP_AGGREGATE);
     p->SetFlag(wxPG_PROP_MISC_PARENT);
 }
@@ -788,7 +859,7 @@ bool wxPropertyGridInterface::EditorValidate()
 void wxPropertyGridInterface::EndAddChildren( wxPGPropArg id )
 {
     wxPG_PROP_ARG_CALL_PROLOG()
-    wxCHECK_RET( p->HasFlag(wxPG_PROP_MISC_PARENT), wxT("only call on properties for which BeginAddChildren was called prior") );
+    wxCHECK_RET( p->HasFlag(wxPG_PROP_MISC_PARENT), wxS("only call on properties for which BeginAddChildren was called prior") );
     p->ClearFlag(wxPG_PROP_MISC_PARENT);
     p->SetFlag(wxPG_PROP_AGGREGATE);
 }
@@ -806,7 +877,7 @@ public:
         m_it.Init( state, flags );
     }
     virtual ~wxPGVIteratorBase_State() { }
-    virtual void Next() { m_it.Next(); }
+    virtual void Next() wxOVERRIDE { m_it.Next(); }
 };
 
 wxPGVIterator wxPropertyGridInterface::GetVIterator( int flags ) const
@@ -932,7 +1003,7 @@ wxString wxPropertyGridInterface::SaveEditableState( int includedStates ) const
     }
 
     // Remove last '|'
-    if ( result.length() )
+    if ( !result.empty() )
         result.RemoveLast();
 
     return result;
@@ -1030,17 +1101,17 @@ bool wxPropertyGridInterface::RestoreEditableState( const wxString& src, int res
                 {
                     if ( restoreStates & SelectionState )
                     {
-                        if ( values.size() > 0 )
+                        if ( !values.empty() )
                         {
                             if ( pageState->IsDisplayed() )
                             {
-                                if ( values[0].length() )
+                                if ( !values[0].empty() )
                                     newSelection = GetPropertyByName(value);
                                 pgSelectionSet = true;
                             }
                             else
                             {
-                                if ( values[0].length() )
+                                if ( !values[0].empty() )
                                     pageState->DoSetSelection(GetPropertyByName(value));
                                 else
                                     pageState->DoClearSelection();
