@@ -18,6 +18,8 @@
 #include <QtWidgets/QApplication>
 #include <QtWidgets/QWidget>
 #include <QtWidgets/QScrollArea>
+#include <QtWidgets/QMenu>
+#include <QtWidgets/QShortcut>
 
 #ifndef WX_PRECOMP
     #include "wx/dcclient.h"
@@ -150,6 +152,7 @@ void wxWindowQt::Init()
 #if wxUSE_ACCEL
     m_qtShortcutHandler = new wxQtShortcutHandler( this );
     m_processingShortcut = false;
+    m_qtShortcuts = NULL;
 #endif
     m_qtWindow = NULL;
     m_qtContainer = NULL;
@@ -185,6 +188,7 @@ wxWindowQt::~wxWindowQt()
 
 #if wxUSE_ACCEL
     m_qtShortcutHandler->deleteLater();
+    delete m_qtShortcuts;
 #endif
 
     // Delete only if the qt widget was created or assigned to this base class
@@ -228,20 +232,18 @@ bool wxWindowQt::Create( wxWindowQt * parent, wxWindowID id, const wxPoint & pos
                 QtSetScrollBar( wxVERTICAL );
         }
         else
-        {
             m_qtWindow = new wxQtWidget( parent, this );
-        }
-
-        GetHandle()->setMouseTracking(true);
     }
 
+    
+    GetHandle()->setMouseTracking(true);
     if ( !wxWindowBase::CreateBase( parent, id, pos, size, style, wxDefaultValidator, name ))
         return false;
 
     parent->AddChild( this );
 
     wxPoint p;
-    if(pos != wxDefaultPosition)
+    if ( pos != wxDefaultPosition )
         p = pos;
 
     DoMoveWindow( p.x, p.y, size.GetWidth(), size.GetHeight() );
@@ -433,6 +435,15 @@ void wxWindowQt::Refresh( bool WXUNUSED( eraseBackground ), const wxRect *rect )
     }
 }
 
+bool wxWindowQt::SetCursor( const wxCursor &cursor )
+{
+    if (!wxWindowBase::SetCursor(cursor))
+        return false;
+
+    GetHandle()->setCursor(cursor.GetHandle());
+    
+    return true;
+}
 
 bool wxWindowQt::SetFont( const wxFont &font )
 {
@@ -498,7 +509,7 @@ wxScrollBar *wxWindowQt::QtGetScrollBar( int orientation ) const
  * passed as parameter */
 wxScrollBar *wxWindowQt::QtSetScrollBar( int orientation, wxScrollBar *scrollBar )
 {
-    QAbstractScrollArea *scrollArea = QtGetScrollBarsContainer();
+    QScrollArea *scrollArea = QtGetScrollBarsContainer();
     wxCHECK_MSG( scrollArea, NULL, "Window without scrolling area" );
 
     // Create a new scrollbar if needed
@@ -522,12 +533,12 @@ wxScrollBar *wxWindowQt::QtSetScrollBar( int orientation, wxScrollBar *scrollBar
     // Let Qt handle layout
     if ( orientation == wxHORIZONTAL )
     {
-        scrollArea->setHorizontalScrollBar( scrollBar->GetHandle() );
+        scrollArea->setHorizontalScrollBar( scrollBar->GetQScrollBar() );
         m_horzScrollBar = scrollBar;
     }
     else
     {
-        scrollArea->setVerticalScrollBar( scrollBar->GetHandle() );
+        scrollArea->setVerticalScrollBar( scrollBar->GetQScrollBar() );
         m_vertScrollBar = scrollBar;
     }
     return scrollBar;
@@ -562,9 +573,8 @@ void wxWindowQt::SetScrollbar( int orientation, int pos, int thumbvisible, int r
 void wxWindowQt::SetScrollPos( int orientation, int pos, bool WXUNUSED( refresh ))
 {
     wxScrollBar *scrollBar = QtGetScrollBar( orientation );
-    wxCHECK_RET( scrollBar, "Invalid scrollbar" );
-
-    scrollBar->SetThumbPosition( pos );
+    if ( scrollBar )
+        scrollBar->SetThumbPosition( pos );
 }
 
 int wxWindowQt::GetScrollPos( int orientation ) const
@@ -731,6 +741,9 @@ void wxWindowQt::SetWindowStyleFlag( long style )
 //        qtFrame->setFrameShadow( QFrame::Plain );
 //    }
 
+    if ( !GetHandle() )
+        return;
+
     Qt::WindowFlags qtFlags = GetHandle()->windowFlags();
 
     if ( HasFlag( wxFRAME_NO_TASKBAR ) )
@@ -926,15 +939,16 @@ void wxWindowQt::SetAcceleratorTable( const wxAcceleratorTable& accel )
     wxWindowBase::SetAcceleratorTable( accel );
 
     // Disable previously set accelerators
-    while ( !m_qtShortcuts.isEmpty() )
-        delete m_qtShortcuts.takeFirst();
+    while ( !m_qtShortcuts->isEmpty() )
+        delete m_qtShortcuts->takeFirst();
 
     // Create new shortcuts (use GetHandle() so all events inside
     // the window are handled, not only in the container subwindow)
+    delete m_qtShortcuts;
     m_qtShortcuts = accel.ConvertShortcutTable( GetHandle() );
 
     // Connect shortcuts to window
-    Q_FOREACH( QShortcut *s, m_qtShortcuts )
+    Q_FOREACH( QShortcut *s, *m_qtShortcuts )
     {
         QObject::connect( s, &QShortcut::activated, m_qtShortcutHandler, &wxQtShortcutHandler::activated );
         QObject::connect( s, &QShortcut::activatedAmbiguously, m_qtShortcutHandler, &wxQtShortcutHandler::activated );
@@ -1095,6 +1109,7 @@ bool wxWindowQt::QtHandlePaintEvent ( QWidget *handler, QPaintEvent *event )
 
                 // send the paint event (wxWindowDC will draw directly):
                 wxPaintEvent paint( GetId() );
+                paint.SetEventObject(this);
                 handled = ProcessWindowEvent(paint);
                 m_updateRegion.Clear();
             }
@@ -1132,6 +1147,8 @@ bool wxWindowQt::QtHandleResizeEvent ( QWidget *WXUNUSED( handler ), QResizeEven
 bool wxWindowQt::QtHandleWheelEvent ( QWidget *WXUNUSED( handler ), QWheelEvent *event )
 {
     wxMouseEvent e( wxEVT_MOUSEWHEEL );
+    e.SetPosition( wxQtConvertPoint( event->pos() ) );
+
     e.m_wheelAxis = ( event->orientation() == Qt::Vertical ) ? wxMOUSE_WHEEL_VERTICAL : wxMOUSE_WHEEL_HORIZONTAL;
     e.m_wheelRotation = event->delta();
     e.m_linesPerAction = 3;
@@ -1146,7 +1163,7 @@ bool wxWindowQt::QtHandleKeyEvent ( QWidget *WXUNUSED( handler ), QKeyEvent *eve
     // qt sends keyup and keydown events for autorepeat, but this is not
     // normal for wx which only sends repeated keydown events
     // discard repeated keyup events
-    if(event->isAutoRepeat() && event->type() == QEvent::KeyRelease)
+    if ( event->isAutoRepeat() && event->type() == QEvent::KeyRelease )
         return true;
 
 #if wxUSE_ACCEL
@@ -1405,12 +1422,7 @@ bool wxWindowQt::QtHandleCloseEvent ( QWidget *handler, QCloseEvent *WXUNUSED( e
     if ( GetHandle() != handler )
         return false;
 
-    int close = Close();
-    if ( close )
-    {
-        Destroy();
-    }
-    return close;
+    return Close();
 }
 
 bool wxWindowQt::QtHandleContextMenuEvent ( QWidget *WXUNUSED( handler ), QContextMenuEvent *event )
@@ -1462,7 +1474,7 @@ QWidget *wxWindowQt::GetHandle() const
     return m_qtWindow;
 }
 
-QAbstractScrollArea *wxWindowQt::QtGetScrollBarsContainer() const
+QScrollArea *wxWindowQt::QtGetScrollBarsContainer() const
 {
     return m_qtContainer;
 }
