@@ -56,16 +56,18 @@
 
 class wxSTCTimer : public wxTimer {
 public:
-    wxSTCTimer(ScintillaWX* swx) {
+    wxSTCTimer(ScintillaWX* swx, ScintillaWX::TickReason reason) {
         m_swx = swx;
+        m_reason = reason;
     }
 
     void Notify() wxOVERRIDE {
-        m_swx->DoTick();
+        m_swx->TickFor(m_reason);
     }
 
 private:
     ScintillaWX* m_swx;
+    ScintillaWX::TickReason m_reason;
 };
 
 
@@ -116,6 +118,7 @@ public:
           m_ct(ct), m_swx(swx), m_cx(wxDefaultCoord), m_cy(wxDefaultCoord)
         {
             SetBackgroundStyle(wxBG_STYLE_CUSTOM);
+            SetName("wxSTCCallTip");
         }
 
     ~wxSTCCallTip() {
@@ -132,7 +135,7 @@ public:
     void OnPaint(wxPaintEvent& WXUNUSED(evt))
     {
         wxAutoBufferedPaintDC dc(this);
-        Surface* surfaceWindow = Surface::Allocate(0);
+        Surface* surfaceWindow = Surface::Allocate(m_swx->technology);
         surfaceWindow->Init(&dc, m_ct->wDraw.GetID());
         m_ct->PaintCT(surfaceWindow);
         surfaceWindow->Release();
@@ -263,10 +266,26 @@ ScintillaWX::ScintillaWX(wxStyledTextCtrl* win) {
 #endif
 #endif // wxHAVE_STC_RECT_FORMAT
 
+    //A timer is needed for each member of TickReason enum except tickPlatform
+    timers[tickCaret] = new wxSTCTimer(this,tickCaret);
+    timers[tickScroll] = new wxSTCTimer(this,tickScroll);
+    timers[tickWiden] = new wxSTCTimer(this,tickWiden);
+    timers[tickDwell] = new wxSTCTimer(this,tickDwell);
+
+    m_surfaceData = NULL;
 }
 
 
 ScintillaWX::~ScintillaWX() {
+    for ( TimersHash::iterator i=timers.begin(); i!=timers.end(); ++i ) {
+        delete i->second;
+    }
+    timers.clear();
+
+    if ( m_surfaceData != NULL ) {
+        delete m_surfaceData;
+    }
+
     Finalise();
 }
 
@@ -306,7 +325,6 @@ void ScintillaWX::Initialise() {
 
 void ScintillaWX::Finalise() {
     ScintillaBase::Finalise();
-    SetTicking(false);
     SetIdle(false);
     DestroySystemCaret();
 }
@@ -357,25 +375,6 @@ bool ScintillaWX::SetIdle(bool on) {
 }
 
 
-void ScintillaWX::SetTicking(bool on) {
-    wxSTCTimer* steTimer;
-    if (timer.ticking != on) {
-        timer.ticking = on;
-        if (timer.ticking) {
-            steTimer = new wxSTCTimer(this);
-            steTimer->Start(timer.tickSize);
-            timer.tickerID = steTimer;
-        } else {
-            steTimer = (wxSTCTimer*)timer.tickerID;
-            steTimer->Stop();
-            delete steTimer;
-            timer.tickerID = 0;
-        }
-    }
-    timer.ticksToWait = caret.period;
-}
-
-
 void ScintillaWX::SetMouseCapture(bool on) {
     if (mouseDownCaptures) {
         if (on && !capturedMouse)
@@ -421,7 +420,7 @@ const int H_SCROLL_STEP = 20;
 bool ScintillaWX::ModifyScrollBars(int nMax, int nPage) {
     bool modified = false;
 
-    int vertEnd = nMax;
+    int vertEnd = nMax+1;
     if (!verticalScrollBarVisible)
         vertEnd = 0;
 
@@ -431,7 +430,7 @@ bool ScintillaWX::ModifyScrollBars(int nMax, int nPage) {
         int  sbThumb  = stc->GetScrollThumb(wxVERTICAL);
         int  sbPos    = stc->GetScrollPos(wxVERTICAL);
         if (sbMax != vertEnd || sbThumb != nPage) {
-            stc->SetScrollbar(wxVERTICAL, sbPos, nPage, vertEnd+1);
+            stc->SetScrollbar(wxVERTICAL, sbPos, nPage, vertEnd);
             modified = true;
         }
     }
@@ -440,7 +439,7 @@ bool ScintillaWX::ModifyScrollBars(int nMax, int nPage) {
         int  sbPage   = stc->m_vScrollBar->GetPageSize();
         int  sbPos    = stc->m_vScrollBar->GetThumbPosition();
         if (sbMax != vertEnd || sbPage != nPage) {
-            stc->m_vScrollBar->SetScrollbar(sbPos, nPage, vertEnd+1, nPage);
+            stc->m_vScrollBar->SetScrollbar(sbPos, nPage, vertEnd, nPage);
             modified = true;
         }
     }
@@ -499,9 +498,10 @@ void ScintillaWX::NotifyParent(SCNotification scn) {
 // a side effect that the AutoComp will also not be destroyed when switching
 // to another window, but I think that is okay.
 void ScintillaWX::CancelModes() {
-    if (! focusEvent)
+    if (! focusEvent) {
         AutoCompleteCancel();
-    ct.CallTipCancel();
+        ct.CallTipCancel();
+    }
     Editor::CancelModes();
 }
 
@@ -544,14 +544,14 @@ void ScintillaWX::Paste() {
         evt.SetString(text);
         stc->GetEventHandler()->ProcessEvent(evt);
 
-        wxWX2MBbuf buf = (wxWX2MBbuf)wx2stc(evt.GetString());
+        const wxCharBuffer buf(wx2stc(evt.GetString()));
 
 #if wxUSE_UNICODE
         // free up the old character buffer in case the text is real big
         data.SetText(wxEmptyString);
         text = wxEmptyString;
 #endif
-        int len = strlen(buf);
+        const size_t len = buf.length();
         SelectionPosition selStart = sel.IsRectangular() ?
             sel.Rectangular().Start() :
             sel.Range(sel.Main()).Start();
@@ -741,6 +741,36 @@ bool ScintillaWX::DestroySystemCaret() {
 #endif
 }
 
+bool ScintillaWX::FineTickerAvailable() {
+    return true;
+}
+
+bool ScintillaWX::FineTickerRunning(TickReason reason) {
+    bool running = false;
+    TimersHash::iterator i = timers.find(reason);
+    wxASSERT_MSG( i != timers.end(), "At least one TickReason is missing a timer.");
+    if ( i != timers.end() ) {
+        running = i->second->IsRunning();
+    }
+    return running;
+}
+
+void ScintillaWX::FineTickerStart(TickReason reason, int millis,
+                                  int WXUNUSED(tolerance)) {
+    TimersHash::iterator i = timers.find(reason);
+    wxASSERT_MSG( i != timers.end(), "At least one TickReason is missing a timer." );
+    if ( i != timers.end() ) {
+        i->second->Start(millis);
+    }
+}
+
+void ScintillaWX::FineTickerCancel(TickReason reason) {
+    TimersHash::iterator i = timers.find(reason);
+    wxASSERT_MSG( i != timers.end(), "At least one TickReason is missing a timer." );
+    if ( i != timers.end() ) {
+        i->second->Stop();
+    }
+}
 
 //----------------------------------------------------------------------
 
@@ -795,11 +825,46 @@ sptr_t ScintillaWX::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam)
       }
 #endif
 
+#if wxUSE_GRAPHICS_DIRECT2D
+        case SCI_SETTECHNOLOGY:
+            if ((wParam == SC_TECHNOLOGY_DEFAULT) || (wParam == SC_TECHNOLOGY_DIRECTWRITE)) {
+                if (technology != static_cast<int>(wParam)) {
+                    SurfaceDataD2D* newSurfaceData(NULL);
+
+                    if (static_cast<int>(wParam) > SC_TECHNOLOGY_DEFAULT) {
+                        newSurfaceData =  new SurfaceDataD2D(this);
+
+                        if (!newSurfaceData->Initialised()) {
+                            // Failed to load Direct2D or DirectWrite so no effect
+                            delete newSurfaceData;
+                            return 0;
+                        }
+                    }
+
+                    technology = static_cast<int>(wParam);
+                    if ( m_surfaceData ) {
+                        delete m_surfaceData;
+                    }
+                    m_surfaceData = newSurfaceData;
+
+                    // Invalidate all cached information including layout.
+                    DropGraphics(true);
+                    InvalidateStyleRedraw();
+                }
+            }
+            break;
+#endif
+
 #ifdef SCI_LEXER
       case SCI_LOADLEXERLIBRARY:
             LexerManager::GetInstance()->Load((const char*)lParam);
             break;
 #endif
+      case SCI_GETDIRECTFUNCTION:
+            return reinterpret_cast<sptr_t>(DirectFunction);
+
+      case SCI_GETDIRECTPOINTER:
+            return reinterpret_cast<sptr_t>(this);
 
       default:
           return ScintillaBase::WndProc(iMessage, wParam, lParam);
@@ -966,7 +1031,6 @@ void ScintillaWX::DoLoseFocus(){
     SetFocusState(false);
     focusEvent = false;
     DestroySystemCaret();
-    SetTicking(false);
 }
 
 void ScintillaWX::DoGainFocus(){
@@ -975,7 +1039,6 @@ void ScintillaWX::DoGainFocus(){
     focusEvent = false;
     DestroySystemCaret();
     CreateSystemCaret();
-    SetTicking(true);
 }
 
 void ScintillaWX::DoSysColourChange() {
@@ -986,6 +1049,15 @@ void ScintillaWX::DoLeftButtonDown(Point pt, unsigned int curTime, bool shift, b
     ButtonDown(pt, curTime, shift, ctrl, alt);
 }
 
+void ScintillaWX::DoRightButtonDown(Point pt, unsigned int curTime, bool shift, bool ctrl, bool alt) {
+    if (!PointInSelection(pt)) {
+        CancelModes();
+        SetEmptySelection(PositionFromLocation(pt));
+    }
+
+    RightButtonDownWithModifiers(pt, curTime, ModifierFlags(shift, ctrl, alt));
+}
+
 void ScintillaWX::DoLeftButtonUp(Point pt, unsigned int curTime, bool ctrl) {
     ButtonUp(pt, curTime, ctrl);
 }
@@ -1013,8 +1085,8 @@ void ScintillaWX::DoMiddleButtonUp(Point pt) {
     if (gotData) {
         wxString   text = wxTextBuffer::Translate(data.GetText(),
                                                   wxConvertEOLMode(pdoc->eolMode));
-        wxWX2MBbuf buf = (wxWX2MBbuf)wx2stc(text);
-        int        len = strlen(buf);
+        const wxCharBuffer buf(wx2stc(text));
+        const size_t len = buf.length();
         int caretMain = sel.MainCaret();
         pdoc->InsertString(caretMain, buf, len);
         SetEmptySelection(caretMain + len);
@@ -1037,8 +1109,8 @@ void ScintillaWX::DoAddChar(int key) {
     wxChar wszChars[2];
     wszChars[0] = (wxChar)key;
     wszChars[1] = 0;
-    wxWX2MBbuf buf = (wxWX2MBbuf)wx2stc(wszChars);
-    AddCharUTF((char*)buf.data(), strlen(buf));
+    const wxCharBuffer buf(wx2stc(wszChars));
+    AddCharUTF(buf, buf.length());
 #else
     AddChar((char)key);
 #endif
@@ -1129,13 +1201,27 @@ void ScintillaWX::DoCommand(int ID) {
 }
 
 
-void ScintillaWX::DoContextMenu(Point pt) {
-    if (displayPopupMenu)
+bool ScintillaWX::DoContextMenu(Point pt) {
+    if (ShouldDisplayPopup(pt))
+    {
+        // To prevent generating EVT_MOUSE_CAPTURE_LOST.
+        if ( HaveMouseCapture() ) {
+            SetMouseCapture(false);
+        }
         ContextMenu(pt);
+        return true;
+    }
+    return false;
 }
 
 void ScintillaWX::DoOnListBox() {
-    AutoCompleteCompleted();
+    AutoCompleteCompleted(0, SC_AC_COMMAND);
+}
+
+
+void ScintillaWX::DoMouseCaptureLost()
+{
+    capturedMouse = false;
 }
 
 
@@ -1246,6 +1332,11 @@ bool ScintillaWX::GetUseAntiAliasing() {
     return vs.extraFontFlag != 0;
 }
 
+sptr_t ScintillaWX::DirectFunction(
+    ScintillaWX* swx, unsigned int iMessage, uptr_t wParam, sptr_t lParam) {
+    return swx->WndProc(iMessage, wParam, lParam);
+}
+
 //----------------------------------------------------------------------
 //----------------------------------------------------------------------
 
