@@ -11,12 +11,10 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#if wxUSE_MEDIACTRL
+#if wxUSE_MEDIACTRL && wxUSE_GSTREAMER && !wxUSE_GSTREAMER_PLAYER
 
 #include "wx/mediactrl.h"
 
-#if wxUSE_GSTREAMER
-
 #include <gst/gst.h>                // main gstreamer header
 
 #if GST_CHECK_VERSION(1,0,0)
@@ -1589,10 +1587,13 @@ double wxGStreamerMediaBackend::GetVolume()
     return dVolume;
 }
 
-#endif //wxUSE_GSTREAMER
-
 // Force link into main library so this backend can be loaded
 #include "wx/html/forcelnk.h"
 FORCE_LINK_ME(basewxmediabackends)
 
-#endif //wxUSE_MEDIACTRL
+#elif defined( __VMS )
+// Mediactrl is presently not working on OpenVMS, but at least we need this
+// to link some tests (TO BE FIXED)
+# include "wx/html/forcelnk.h"
+FORCE_LINK_ME(basewxmediabackends)
+#endif // wxUSE_MEDIACTRL && wxUSE_GSTREAMER && !wxUSE_GSTREAMER_PLAYER
