@@ -49,6 +49,10 @@ wxObject *wxSearchCtrlXmlHandler::DoCreateResource()
 
     SetupWindow(ctrl);
 
+    const wxString& hint = GetText(wxS("hint"));
+    if ( !hint.empty() )
+        ctrl->SetDescriptiveText(hint);
+
     return ctrl;
 }
 
