@@ -3,7 +3,6 @@
 // Purpose:     wxList unit test
 // Author:      Vadim Zeitlin, Mattia Barbon
 // Created:     2004-12-08
-// RCS-ID:      $Id$
 // Copyright:   (c) 2004 Vadim Zeitlin, Mattia Barbon
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -43,13 +42,13 @@ private:
     void wxStdListTest();
     void wxListCtorTest();
 
-    DECLARE_NO_COPY_CLASS(ListsTestCase)
+    wxDECLARE_NO_COPY_CLASS(ListsTestCase);
 };
 
 // register in the unnamed registry so that these tests are run by default
 CPPUNIT_TEST_SUITE_REGISTRATION( ListsTestCase );
 
-// also include in it's own registry so that these tests can be run alone
+// also include in its own registry so that these tests can be run alone
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ListsTestCase, "ListsTestCase" );
 
 class Baz // Foo is already taken in the hash test
@@ -133,12 +132,13 @@ void ListsTestCase::wxStdListTest()
         CPPUNIT_ASSERT( *rit == i + &i );
     }
 
-    CPPUNIT_ASSERT( *list1.rbegin() == *--list1.end() &&
-                    *list1.begin() == *--list1.rend() );
-    CPPUNIT_ASSERT( *list1.begin() == *--++list1.begin() &&
-                    *list1.rbegin() == *--++list1.rbegin() );
+    CPPUNIT_ASSERT( *list1.rbegin() == *--list1.end() );
+    CPPUNIT_ASSERT( *list1.begin() == *--list1.rend() );
+    CPPUNIT_ASSERT( *list1.begin() == *--++list1.begin() );
+    CPPUNIT_ASSERT( *list1.rbegin() == *--++list1.rbegin() );
 
-    CPPUNIT_ASSERT( list1.front() == &i && list1.back() == &i + 4 );
+    CPPUNIT_ASSERT( list1.front() == &i );
+    CPPUNIT_ASSERT( list1.back() == &i + 4 );
 
     list1.erase(list1.begin());
     list1.erase(--list1.end());
