@@ -3,7 +3,6 @@
 // Purpose:     wxTextFile unit test
 // Author:      Vadim Zeitlin
 // Created:     2006-03-31
-// RCS-ID:      $Id$
 // Copyright:   (c) 2006 Vadim Zeitlin
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -36,7 +35,10 @@
 class TextFileTestCase : public CppUnit::TestCase
 {
 public:
-    TextFileTestCase() { }
+    TextFileTestCase()
+    {
+        srand((unsigned)time(NULL));
+    }
 
     virtual void tearDown() { unlink(GetTestFileName()); }
 
@@ -44,9 +46,14 @@ private:
     CPPUNIT_TEST_SUITE( TextFileTestCase );
         CPPUNIT_TEST( ReadEmpty );
         CPPUNIT_TEST( ReadDOS );
+        CPPUNIT_TEST( ReadDOSLast );
         CPPUNIT_TEST( ReadUnix );
+        CPPUNIT_TEST( ReadUnixLast );
         CPPUNIT_TEST( ReadMac );
+        CPPUNIT_TEST( ReadMacLast );
         CPPUNIT_TEST( ReadMixed );
+        CPPUNIT_TEST( ReadMixedWithFuzzing );
+        CPPUNIT_TEST( ReadCRCRLF );
 #if wxUSE_UNICODE
         CPPUNIT_TEST( ReadUTF8 );
         CPPUNIT_TEST( ReadUTF16 );
@@ -56,9 +63,14 @@ private:
 
     void ReadEmpty();
     void ReadDOS();
+    void ReadDOSLast();
     void ReadUnix();
+    void ReadUnixLast();
     void ReadMac();
+    void ReadMacLast();
     void ReadMixed();
+    void ReadMixedWithFuzzing();
+    void ReadCRCRLF();
 #if wxUSE_UNICODE
     void ReadUTF8();
     void ReadUTF16();
@@ -79,13 +91,13 @@ private:
     static void CreateTestFile(size_t len, const char *contents);
 
 
-    DECLARE_NO_COPY_CLASS(TextFileTestCase)
+    wxDECLARE_NO_COPY_CLASS(TextFileTestCase);
 };
 
 // register in the unnamed registry so that these tests are run by default
 CPPUNIT_TEST_SUITE_REGISTRATION( TextFileTestCase );
 
-// also include in it's own registry so that these tests can be run alone
+// also include in its own registry so that these tests can be run alone
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TextFileTestCase, "TextFileTestCase" );
 
 void TextFileTestCase::CreateTestFile(size_t len, const char *contents)
@@ -105,6 +117,9 @@ void TextFileTestCase::ReadEmpty()
     CPPUNIT_ASSERT( f.Open(wxString::FromAscii(GetTestFileName())) );
 
     CPPUNIT_ASSERT_EQUAL( (size_t)0, f.GetLineCount() );
+    CPPUNIT_ASSERT( f.Eof() );
+    CPPUNIT_ASSERT_EQUAL( "", f.GetFirstLine() );
+    CPPUNIT_ASSERT_EQUAL( "", f.GetLastLine() );
 }
 
 void TextFileTestCase::ReadDOS()
@@ -121,6 +136,18 @@ void TextFileTestCase::ReadDOS()
     CPPUNIT_ASSERT_EQUAL( wxString(wxT("baz")), f.GetLastLine() );
 }
 
+void TextFileTestCase::ReadDOSLast()
+{
+    CreateTestFile("foo\r\n");
+
+    wxTextFile f;
+    CPPUNIT_ASSERT( f.Open(GetTestFileName()) );
+
+    CPPUNIT_ASSERT_EQUAL( 1, f.GetLineCount() );
+    CPPUNIT_ASSERT_EQUAL( wxTextFileType_Dos, f.GetLineType(0) );
+    CPPUNIT_ASSERT_EQUAL( "foo", f.GetFirstLine() );
+}
+
 void TextFileTestCase::ReadUnix()
 {
     CreateTestFile("foo\nbar\nbaz");
@@ -135,20 +162,48 @@ void TextFileTestCase::ReadUnix()
     CPPUNIT_ASSERT_EQUAL( wxString(wxT("baz")), f.GetLastLine() );
 }
 
+void TextFileTestCase::ReadUnixLast()
+{
+    CreateTestFile("foo\n");
+
+    wxTextFile f;
+    CPPUNIT_ASSERT( f.Open(GetTestFileName()) );
+
+    CPPUNIT_ASSERT_EQUAL( 1, f.GetLineCount() );
+    CPPUNIT_ASSERT_EQUAL( wxTextFileType_Unix, f.GetLineType(0) );
+    CPPUNIT_ASSERT_EQUAL( "foo", f.GetFirstLine() );
+}
+
 void TextFileTestCase::ReadMac()
 {
-    CreateTestFile("foo\rbar\rbaz");
+    CreateTestFile("foo\rbar\r\rbaz");
 
     wxTextFile f;
     CPPUNIT_ASSERT( f.Open(wxString::FromAscii(GetTestFileName())) );
 
-    CPPUNIT_ASSERT_EQUAL( (size_t)3, f.GetLineCount() );
+    CPPUNIT_ASSERT_EQUAL( (size_t)4, f.GetLineCount() );
     CPPUNIT_ASSERT_EQUAL( wxTextFileType_Mac, f.GetLineType(0) );
-    CPPUNIT_ASSERT_EQUAL( wxTextFileType_None, f.GetLineType(2) );
+    CPPUNIT_ASSERT_EQUAL( wxTextFileType_Mac, f.GetLineType(1) );
+    CPPUNIT_ASSERT_EQUAL( wxTextFileType_Mac, f.GetLineType(2) );
+    CPPUNIT_ASSERT_EQUAL( wxTextFileType_None, f.GetLineType(3) );
+    CPPUNIT_ASSERT_EQUAL( wxString(wxT("foo")), f.GetLine(0) );
     CPPUNIT_ASSERT_EQUAL( wxString(wxT("bar")), f.GetLine(1) );
+    CPPUNIT_ASSERT_EQUAL( wxString(wxT("")), f.GetLine(2) );
     CPPUNIT_ASSERT_EQUAL( wxString(wxT("baz")), f.GetLastLine() );
 }
 
+void TextFileTestCase::ReadMacLast()
+{
+    CreateTestFile("foo\r");
+
+    wxTextFile f;
+    CPPUNIT_ASSERT( f.Open(GetTestFileName()) );
+
+    CPPUNIT_ASSERT_EQUAL( 1, f.GetLineCount() );
+    CPPUNIT_ASSERT_EQUAL( wxTextFileType_Mac, f.GetLineType(0) );
+    CPPUNIT_ASSERT_EQUAL( "foo", f.GetFirstLine() );
+}
+
 void TextFileTestCase::ReadMixed()
 {
     CreateTestFile("foo\rbar\r\nbaz\n");
@@ -165,6 +220,58 @@ void TextFileTestCase::ReadMixed()
     CPPUNIT_ASSERT_EQUAL( wxString(wxT("baz")), f.GetLastLine() );
 }
 
+void TextFileTestCase::ReadMixedWithFuzzing()
+{
+    for ( int iteration = 0; iteration < 100; iteration++)
+    {
+        // Create a random buffer with lots of newlines. This is intended to catch
+        // bad parsing in unexpected situations such as the one from ReadCRCRLF()
+        // (which is so common it deserves a test of its own).
+        static const char CHOICES[] = {'\r', '\n', 'X'};
+
+        const size_t BUF_LEN = 100;
+        char data[BUF_LEN + 1];
+        data[0] = 'X';
+        data[BUF_LEN] = '\0';
+        unsigned linesCnt = 0;
+        for ( size_t i = 1; i < BUF_LEN; i++ )
+        {
+            char ch = CHOICES[rand() % WXSIZEOF(CHOICES)];
+            data[i] = ch;
+            if ( ch == '\r' || (ch == '\n' && data[i-1] != '\r') )
+                linesCnt++;
+        }
+        if (data[BUF_LEN-1] != '\r' && data[BUF_LEN-1] != '\n')
+            linesCnt++; // last line was unterminated
+
+        CreateTestFile(data);
+
+        wxTextFile f;
+        CPPUNIT_ASSERT( f.Open(wxString::FromAscii(GetTestFileName())) );
+        CPPUNIT_ASSERT_EQUAL( (size_t)linesCnt, f.GetLineCount() );
+    }
+}
+
+void TextFileTestCase::ReadCRCRLF()
+{
+    // Notepad may create files with CRCRLF line endings (see
+    // https://stackoverflow.com/questions/6998506/text-file-with-0d-0d-0a-line-breaks).
+    // Older versions of wx would loose all data when reading such files.
+    // Test that the data are read, but don't worry about empty lines in between or
+    // line endings. Also include a longer streak of CRs, because they can
+    // happen as well.
+    CreateTestFile("foo\r\r\nbar\r\r\r\nbaz\r\r\n");
+
+    wxTextFile f;
+    CPPUNIT_ASSERT( f.Open(wxString::FromAscii(GetTestFileName())) );
+
+    wxString all;
+    for ( wxString str = f.GetFirstLine(); !f.Eof(); str = f.GetNextLine() )
+        all += str;
+
+    CPPUNIT_ASSERT_EQUAL( "foobarbaz", all );
+}
+
 #if wxUSE_UNICODE
 
 void TextFileTestCase::ReadUTF8()
@@ -231,5 +338,29 @@ void TextFileTestCase::ReadBig()
                           f[NUM_LINES - 1] );
 }
 
-#endif // wxUSE_TEXTFILE
+#ifdef __LINUX__
 
+// Check if using wxTextFile with special files, whose reported size doesn't
+// correspond to the real amount of data in them, works.
+TEST_CASE("wxTextFile::Special", "[textfile][linux][special-file]")
+{
+    SECTION("/proc")
+    {
+        wxTextFile f;
+        CHECK( f.Open("/proc/diskstats") );
+        CHECK( f.GetLineCount() > 1 );
+    }
+
+    SECTION("/sys")
+    {
+        wxTextFile f;
+        CHECK( f.Open("/sys/power/state") );
+        REQUIRE( f.GetLineCount() == 1 );
+        INFO( "/sys/power/state contains \"" << f[0] << "\"" );
+        CHECK( f[0].find("mem") != wxString::npos );
+    }
+}
+
+#endif // __LINUX__
+
+#endif // wxUSE_TEXTFILE
